@@ -80,13 +80,21 @@
 struct ipadb_mspac;
 
 enum ipadb_user_auth {
-  IPADB_USER_AUTH_EMPTY    = 0,
+  IPADB_USER_AUTH_NONE     = 0,
   IPADB_USER_AUTH_DISABLED = 1 << 0,
   IPADB_USER_AUTH_PASSWORD = 1 << 1,
   IPADB_USER_AUTH_RADIUS   = 1 << 2,
   IPADB_USER_AUTH_OTP      = 1 << 3,
 };
 
+struct ipadb_global_config {
+	time_t last_update;
+	bool disable_last_success;
+	bool disable_lockout;
+	char **authz_data;
+	enum ipadb_user_auth user_auth;
+};
+
 struct ipadb_context {
     char *uri;
     char *base;
@@ -99,10 +107,9 @@ struct ipadb_context {
     krb5_key_salt_tuple *supp_encs;
     int n_supp_encs;
     struct ipadb_mspac *mspac;
-    bool disable_last_success;
-    bool disable_lockout;
-    char **authz_data;
-    enum ipadb_user_auth user_auth;
+
+    /* Don't access this directly, use ipadb_get_global_config(). */
+    struct ipadb_global_config config;
 };
 
 #define IPA_E_DATA_MAGIC 0x0eda7a
@@ -117,6 +124,7 @@ struct ipadb_e_data {
     struct ipapwd_policy *pol;
     time_t last_admin_unlock;
     char **authz_data;
+    bool has_tktpolaux;
 };
 
 struct ipadb_context *ipadb_get_context(krb5_context kcontext);
@@ -174,10 +182,17 @@ krb5_error_code ipadb_put_principal(krb5_context kcontext,
                                     char **db_args);
 krb5_error_code ipadb_delete_principal(krb5_context kcontext,
                                        krb5_const_principal search_for);
+#if KRB5_KDB_API_VERSION < 8
 krb5_error_code ipadb_iterate(krb5_context kcontext,
                               char *match_entry,
                               int (*func)(krb5_pointer, krb5_db_entry *),
                               krb5_pointer func_arg);
+#else
+krb5_error_code ipadb_iterate(krb5_context kcontext,
+                              char *match_entry,
+                              int (*func)(krb5_pointer, krb5_db_entry *),
+                              krb5_pointer func_arg, krb5_flags iterflags);
+#endif
 
 /* POLICY FUNCTIONS */
 
@@ -274,5 +289,7 @@ void ipadb_audit_as_req(krb5_context kcontext,
                         krb5_error_code error_code);
 
 /* AUTH METHODS */
-void ipadb_get_user_auth(LDAP *lcontext, LDAPMessage *le,
-                         enum ipadb_user_auth *user_auth);
+void ipadb_parse_user_auth(LDAP *lcontext, LDAPMessage *le,
+                           enum ipadb_user_auth *user_auth);
+const struct ipadb_global_config *
+ipadb_get_global_config(struct ipadb_context *ipactx);
