@@ -66,6 +66,7 @@ static char *std_principal_attrs[] = {
     "passwordHistory",
     IPA_KRB_AUTHZ_DATA_ATTR,
     IPA_USER_AUTH_TYPE,
+    "ipatokenRadiusConfigLink",
 
     "objectClass",
     NULL
@@ -224,6 +225,129 @@ static int ipadb_ldap_attr_to_key_data(LDAP *lcontext, LDAPMessage *le,
     return ret;
 }
 
+static void ipadb_validate_otp(struct ipadb_context *ipactx,
+                               LDAPMessage *lentry,
+                               enum ipadb_user_auth *ua)
+{
+    static const char *attrs[] = { "dn", NULL };
+    static const char *dttmpl = "%Y%m%d%H%M%SZ";
+    static const char *ftmpl = "(&"
+        "(objectClass=ipaToken)(ipatokenOwner=%s)"
+        "(|(ipatokenNotBefore<=%s)(!(ipatokenNotBefore=*)))"
+        "(|(ipatokenNotAfter>=%s)(!(ipatokenNotAfter=*)))"
+        "(|(ipatokenDisabled=FALSE)(!(ipatokenDisabled=*)))"
+    ")";
+    krb5_error_code kerr = 0;
+    LDAPMessage *res = NULL;
+    char datetime[16] = {};
+    char *filter = NULL;
+    struct tm tm = {};
+    char *dn = NULL;
+    time_t now = 0;
+    int count = 0;
+
+    if (!(*ua & IPADB_USER_AUTH_OTP))
+        return;
+
+    /* Get the current time. */
+    if (time(&now) == (time_t) -1)
+        return;
+    if (gmtime_r(&now, &tm) == NULL)
+        return;
+
+    /* Make the current time string. */
+    if (strftime(datetime, sizeof(datetime), dttmpl, &tm) == 0)
+        return;
+
+    /* Make the filter. */
+    dn = ldap_get_dn(ipactx->lcontext, lentry);
+    if (dn == NULL)
+        return;
+    count = asprintf(&filter, ftmpl, dn, datetime, datetime);
+    ldap_memfree(dn);
+    if (count < 0)
+        return;
+
+    /* Fetch the active token list. */
+    kerr = ipadb_simple_search(ipactx, ipactx->base, LDAP_SCOPE_SUBTREE,
+                               filter, (char**) attrs, &res);
+    free(filter);
+    if (kerr != 0 || res == NULL)
+        return;
+
+    /* Count the number of active tokens. */
+    count = ldap_count_entries(ipactx->lcontext, res);
+    ldap_msgfree(res);
+
+    /* If the user is configured for OTP, but has no active tokens, remove
+     * OTP from the list since the user obviously can't log in this way. */
+    if (count == 0)
+        *ua &= ~IPADB_USER_AUTH_OTP;
+}
+
+static void ipadb_validate_radius(struct ipadb_context *ipactx,
+                                  LDAPMessage *lentry,
+                                  enum ipadb_user_auth *ua)
+{
+    struct berval **vals;
+
+    if (!(*ua & IPADB_USER_AUTH_RADIUS))
+        return;
+
+    /* Ensure that the user has a link to a RADIUS config. */
+    vals = ldap_get_values_len(ipactx->lcontext, lentry,
+                               "ipatokenRadiusConfigLink");
+    if (vals == NULL || vals[0] == NULL)
+        *ua &= ~IPADB_USER_AUTH_RADIUS;
+
+    if (vals != NULL)
+        ldap_value_free_len(vals);
+}
+
+static void ipadb_validate_password(struct ipadb_context *ipactx,
+                                    LDAPMessage *lentry,
+                                    enum ipadb_user_auth *ua)
+{
+    /* If no mechanisms are set, use password. */
+    if (*ua == IPADB_USER_AUTH_NONE)
+        *ua |= IPADB_USER_AUTH_PASSWORD;
+
+    /* If any other mechanism has passed validation, don't use password. */
+    else if (*ua & ~IPADB_USER_AUTH_PASSWORD)
+        *ua &= ~IPADB_USER_AUTH_PASSWORD;
+}
+
+static enum ipadb_user_auth ipadb_get_user_auth(struct ipadb_context *ipactx,
+                                                LDAPMessage *lentry)
+{
+    enum ipadb_user_auth gua = IPADB_USER_AUTH_NONE;
+    enum ipadb_user_auth ua = IPADB_USER_AUTH_NONE;
+    const struct ipadb_global_config *gcfg = NULL;
+
+    /* Get the user's user_auth settings. */
+    ipadb_parse_user_auth(ipactx->lcontext, lentry, &ua);
+
+    /* Get the global user_auth settings. */
+    gcfg = ipadb_get_global_config(ipactx);
+    if (gcfg != NULL)
+        gua = gcfg->user_auth;
+
+    /* If the disabled flag is set, ignore everything else. */
+    if ((ua | gua) & IPADB_USER_AUTH_DISABLED)
+        return IPADB_USER_AUTH_DISABLED;
+
+    /* Determine which user_auth policy is active: user or global. */
+    if (ua == IPADB_USER_AUTH_NONE)
+        ua = gua;
+
+    /* Perform flag validation. */
+    ipadb_validate_otp(ipactx, lentry, &ua);
+    ipadb_validate_radius(ipactx, lentry, &ua);
+    ipadb_validate_password(ipactx, lentry, &ua);
+
+    return ua;
+}
+
 static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
                                               char *principal,
                                               LDAPMessage *lentry,
@@ -231,9 +355,8 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
                                               uint32_t *polmask)
 {
     krb5_octet otp_string[] = {'o', 't', 'p', 0, '[', ']', 0 };
-    enum ipadb_user_auth user_ua = IPADB_USER_AUTH_EMPTY;
-    enum ipadb_user_auth *active_ua = &user_ua;
     struct ipadb_context *ipactx;
+    enum ipadb_user_auth ua;
     LDAP *lcontext;
     krb5_db_entry *entry;
     struct ipadb_e_data *ied;
@@ -267,16 +390,8 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
     entry->magic = KRB5_KDB_MAGIC_NUMBER;
     entry->len = KRB5_KDB_V1_BASE_LENGTH;
 
-    /* Get the user's user_auth settings. */
-    ipadb_get_user_auth(ipactx->lcontext, lentry, &user_ua);
-
-    /* TODO: Should we confirm the existence of ipatokenRadiusConfigLink in
-     *       the case of RADIUS? Existence of a token for OTP? */
-
-    /* Determine which user_auth policy is active: user or global. */
-    if ((ipactx->user_auth & IPADB_USER_AUTH_DISABLED)
-        || user_ua == IPADB_USER_AUTH_EMPTY)
-        active_ua = &ipactx->user_auth;
+    /* Get User Auth configuration. */
+    ua = ipadb_get_user_auth(ipactx, lentry);
 
     /* ignore mask for now */
 
@@ -321,6 +436,10 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
     switch (ret) {
     case 0:
         entry->pw_expiration = restime;
+
+        /* If we are using only RADIUS, we don't know expiration. */
+        if (ua == IPADB_USER_AUTH_RADIUS)
+            entry->pw_expiration = 0;
     case ENOENT:
         break;
     default:
@@ -410,8 +529,7 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
     switch (ret) {
     case 0:
         /* Only set a principal's key if password auth should be used. */
-        if ((*active_ua & ~IPADB_USER_AUTH_DISABLED) != IPADB_USER_AUTH_EMPTY
-            && !(*active_ua & IPADB_USER_AUTH_PASSWORD)) {
+        if (!(ua & IPADB_USER_AUTH_PASSWORD)) {
             /* This is the same behavior as ENOENT below. */
             ipa_krb5_free_key_data(res_key_data, result);
             break;
@@ -468,6 +586,17 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
         ied->ipa_user = true;
     }
 
+    /* check if it has the krbTicketPolicyAux objectclass */
+    ret = ipadb_ldap_attr_has_value(lcontext, lentry,
+                                    "objectClass", "krbTicketPolicyAux");
+    if (ret != 0 && ret != ENOENT) {
+        kerr = ret;
+        goto done;
+    }
+    if (ret == 0) {
+        ied->has_tktpolaux = true;
+    }
+
     ret = ipadb_ldap_attr_to_str(lcontext, lentry,
                                  "krbPwdPolicyReference", &restring);
     switch (ret) {
@@ -540,10 +669,11 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
     }
 
     /* If enabled, set the otp user string, enabling otp. */
-    if ((*active_ua & (IPADB_USER_AUTH_RADIUS | IPADB_USER_AUTH_OTP)) &&
-        !(*active_ua & IPADB_USER_AUTH_DISABLED)) {
-        ret = ipadb_set_tl_data(entry, KRB5_TL_STRING_ATTRS,
-                                sizeof(otp_string), otp_string);
+    if (ua & (IPADB_USER_AUTH_RADIUS | IPADB_USER_AUTH_OTP)) {
+        kerr = ipadb_set_tl_data(entry, KRB5_TL_STRING_ATTRS,
+                                 sizeof(otp_string), otp_string);
+        if (kerr)
+            goto done;
     }
 
     kerr = 0;
@@ -653,6 +783,9 @@ static krb5_error_code ipadb_find_principal(krb5_context kcontext,
             } else {
                 found = (strcmp(vals[i]->bv_val, (*principal)) == 0);
             }
+            if (found) {
+                break;
+            }
         }
 
         ldap_value_free_len(vals);
@@ -669,11 +802,7 @@ static krb5_error_code ipadb_find_principal(krb5_context kcontext,
 
         /* Again, if aliases are accepted by KDC, use case-insensitive comparison */
         if ((flags & KRB5_KDB_FLAG_ALIAS_OK) != 0) {
-            if (ulc_casecmp(vals[0]->bv_val, vals[0]->bv_len,
-                            (*principal), strlen(*principal),
-                            NULL, NULL, &result) != 0)
-                return KRB5_KDB_INTERNAL_ERROR;
-            found = (result == 0);
+            found = true;
         } else {
             found = (strcmp(vals[0]->bv_val, (*principal)) == 0);
         }
@@ -1266,8 +1395,26 @@ static krb5_error_code ipadb_get_ldap_mod_key_data(struct ipadb_mods *imods,
 {
     krb5_error_code kerr;
     struct berval *bval = NULL;
+    LDAPMod *mod;
     int ret;
 
+    /* If the key data is empty, remove all keys. */
+    if (n_key_data == 0 || key_data == NULL) {
+        kerr = ipadb_mods_new(imods, &mod);
+        if (kerr != 0)
+            return kerr;
+
+        mod->mod_op = LDAP_MOD_DELETE;
+        mod->mod_bvalues = NULL;
+        mod->mod_type = strdup("krbPrincipalKey");
+        if (mod->mod_type == NULL) {
+            ipadb_mods_free_tip(imods);
+            return ENOMEM;
+        }
+
+        return 0;
+    }
+
     ret = ber_encode_krb5_key_data(key_data, n_key_data, mkvno, &bval);
     if (ret != 0) {
         kerr = ret;
@@ -1327,10 +1474,38 @@ done:
     return kerr;
 }
 
+static krb5_error_code ipadb_principal_to_mods(krb5_context kcontext,
+                                               struct ipadb_mods *imods,
+                                               char *principal,
+                                               int mod_op)
+{
+    krb5_error_code kerr;
+
+    if (principal == NULL) {
+       kerr = EINVAL;
+       goto done;
+    }
+
+    kerr = ipadb_get_ldap_mod_str(imods, "krbPrincipalName",
+                                  principal, mod_op);
+    if (kerr) {
+        goto done;
+    }
+    kerr = ipadb_get_ldap_mod_str(imods, "ipaKrbPrincipalAlias",
+                                  principal, mod_op);
+    if (kerr) {
+        goto done;
+    }
+
+    kerr = 0;
+
+done:
+    return kerr;
+}
+
 static krb5_error_code ipadb_entry_to_mods(krb5_context kcontext,
                                            struct ipadb_mods *imods,
                                            krb5_db_entry *entry,
-                                           char *principal,
                                            int mod_op)
 {
     krb5_error_code kerr;
@@ -1339,20 +1514,6 @@ static krb5_error_code ipadb_entry_to_mods(krb5_context kcontext,
 
     /* check each mask flag in order */
 
-    /* KADM5_PRINCIPAL */
-    if (entry->mask & KMASK_PRINCIPAL) {
-        kerr = ipadb_get_ldap_mod_str(imods, "krbPrincipalName",
-                                      principal, mod_op);
-        if (kerr) {
-            goto done;
-        }
-        kerr = ipadb_get_ldap_mod_str(imods, "ipaKrbPrincipalAlias",
-                                      principal, mod_op);
-        if (kerr) {
-            goto done;
-        }
-    }
-
     /* KADM5_PRINC_EXPIRE_TIME */
     if (entry->mask & KMASK_PRINC_EXPIRE_TIME) {
         kerr = ipadb_get_ldap_mod_time(imods,
@@ -1411,6 +1572,29 @@ static krb5_error_code ipadb_entry_to_mods(krb5_context kcontext,
 
     /* KADM5_ATTRIBUTES */
     if (entry->mask & KMASK_ATTRIBUTES) {
+        /* if the object does not have the krbTicketPolicyAux class
+         * we need to add it or this will fail, only for modifications.
+         * We always add this objectclass by default when doing an add
+         * from scratch. */
+        if ((mod_op == LDAP_MOD_REPLACE) && entry->e_data) {
+            struct ipadb_e_data *ied;
+
+            ied = (struct ipadb_e_data *)entry->e_data;
+            if (ied->magic != IPA_E_DATA_MAGIC) {
+                kerr = EINVAL;
+                goto done;
+            }
+
+            if (!ied->has_tktpolaux) {
+                kerr = ipadb_get_ldap_mod_str(imods, "objectclass",
+                                              "krbTicketPolicyAux",
+                                              LDAP_MOD_ADD);
+                if (kerr) {
+                    goto done;
+                }
+            }
+        }
+
         kerr = ipadb_get_ldap_mod_int(imods,
                                       "krbTicketFlags",
                                       (int)entry->attributes,
@@ -1693,8 +1877,12 @@ static krb5_error_code ipadb_add_principal(krb5_context kcontext,
         goto done;
     }
 
-    kerr = ipadb_entry_to_mods(kcontext, imods,
-                               entry, principal, LDAP_MOD_ADD);
+    kerr = ipadb_principal_to_mods(kcontext, imods, principal, LDAP_MOD_ADD);
+    if (kerr != 0) {
+        goto done;
+    }
+
+    kerr = ipadb_entry_to_mods(kcontext, imods, entry, LDAP_MOD_ADD);
     if (kerr != 0) {
         goto done;
     }
@@ -1725,6 +1913,11 @@ static krb5_error_code ipadb_modify_principal(krb5_context kcontext,
         return KRB5_KDB_DBNOTINITED;
     }
 
+    kerr = new_ipadb_mods(&imods);
+    if (kerr) {
+        goto done;
+    }
+
     ied = (struct ipadb_e_data *)entry->e_data;
     if (!ied || !ied->entry_dn) {
         kerr = krb5_unparse_name(kcontext, entry->princ, &principal);
@@ -1749,15 +1942,16 @@ static krb5_error_code ipadb_modify_principal(krb5_context kcontext,
             kerr = KRB5_KDB_INTERNAL_ERROR;
             goto done;
         }
+
+        kerr = ipadb_principal_to_mods(kcontext, imods, principal,
+                                       LDAP_MOD_REPLACE);
+        if (kerr != 0) {
+            goto done;
+        }
+
     }
 
-    kerr = new_ipadb_mods(&imods);
-    if (kerr) {
-        goto done;
-    }
-
-    kerr = ipadb_entry_to_mods(kcontext, imods,
-                               entry, principal, LDAP_MOD_REPLACE);
+    kerr = ipadb_entry_to_mods(kcontext, imods, entry, LDAP_MOD_REPLACE);
     if (kerr != 0) {
         goto done;
     }
@@ -1917,10 +2111,17 @@ done:
     return kerr;
 }
 
+#if KRB5_KDB_API_VERSION < 8
 krb5_error_code ipadb_iterate(krb5_context kcontext,
                               char *match_entry,
                               int (*func)(krb5_pointer, krb5_db_entry *),
                               krb5_pointer func_arg)
+#else
+krb5_error_code ipadb_iterate(krb5_context kcontext,
+                              char *match_entry,
+                              int (*func)(krb5_pointer, krb5_db_entry *),
+                              krb5_pointer func_arg, krb5_flags iterflags)
+#endif
 {
     struct ipadb_context *ipactx;
     krb5_error_code kerr;
