@@ -41,17 +41,17 @@
 #  include <config.h>
 #endif
 
+#include "../libotp/otp_token.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <stdbool.h>
 
 #include <prio.h>
 #include <ssl.h>
-#include <dirsrv/slapi-plugin.h>
 #include <krb5.h>
 #include <kdb.h>
 #include <lber.h>
@@ -76,30 +76,6 @@
 #define IPA_CHANGETYPE_ADMIN 1
 #define IPA_CHANGETYPE_DSMGR 2
 
-/*
- * Attribute type defines
- */
-#define IPA_USER_AUTH_TYPE           "ipaUserAuthType"
-#define IPA_OTP_TOKEN_OWNER_TYPE     "ipaTokenOwner"
-#define IPA_OTP_TOKEN_LENGTH_TYPE    "ipaTokenOTPDigits"
-#define IPA_OTP_TOKEN_KEY_TYPE       "ipaTokenOTPKey"
-#define IPA_OTP_TOKEN_ALGORITHM_TYPE "ipaTokenOTPAlgorithm"
-#define IPA_OTP_TOKEN_OFFSET_TYPE    "ipaTokenTOTPClockOffset"
-#define IPA_OTP_TOKEN_STEP_TYPE      "ipaTokenTOTPTimeStep"
-
-/* Authentication type defines */
-#define IPA_OTP_AUTH_TYPE_NONE     0
-#define IPA_OTP_AUTH_TYPE_DISABLED 1
-#define IPA_OTP_AUTH_TYPE_PASSWORD 2
-#define IPA_OTP_AUTH_TYPE_OTP      4
-#define IPA_OTP_AUTH_TYPE_PKINIT   8
-#define IPA_OTP_AUTH_TYPE_RADIUS   16
-#define IPA_OTP_AUTH_TYPE_VALUE_DISABLED "DISABLED"
-#define IPA_OTP_AUTH_TYPE_VALUE_PASSWORD "PASSWORD"
-#define IPA_OTP_AUTH_TYPE_VALUE_OTP      "OTP"
-#define IPA_OTP_AUTH_TYPE_VALUE_PKINIT   "PKINIT"
-#define IPA_OTP_AUTH_TYPE_VALUE_RADIUS   "RADIUS"
-
 struct ipapwd_data {
     Slapi_Entry *target;
     char *dn;
@@ -135,9 +111,6 @@ struct ipapwd_krbcfg {
     bool allow_nt_hash;
 };
 
-bool ipapwd_is_auth_type_allowed(char **auth_type_list, int auth_type);
-bool ipapwd_parse_otp_config_entry(Slapi_Entry * e, bool apply);
-bool ipapwd_otp_is_disabled(void);
 int ipapwd_entry_checks(Slapi_PBlock *pb, struct slapi_entry *e,
                         int *is_root, int *is_krb, int *is_smb, int *is_ipant,
                         char *attr, int access);
@@ -168,6 +141,12 @@ struct ipapwd_keyset {
 
 void ipapwd_keyset_free(struct ipapwd_keyset **pkset);
 
+Slapi_Value **ipapwd_encrypt_encode_key(struct ipapwd_krbcfg *krbcfg,
+                                        struct ipapwd_data *data,
+                                        int num_encsalts,
+                                        krb5_key_salt_tuple *encsalts,
+                                        char **errMesg);
+
 int ipapwd_gen_hashes(struct ipapwd_krbcfg *krbcfg,
                       struct ipapwd_data *data, char *userpw,
                       int is_krb, int is_smb, int is_ipant,
@@ -184,9 +163,3 @@ int ipapwd_post_init_betxn(Slapi_PBlock *pb);
 
 /* from ipa_pwd_extop.c */
 void *ipapwd_get_plugin_id(void);
-Slapi_DN *ipapwd_get_otp_config_area(void);
-Slapi_DN *ipapwd_get_plugin_sdn(void);
-bool ipapwd_get_plugin_started(void);
-
-/* from auth.c */
-bool ipapwd_do_otp_auth(Slapi_Entry *bind_entry, struct berval *creds);
