@@ -23,13 +23,14 @@ Password migration script
 import cgi
 import errno
 import glob
-import wsgiref
+from wsgiref.util import request_uri
 
 from ipapython.ipa_log_manager import root_logger
 from ipapython.ipautil import get_ipa_basedn
 from ipapython.dn import DN
 from ipapython.ipaldap import IPAdmin
 from ipalib import errors
+from ipaplatform.paths import paths
 
 
 def wsgi_redirect(start_response, loc):
@@ -37,7 +38,7 @@ def wsgi_redirect(start_response, loc):
     return []
 
 def get_ui_url(environ):
-    full_url = wsgiref.util.request_uri(environ)
+    full_url = request_uri(environ)
     index = full_url.rfind(environ.get('SCRIPT_NAME',''))
     if index == -1:
         raise ValueError('Cannot strip the script URL from full URL "%s"' % full_url)
@@ -89,7 +90,7 @@ def application(environ, start_response):
     if not form_data.has_key('username') or not form_data.has_key('password'):
         return wsgi_redirect(start_response, 'invalid.html')
 
-    slapd_sockets = glob.glob('/var/run/slapd-*.socket')
+    slapd_sockets = glob.glob(paths.ALL_SLAPD_INSTANCE_SOCKETS)
     if slapd_sockets:
         ldap_uri = 'ldapi://%s' % slapd_sockets[0].replace('/', '%2f')
     else:
