@@ -18,12 +18,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/**
- * Application controller
- *
- * Controls interaction between navigation, menu and facets.
- */
-
 define([
         'dojo/_base/declare',
         'dojo/_base/lang',
@@ -33,24 +27,35 @@ define([
         'dojo/topic',
         'dojo/query',
         'dojo/dom-class',
-       './json2',
+        './auth',
+        './json2',
         './widgets/App',
+        './widgets/FacetContainer',
         './ipa',
-       './navigation/Menu',
+        './reg',
+        './navigation/Menu',
         './navigation/Router',
-        './navigation/menu_spec'
+        './navigation/routing',
+        './navigation/menu_spec',
+        './plugins/load_page'
        ],
-       function(declare, lang, array, Deferred, on, topic, query, dom_class,
-            JSON, App_widget, IPA, Menu, Router, menu_spec) {
+       function(declare, lang, array, Deferred, on, topic, query, dom_class, auth,
+            JSON, App_widget, FacetContainer, IPA, reg, Menu, Router, routing, menu_spec) {
 
     /**
-     * Main application
+     * Application controller
      *
-     * This class serves as top level widget. It creates basic UI: controls
-     * rendering of header, footer and placeholder for facets.
+     * Controls interaction between navigation, menu and facets.
+     *
+     * @class Application_controller
      */
     var App = declare(null, {
 
+        /**
+         * Facet container map
+         */
+        containers: null,
+
         app_widget: null,
 
         router: null,
@@ -61,25 +66,68 @@ define([
 
         facet_changing: false,
 
+        /**
+         * Currently displayed facet
+         *
+         */
+        current_facet: null,
+
+        /**
+         * Currently displayed facet container
+         */
+        current_container: null,
+
         init: function() {
             this.menu = new Menu();
             this.router = new Router();
+            routing.init(this.router);
+
+            var body_node = query('body')[0];
             this.app_widget = new App_widget();
+            this.app_widget.container_node = body_node;
             this.app_widget.menu_widget.set_menu(this.menu);
-            this.app_widget.container_node = query('body')[0];
+
+            var simple_container = new FacetContainer();
+            simple_container.container_node = body_node;
+
+            var notification_container = new FacetContainer({
+                container_node: body_node,
+                id: "notification",
+                'class': ''
+            });
+
+            this.containers = {
+                // Default view
+                main: {
+                    widget: this.app_widget
+                },
+                // Mainly for standalone facets
+                simple: {
+                    widget: simple_container
+                }
+            };
 
             on(this.app_widget.menu_widget, 'item-select', lang.hitch(this, this.on_menu_click));
             on(this.app_widget, 'profile-click', lang.hitch(this, this.on_profile));
             on(this.app_widget, 'logout-click', lang.hitch(this, this.on_logout));
-            on(this.menu, 'selected', lang.hitch(this, this.on_menu_select));
+            on(this.app_widget, 'password-reset-click', lang.hitch(this, this.on_password_reset));
+            on(this.app_widget, 'about-click', lang.hitch(this, this.on_about));
 
             on(this.router, 'facet-show', lang.hitch(this, this.on_facet_show));
             on(this.router, 'facet-change', lang.hitch(this, this.on_facet_change));
             on(this.router, 'facet-change-canceled', lang.hitch(this, this.on_facet_canceled));
             on(this.router, 'error', lang.hitch(this, this.on_router_error));
             topic.subscribe('phase-error', lang.hitch(this, this.on_phase_error));
+            topic.subscribe('authenticate', lang.hitch(this, this.on_authenticate));
 
             this.app_widget.render();
+            this.app_widget.hide();
+            simple_container.render();
+            simple_container.hide();
+            notification_container.render();
+            var load_facet = reg.facet.get('load');
+            this.show_facet(load_facet);
+            IPA.opened_dialogs.start_handling(this);
         },
 
         /**
@@ -121,8 +169,12 @@ define([
         start_runtime: function() {
             this.run_time = new Deferred();
 
+            // hide load or login facets
+            this.hide_facet();
+
             IPA.update_password_expiration();
 
+
             // now we are ready for displaying a facet,
             // it can match a facet if hash is set
             this.router.startup();
@@ -139,7 +191,7 @@ define([
             if (IPA.is_selfservice) {
                 this.on_profile();
             } else {
-                this.router.navigate_to_entity_facet('user', 'search');
+                routing.navigate(routing.default_path);
             }
         },
 
@@ -177,31 +229,44 @@ define([
         },
 
         on_profile: function() {
-            this.router.navigate_to_entity_facet('user', 'details', [IPA.whoami.uid[0]]);
+            routing.navigate(['entity', 'user', 'details', [IPA.whoami.uid[0]]]);
         },
 
         on_logout: function(event) {
             this.run_time.resolve();
         },
 
+        on_password_reset: function() {
+            IPA.password_selfservice();
+        },
+
+        on_about: function() {
+            var dialog = IPA.about_dialog();
+            dialog.open();
+        },
+
         on_phase_error: function(error) {
 
-            window.console.error(error);
             error = error || {};
             var name = error.name || 'Runtime error';
             var error_container = $('<div/>', {
-                'class': 'facet-content facet-error'
-            }).appendTo($('.content').empty());
-            error_container.append('<h1>'+name+'</h1>');
+                'class': 'container facet-content facet-error'
+            }).appendTo($('.app-container .content').empty());
+            error_container.append($('<h1/>', { text: name }));
             var details = $('<div/>', {
                 'class': 'error-details'
             }).appendTo(error_container);
 
-            details.append('<p> Web UI got in unrecoverable state during "'+error.phase+'" phase.</p>');
-
+            details.append($('<p/>', { text: 'Web UI got in unrecoverable state during "' + error.phase + '" phase' }));
+            if (error.name) window.console.error(error.name);
             if (error.results) {
-                details.append('<strong>Technical details:</strong>');
-                details.append('<p>'+JSON.stringify(error.results)+'</p>');
+                var msg = error.results.message;
+                var stack = error.results.stack.toString();
+                window.console.error(msg);
+                window.console.error(stack);
+                details.append('<h3>Technical details:</h3>');
+                details.append($('<div/>', { text: error.results.message }));
+                details.append($('<div/>').append($('<code/>', { text: stack })));
             }
         },
 
@@ -210,6 +275,8 @@ define([
             var new_facet = event.facet;
             var current_facet = this.current_facet;
 
+            if (current_facet === new_facet) return;
+
             if (current_facet && !current_facet.can_leave()) {
                 var permit_clb = lang.hitch(this, function() {
                     // Some facet's might not call reset before this call but after
@@ -230,32 +297,68 @@ define([
 
         on_facet_state_changed: function(event) {
             if (event.facet === this.current_facet) {
-                var hash = this.router.create_hash(event.facet, event.state);
-                this.router.update_hash(hash, true);
+                routing.update_hash(event.facet, event.state);
             }
         },
 
         on_facet_show: function(event) {
-            var facet = event.facet;
+            this.show_facet(event.facet);
+        },
+
+        show_facet: function(facet) {
+
+            // prevent changing facet when authenticating
+            if (this.current_facet && this.current_facet.name === 'login' &&
+                !auth.current.authenticated && facet.requires_auth) {
+                return;
+            }
+
+            // choose container
+            var container = this.containers[facet.preferred_container];
+            if (!container) container = this.containers.main;
+
+            if (this.current_container !== container) {
+
+                if (this.current_container) {
+                    this.current_container.widget.hide();
+                }
+
+                this.current_container = container;
+                this.current_container.widget.show();
+            }
 
             // update menu
             var menu_item = this._find_menu_item(facet);
             if (menu_item) this.menu.select(menu_item);
 
-            if (!facet.container) {
-                facet.container_node = this.app_widget.content_node;
+            // show facet
+            if (!facet.container_node) {
+                facet.container_node = container.widget.content_node;
                 on(facet, 'facet-state-change', lang.hitch(this, this.on_facet_state_changed));
             }
+
+            if (this.current_facet !== facet) {
+                IPA.opened_dialogs.hide();
+            }
+
+            this.hide_facet();
+            this.current_facet = facet;
+            facet.show();
+            IPA.opened_dialogs.focus_top();
+        },
+
+        hide_facet: function() {
+
             if (this.current_facet) {
                 this.current_facet.hide();
             }
-            this.current_facet = facet;
-            facet.show();
+            this.current_facet = null;
         },
 
+
         _find_menu_item: function(facet) {
 
-            var items;
+            var items = [];
 
             // entity facets
             if (facet.entity) {
@@ -311,14 +414,15 @@ define([
             if (!child) {
                 if(menu_item.entity) {
                     // entity pages
-                    this.router.navigate_to_entity_facet(
+                    routing.navigate([
+                        'entity',
                         menu_item.entity,
                         menu_item.facet,
                         menu_item.pkeys,
-                        menu_item.args);
+                        menu_item.args]);
                 } else if (menu_item.facet) {
                     // concrete facets
-                    this.router.navigate_to_facet(menu_item.facet, menu_item.args);
+                    routing.navigate(['generic', menu_item.facet, menu_item.args]);
                 } else {
                     // categories, select first posible child, it may be the last
                     var children = this.menu.query({parent: menu_item.name });
@@ -338,29 +442,43 @@ define([
         },
 
         /**
-         * Watches menu changes and adjusts facet space when there is
-         * a need for larger menu space.
-         *
-         * Show extended menu space when:
-         *     * there is 3+ levels of menu
-         *
-         * Don't show when:
-         *     * all items of levels 3+ are hidden
+         * Starts authentication process in authentication UI
+         * @returns {undefined}
          */
-        on_menu_select: function(select_state) {
+        on_authenticate: function() {
 
-            var visible_levels = 0;
-            var levels = select_state.new_selection.length;
-            for (var i=0; i< levels; i++) {
-                var item = select_state.new_selection[i];
-                if(!item.hidden) visible_levels++;
+            var self = this;
+            if (this.auth_ui === 'dialog') {
+                var dummy_command = {
+                    execute: function() {
+                        topic.publish('auth-successful');
+                    }
+                };
+
+                var dialog = IPA.unauthorized_dialog({
+                    close_on_escape: false,
+                    error_thrown: { name: '', message: ''},
+                    command: dummy_command
+                });
+
+                dialog.open();
+            } else {
+                var facet = this.current_facet;
+
+                // we don't want the load facet to be displayed after successful auth
+                if (facet && facet.name === 'load') {
+                    facet = null;
+                }
+                var login_facet = reg.facet.get('login');
+
+                on.once(login_facet, "logged_in", function() {
+                    if (facet) {
+                        self.show_facet(facet);
+                    }
+                    topic.publish('auth-successful');
+                });
+                this.show_facet(login_facet);
             }
-
-            var three_levels = visible_levels >= 3;
-
-            dom_class.toggle(this.app_widget.content_node,
-                             'nav-space-3',
-                             three_levels);
         }
     });
 
