@@ -124,7 +124,7 @@ exp.facet = IPA.facet = function(spec, no_init) {
 
     spec = spec || {};
 
-    var that = new Evented();
+    var that = IPA.object();
 
     /**
      * Name of preferred facet container
@@ -211,7 +211,8 @@ exp.facet = IPA.facet = function(spec, no_init) {
      * Facet header
      * @property {facet.facet_header}
      */
-    that.header = spec.header || IPA.facet_header({ facet: that });
+    that.header = builder.build('',  spec.header || {}, {},
+        { $pre_ops: [{ facet: that }], $factory: IPA.facet_header });
 
     /**
      * Hard override for `needs_update()` logic. When set, `needs_update`
@@ -298,6 +299,15 @@ exp.facet = IPA.facet = function(spec, no_init) {
      */
     that.redirect_info = spec.redirect_info;
 
+    /**
+     * Name of containing facet of containing entity
+     *
+     * A guide for breadcrumb navigation
+     *
+     * @property {string}
+     */
+    that.containing_facet = spec.containing_facet;
+
 
     /**
      * Facet requires authenticated user
@@ -885,12 +895,12 @@ exp.facet = IPA.facet = function(spec, no_init) {
         title = title.replace('${error}', error_thrown.name);
 
         that.error_container.empty();
-        that.error_container.append('<h1>'+title+'</h1>');
+        that.error_container.append($('<h1/>', { text: title }));
 
         var details = $('<div/>', {
             'class': 'error-details'
         }).appendTo(that.error_container);
-        details.append('<p>'+error_thrown.message+'</p>');
+        details.append($('<p/>', { text: error_thrown.message }));
 
         $('<div/>', {
             text: text.get('@i18n:error_report.options')
@@ -922,7 +932,9 @@ exp.facet = IPA.facet = function(spec, no_init) {
             }
         );
 
-        that.error_container.append('<p>'+text.get('@i18n:error_report.problem_persists')+'</p>');
+        that.error_container.append($('<p/>', {
+            text: text.get('@i18n:error_report.problem_persists')
+        }));
 
         that.show_error();
     };
@@ -1118,9 +1130,9 @@ exp.facet_header = IPA.facet_header = function(spec) {
         that.update_breadcrumb(limited_value);
 
         var title_info = {
-            title: that.facet.label,
+            text: that.facet.label,
             pkey: limited_value,
-            pkey_tooltip: value
+            pkey_title: value
         };
         that.title_widget.update(title_info);
     };
@@ -1130,25 +1142,30 @@ exp.facet_header = IPA.facet_header = function(spec) {
         if (!that.breadcrumb) return;
 
         var items = [];
-        var item, i, l;
+        var item, i, l, keys, target_facet, target_facet_keys, containing_entity;
 
         // all pkeys should be available in facet
-        var keys = that.facet.get_pkeys();
-        var entity = that.facet.entity.get_containing_entity();
-        i = keys.length - 2; //set pointer to first containing entity
-        while (entity) {
+        keys = that.facet.get_pkeys();
+
+        target_facet_keys = keys;
+        containing_entity = that.facet.entity.get_containing_entity();
+        target_facet = that.facet;
+
+        while (containing_entity) {
+            target_facet = containing_entity.get_facet(
+                target_facet.containing_facet || 'default');
+            target_facet_keys = target_facet_keys.slice(0, -1);
             items.unshift({
-                text: keys[i],
-                title: entity.metadata.label_singular,
-                handler: function(entity) {
+                text: target_facet_keys.slice(-1),
+                title: containing_entity.metadata.label_singular,
+                handler: function(facet, keys) {
                     return function() {
-                        navigation.show_entity(entity.name, 'default');
+                        navigation.show(facet, keys);
                         return false;
                     };
-                }(entity)
+                }(target_facet, target_facet_keys)
             });
-            entity = entity.get_containing_entity();
-            i--;
+            containing_entity = containing_entity.get_containing_entity();
         }
 
         //calculation of breadcrumb keys length
@@ -1167,12 +1184,12 @@ exp.facet_header = IPA.facet_header = function(spec) {
         }
         max_key_l = ((max_bc_l - bc_l) / to_limit) - 4;
 
+        target_facet = target_facet.get_redirect_facet();
         // main level item
-        var redirect_facet = that.facet.get_redirect_facet();
         items.unshift({
-            text: redirect_facet.label,
+            text: target_facet.label,
             handler: function() {
-                that.facet.redirect();
+                navigation.show(target_facet);
                 return false;
             }
         });
@@ -1199,7 +1216,7 @@ exp.facet_header = IPA.facet_header = function(spec) {
                 click: item.handler
             }).appendTo(bc_item);
         } else {
-            bc_item.append(item.text);
+            bc_item.text(item.text);
         }
         return bc_item;
     };
@@ -1214,7 +1231,6 @@ exp.facet_header = IPA.facet_header = function(spec) {
 
         var li = $('<li/>', {
             name: other_facet.name,
-            title: other_facet.name,
             click: function() {
                 if (li.hasClass('entity-facet-disabled')) {
                     return false;
@@ -1277,7 +1293,7 @@ exp.facet_header = IPA.facet_header = function(spec) {
         }
 
         that.title_widget.create(container);
-        that.title_widget.update({ title: that.facet.label });
+        that.title_widget.update({ text: that.facet.label });
 
         if (!that.facet.disable_facet_tabs) {
             that.facet_tabs = $('<div/>', {
@@ -1349,7 +1365,7 @@ exp.facet_header = IPA.facet_header = function(spec) {
 
     /**
      * Reflect facet's action state summary into title widget class and icon
-     * tooltip.
+     * title.
      */
     that.update_summary = function() {
         var summary = that.facet.action_state.summary();
@@ -1357,7 +1373,7 @@ exp.facet_header = IPA.facet_header = function(spec) {
         if (summary.state.length > 0) {
             var css_class = summary.state.join(' ');
             that.title_widget.set_class(css_class);
-            that.title_widget.set_icon_tooltip(summary.description);
+            that.title_widget.set_icon_title(summary.description);
         }
     };
 
@@ -1387,6 +1403,8 @@ exp.facet_header = IPA.facet_header = function(spec) {
         that.load();
     };
 
+    that.facet_header_set_pkey = that.set_pkey;
+
     return that;
 };
 
@@ -1409,30 +1427,26 @@ exp.facet_title = IPA.facet_title = function(spec) {
      *
      * @param {Object} data
      * @param {string} data.pkey
+     * @param {string} data.text
      * @param {string} data.title
-     * @param {string} data.tooltip
-     * @param {string} data.icon_tooltip
+     * @param {string} data.icon_title
      * @param {string} data.css_class css class for title container
      */
     that.update = function(data) {
 
-        var tooltip = data.tooltip || data.title;
-        var pkey_tooltip = data.pkey_tooltip || data.pkey;
-        var icon_tooltip = data.icon_tooltip || '';
-
-        that.title.text(data.title);
-        that.title.prop('title', tooltip);
+        that.title.text(data.text);
+        that.title.prop('title', data.title || '');
         that.title_container.toggleClass('no-pkey', !data.pkey);
 
         if (data.pkey) {
-            that.title.text(data.title + ': ');
+            that.title.text(data.text + ': ');
             that.pkey.text(data.pkey);
-            that.pkey.attr('title', pkey_tooltip);
+            that.pkey.attr('title', data.pkey_title || data.pkey);
         }
 
         if (data.css_class) that.set_class(data.css_class);
 
-        that.set_icon_tooltip(icon_tooltip);
+        that.set_icon_title(data.icon_title || '');
     };
 
     /**
@@ -1487,12 +1501,12 @@ exp.facet_title = IPA.facet_title = function(spec) {
     };
 
     /**
-     * Set icon tooltip
+     * Set icon title
      *
-     * @param {string} tooltip
+     * @param {string} title
      */
-    that.set_icon_tooltip = function(tooltip) {
-        that.icon.attr('title', tooltip);
+    that.set_icon_title = function(title) {
+        that.icon.attr('title', title);
     };
 
     return that;
@@ -1811,7 +1825,7 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
             function(xhr, text_status, error_thrown) {
                 that.load_records([]);
                 var summary = that.table.summary.empty();
-                summary.append(error_thrown.name+': '+error_thrown.message);
+                summary.text(error_thrown.name+': '+error_thrown.message);
             }
         );
     };
@@ -1956,7 +1970,9 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
 
             var metadata = IPA.get_entity_param(entity.name, column.name);
             column.primary_key = metadata && metadata.primary_key;
-            column.link = (column.link === undefined ? true : column.link) && column.primary_key;
+            if (column.primary_key) {
+                column.link = column.link === undefined ? true : column.link;
+            }
 
             if (column.link && column.primary_key) {
                 column.link_handler = function(value) {
