@@ -112,6 +112,12 @@ return {
                             $type: 'checkbox',
                             acl_param: 'krbticketflags',
                             flags: ['w_if_no_aci']
+                        },
+                        {
+                            name: 'ipaassignedidview',
+                            $type: 'link',
+                            label: '@i18n:objects.idview.ipaassignedidview',
+                            other_entity: 'idview'
                         }
                     ]
                 },
@@ -139,6 +145,90 @@ return {
                             label: '@i18n:objects.host.status'
                         }
                     ]
+                },
+                {
+                    $factory: IPA.section,
+                    name: 'divider',
+                    layout_css_class: 'col-sm-12',
+                    fields: []
+                },
+                {
+                    name: 'read',
+                    label: '@i18n:keytab.allowed_to_retrieve',
+                    $factory: IPA.section,
+                    fields: [
+                        {
+                            $type: 'association_table',
+                            id: 'host_ipaallowedtoperform_read_keys_user',
+                            name: 'ipaallowedtoperform_read_keys_user',
+                            add_method: 'allow_retrieve_keytab',
+                            remove_method: 'disallow_retrieve_keytab',
+                            add_title: '@i18n:keytab.add_retrive',
+                            remove_title: '@i18n:keytab.remove_retrieve',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_read_keys_user',
+                                    label: '@mo:user.label_singular',
+                                    link: true
+                                }
+                            ]
+                        },
+                        {
+                            $type: 'association_table',
+                            id: 'host_ipaallowedtoperform_read_keys_group',
+                            name: 'ipaallowedtoperform_read_keys_group',
+                            add_method: 'allow_retrieve_keytab',
+                            remove_method: 'disallow_retrieve_keytab',
+                            add_title: '@i18n:keytab.add_retrive',
+                            remove_title: '@i18n:keytab.remove_retrieve',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_read_keys_group',
+                                    label: '@mo:group.label_singular',
+                                    link: true
+                                }
+                            ]
+                        }
+                    ]
+                },
+                {
+                    name: 'write',
+                    label: '@i18n:keytab.allowed_to_create',
+                    $factory: IPA.section,
+                    fields: [
+                        {
+                            $type: 'association_table',
+                            id: 'host_ipaallowedtoperform_write_keys_user',
+                            name: 'ipaallowedtoperform_write_keys_user',
+                            add_method: 'allow_create_keytab',
+                            remove_method: 'disallow_create_keytab',
+                            add_title: '@i18n:keytab.add_create',
+                            remove_title: '@i18n:keytab.remove_create',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_write_keys_user',
+                                    label: '@mo:user.label_singular',
+                                    link: true
+                                }
+                            ]
+                        },
+                        {
+                            $type: 'association_table',
+                            id: 'host_ipaallowedtoperform_write_keys_group',
+                            name: 'ipaallowedtoperform_write_keys_group',
+                            add_method: 'allow_create_keytab',
+                            remove_method: 'disallow_create_keytab',
+                            add_title: '@i18n:keytab.add_create',
+                            remove_title: '@i18n:keytab.remove_create',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_write_keys_group',
+                                    label: '@mo:group.label_singular',
+                                    link: true
+                                }
+                            ]
+                        }
+                    ]
                 }
             ],
             actions: [
@@ -531,7 +621,7 @@ IPA.host_dnsrecord_entity_link_widget = function(spec) {
 IPA.force_host_add_checkbox_widget = function(spec) {
     var metadata = IPA.get_command_option('host_add', spec.name);
     spec.label = metadata.label;
-    spec.tooltip = metadata.doc;
+    spec.title = metadata.doc;
     return IPA.checkbox_widget(spec);
 };
 
@@ -599,7 +689,7 @@ IPA.host_keytab_widget = function(spec) {
 
     that.update = function(values) {
         set_status(values[0] ? 'present' : 'missing');
-        that.on_value_changed();
+        that.on_value_changed(values);
     };
 
     that.clear = function() {
@@ -759,7 +849,7 @@ IPA.host_password_widget = function(spec) {
 
     that.update = function(values) {
         set_status(values[0] ? 'present' : 'missing');
-        that.on_value_changed();
+        that.on_value_changed(values);
     };
 
     that.clear = function() {
