@@ -23,7 +23,9 @@
 */
 
 define([
+        'dojo/_base/declare',
         'dojo/Deferred',
+        'dojo/Evented',
         'dojo/keys',
         'dojo/topic',
         './jquery',
@@ -38,8 +40,8 @@ define([
         './text',
         './util',
         'exports'
-    ], function(Deferred, keys, topic, $, JSON, i18n, auth, datetime,
-        metadata_provider, builder, reg, rpc, text, util, exports) {
+    ], function(declare, Deferred, Evented, keys, topic, $, JSON, i18n, auth,
+        datetime, metadata_provider, builder, reg, rpc, text, util, exports) {
 
 /**
  * @class
@@ -337,19 +339,14 @@ var IPA = function () {
         }
     };
 
-    that.obj_cls = function() {};
-    that.obj_cls.prototype.__fw_obj = true;
+    that.obj_cls = declare([Evented]);
 
     return that;
 }();
 
 /**
  * Framework objects created by factories should use this
- * instead of empty object when creating base objects. As an alternative
- * they can just set __fw_obj property.
- *
- * __fw_obj property serves for telling the framework that it's instantiated
- * object and not an object specification (spec).
+ * instead of empty object when creating base objects.
  *
  * @class
  */
@@ -632,8 +629,6 @@ IPA.password_selfservice = function() {
             IPA.update_password_expiration();
         };
         command.execute();
-
-        IPA.notify_success(text.get('@i18n:password.password_change_complete'));
     });
     reset_dialog.open();
 };
@@ -1138,6 +1133,7 @@ IPA.notify = function(message, type, timeout) {
 
     if (typeof message === 'string') {
         message = text.get(message);
+        message = document.createTextNode(message);
     }
 
     var notification_area = $('#notification .notification-area');
@@ -1202,7 +1198,7 @@ IPA.get_succeeded = function(data) {
  */
 IPA.config = {
     default_priority: 500,
-    message_timeout: 3000, // [ms]
+    message_timeout: 5000, // [ms]
     message_timeout_length: 50 // [chars]
 };
 
