@@ -153,7 +153,23 @@ return {
                             $type: 'checkboxes',
                             name: 'ipauserauthtype',
                             flags: ['w_if_no_aci'],
-                            options: ['password', 'radius', 'otp']
+                            options: [
+                                { label: '@i18n:authtype.type_password', value: 'password' },
+                                { label: '@i18n:authtype.type_radius', value: 'radius' },
+                                { label: '@i18n:authtype.type_otp', value: 'otp' }
+                            ],
+                            tooltip: '@i18n:authtype.user_tooltip'
+                        },
+                        {
+                            $type: 'entity_select',
+                            name: 'ipatokenradiusconfiglink',
+                            flags: ['w_if_no_aci'],
+                            other_entity: 'radiusproxy',
+                            other_field: 'cn'
+                        },
+                        {
+                            name: 'ipatokenradiususername',
+                            flags: ['w_if_no_aci']
                         }
                     ]
                 },
@@ -264,7 +280,7 @@ return {
                 }
             ],
             actions: [
-                'select',
+                'add_otptoken',
                 'enable',
                 'disable',
                 'delete',
@@ -283,7 +299,7 @@ return {
                     label: '@i18n:actions.automember_rebuild'
                 }
             ],
-            header_actions: ['reset_password', 'enable', 'disable', 'delete', 'unlock', 'automember_rebuild'],
+            header_actions: ['reset_password', 'enable', 'disable', 'delete', 'unlock', 'add_otptoken', 'automember_rebuild'],
             state: {
                 evaluators: [
                     {
@@ -297,7 +313,8 @@ return {
                         name: 'reset_password_acl_evaluator',
                         adapter: { $type: 'batch', result_index: 0 },
                         attribute: 'userpassword'
-                    }
+                    },
+                    IPA.user.self_service_other_user_evaluator
                 ],
                 summary_conditions: [
                     IPA.enabled_summary_cond,
@@ -509,7 +526,7 @@ IPA.user_password_widget = function(spec) {
         } else {
             that.display_control.text(that.unset_value);
         }
-        that.on_value_changed();
+        that.on_value_changed(values);
     };
 
     that.clear = function() {
@@ -603,6 +620,59 @@ IPA.user.reset_password_action = function(spec) {
     return that;
 };
 
+
+IPA.user.add_otptoken_action = function(spec) {
+
+    spec = spec || {};
+    spec.name = spec.name || 'add_otptoken';
+    spec.label = spec.label || '@i18n:objects.otptoken.add_token';
+    spec.disable_cond = spec.disable_cond || ['self-service-other'];
+
+    var that = IPA.action(spec);
+
+    that.execute_action = function(facet) {
+
+        var otp_e = reg.entity.get('otptoken');
+        var dialog = otp_e.get_dialog('add');
+        dialog.open();
+        if (!IPA.is_selfservice) {
+            var owner = facet.get_pkey();
+            dialog.get_field('ipatokenowner').set_value([owner]);
+        }
+    };
+
+    return that;
+};
+
+IPA.user.self_service_other_user_evaluator = function(spec) {
+
+    spec = spec || {};
+    spec.event = spec.event || 'post_load';
+
+    var that = IPA.state_evaluator(spec);
+    that.name = spec.name || 'self_service_other_user_evaluator';
+    that.param = spec.param || 'uid';
+    that.adapter = builder.build('adapter', spec.adapter || 'adapter', { context: that });
+
+    /**
+     * Evaluates if user is in self-service and viewing himself
+     */
+    that.on_event = function(data) {
+
+        var old_state = that.state;
+        that.state = [];
+
+        var value = that.adapter.load(data);
+        if (IPA.is_selfservice && IPA.whoami.uid[0] !== value[0]) {
+            that.state.push('self-service-other');
+        }
+
+        that.notify_on_change(old_state);
+    };
+
+    return that;
+};
+
 exp.entity_spec = make_spec();
 exp.register = function() {
     var e = reg.entity;
@@ -610,6 +680,7 @@ exp.register = function() {
     var d = reg.dialog;
     e.register({type: 'user', spec: exp.entity_spec});
     a.register('reset_password', IPA.user.reset_password_action);
+    a.register('add_otptoken', IPA.user.add_otptoken_action);
     d.copy('password', 'user_password', {
         factory: IPA.user.password_dialog,
         pre_ops: [IPA.user.password_dialog_pre_op]
