@@ -26,11 +26,10 @@ from dns import resolver, rdatatype
 from dns.exception import DNSException
 from ipalib import errors
 from ipapython import ipaldap
+from ipaplatform.paths import paths
 from ipapython.ipautil import valid_ip, get_ipa_basedn, realm_to_suffix
 from ipapython.dn import DN
 
-CACERT = '/etc/ipa/ca.crt'
-
 NOT_FQDN = -1
 NO_LDAP_SERVER = -2
 REALM_NOT_FOUND = -3
@@ -77,7 +76,7 @@ class IPADiscovery(object):
         domains = []
         domain = None
         try:
-            fp = open('/etc/resolv.conf', 'r')
+            fp = open(paths.RESOLV_CONF, 'r')
             lines = fp.readlines()
             fp.close()
 
@@ -140,7 +139,7 @@ class IPADiscovery(object):
                 domain = domain[p+1:]
         return (None, None)
 
-    def search(self, domain = "", servers = "", hostname=None, ca_cert_path=None):
+    def search(self, domain="", servers="", realm=None, hostname=None, ca_cert_path=None):
         """
         Use DNS discovery to identify valid IPA servers.
 
@@ -219,13 +218,21 @@ class IPADiscovery(object):
 
         #search for kerberos
         root_logger.debug("[Kerberos realm search]")
-        krb_realm, kdc = self.ipadnssearchkrb(self.domain)
-        if not servers and not krb_realm:
+        if realm:
+            root_logger.debug("Kerberos realm forced")
+            self.realm = realm
+            self.realm_source = 'Forced'
+        else:
+            realm = self.ipadnssearchkrbrealm()
+            self.realm = realm
+            self.realm_source = (
+                'Discovered Kerberos DNS records from %s' % self.domain)
+
+        if not servers and not realm:
             return REALM_NOT_FOUND
 
-        self.realm = krb_realm
-        self.kdc = kdc
-        self.realm_source = self.kdc_source = (
+        self.kdc = self.ipadnssearchkrbkdc()
+        self.kdc_source = (
             'Discovered Kerberos DNS records from %s' % self.domain)
 
         # We may have received multiple servers corresponding to the domain
@@ -336,6 +343,10 @@ class IPADiscovery(object):
                                      no_schema=True, decode_attrs=False)
             try:
                 lh.do_simple_bind(DN(), '')
+
+                # get IPA base DN
+                root_logger.debug("Search LDAP server for IPA base DN")
+                basedn = get_ipa_basedn(lh)
             except errors.ACIError:
                 root_logger.debug("LDAP Error: Anonymous access not allowed")
                 return [NO_ACCESS_TO_LDAP]
@@ -351,10 +362,6 @@ class IPADiscovery(object):
                 else:
                     return [UNKNOWN_ERROR]
 
-            # get IPA base DN
-            root_logger.debug("Search LDAP server for IPA base DN")
-            basedn = get_ipa_basedn(lh)
-
             if basedn is None:
                 root_logger.debug("The server is not an IPA server")
                 return [NOT_IPA_SERVER]
@@ -376,17 +383,22 @@ class IPADiscovery(object):
 
             for lres in lret:
                 root_logger.debug("Found: %s", lres.dn)
-                lrealms.append(lres.single_value('cn'))
+                lrealms.append(lres.single_value['cn'])
 
             if trealm:
                 for r in lrealms:
                     if trealm == r:
                         return [0, thost, trealm]
                 # must match or something is very wrong
+                root_logger.debug("Realm %s does not match any realm in LDAP "
+                                  "database", trealm)
                 return [REALM_NOT_FOUND]
             else:
                 if len(lrealms) != 1:
                     #which one? we can't attach to a multi-realm server without DNS working
+                    root_logger.debug("Multiple realms found, cannot decide "
+                                      "which realm is the right without "
+                                      "working DNS")
                     return [REALM_NOT_FOUND]
                 else:
                     return [0, thost, lrealms[0]]
@@ -453,11 +465,12 @@ class IPADiscovery(object):
 
         return servers
 
-    def ipadnssearchkrb(self, tdomain):
+    def ipadnssearchkrbrealm(self, domain=None):
         realm = None
-        kdc = None
+        if not domain:
+            domain = self.domain
         # now, check for a Kerberos realm the local host or domain is in
-        qname = "_kerberos." + tdomain
+        qname = "_kerberos." + domain
 
         root_logger.debug("Search DNS for TXT record of %s", qname)
 
@@ -473,18 +486,21 @@ class IPADiscovery(object):
                 realm = answer.strings[0]
                 if realm:
                     break
+        return realm
 
-        if realm:
-            # now fetch server information for the realm
-            domain = realm.lower()
+    def ipadnssearchkrbkdc(self, domain=None):
+        kdc = None
 
-            kdc = self.ipadns_search_srv(domain, '_kerberos._udp', 88,
-                    break_on_first=False)
+        if not domain:
+            domain = self.domain
 
-            if kdc:
-                kdc = ','.join(kdc)
-            else:
-                root_logger.debug("SRV record for KDC not found! Realm: %s, SRV record: %s" % (realm, qname))
-                kdc = None
+        kdc = self.ipadns_search_srv(domain, '_kerberos._udp', 88,
+                                     break_on_first=False)
 
-        return realm, kdc
+        if kdc:
+            kdc = ','.join(kdc)
+        else:
+            root_logger.debug("SRV record for KDC not found! Domain: %s" % domain)
+            kdc = None
+
+        return kdc
