@@ -113,6 +113,7 @@ DEFAULT_CONFIG = (
     ('container_realm_domains', DN(('cn', 'Realm Domains'), ('cn', 'ipa'), ('cn', 'etc'))),
     ('container_otp', DN(('cn', 'otp'))),
     ('container_radiusproxy', DN(('cn', 'radiusproxy'))),
+    ('container_views', DN(('cn', 'views'), ('cn', 'accounts'))),
 
     # Ports, hosts, and URIs:
     ('xmlrpc_uri', 'http://localhost:8888/ipa/xml'),
@@ -121,6 +122,10 @@ DEFAULT_CONFIG = (
 
     ('rpc_protocol', 'jsonrpc'),
 
+    # Define an inclusive range of SSL/TLS version support
+    ('tls_version_min', 'tls1.0'),
+    ('tls_version_max', 'tls1.2'),
+
     # Time to wait for a service to start, in seconds
     ('startup_timeout', 300),
 
@@ -211,3 +216,6 @@ DEFAULT_CONFIG = (
 )
 
 LDAP_GENERALIZED_TIME_FORMAT = "%Y%m%d%H%M%SZ"
+
+IPA_ANCHOR_PREFIX = ':IPA:'
+SID_ANCHOR_PREFIX = ':SID:'
