@@ -105,6 +105,7 @@ current block assignments:
 from ipalib.text import ngettext as ungettext
 
 import messages
+from ipaplatform.paths import paths
 
 
 class PrivateError(StandardError):
@@ -142,7 +143,7 @@ class SubprocessError(PrivateError):
     The exit code of the sub-process is available via the ``returncode``
     instance attribute.  For example:
 
-    >>> e = SubprocessError(returncode=1, argv=('/bin/false',))
+    >>> e = SubprocessError(returncode=1, argv=(paths.BIN_FALSE,))
     >>> e.returncode
     1
     >>> e.argv  # argv is also available
@@ -583,6 +584,12 @@ class InvalidSessionPassword(SessionError):
     errno = 1201
     format= _('Principal %(principal)s cannot be authenticated: %(message)s')
 
+class PasswordExpired(InvalidSessionPassword):
+    """
+    **1202** Raised when we cannot obtain a TGT for a principal because the password is expired.
+    """
+    errno = 1202
+
 ##############################################################################
 # 2000 - 2999: Authorization errors
 class AuthorizationError(PublicError):
@@ -810,6 +817,22 @@ class DeprecationError(InvocationError):
     errno = 3015
     format = _("Command '%(name)s' has been deprecated")
 
+class NotAForestRootError(InvocationError):
+    """
+    **3016** Raised when an attempt to establish trust is done against non-root domain
+             Forest root domain has the same name as the forest itself
+
+    For example:
+
+    >>> raise NotAForestRootError(forest='example.test', domain='jointops.test')
+    Traceback (most recent call last):
+      ...
+    NotAForestRootError: Domain 'jointops.test' is not a root domain for forest 'example.test'
+    """
+
+    errno = 3016
+    format = _("Domain '%(domain)s' is not a root domain for forest '%(forest)s'")
+
 
 ##############################################################################
 # 4000 - 4999: Execution errors
@@ -1112,19 +1135,19 @@ class DefaultGroupError(ExecutionError):
 
 class DNSNotARecordError(ExecutionError):
     """
-    **4019** Raised when a hostname is not a DNS A record
+    **4019** Raised when a hostname is not a DNS A/AAAA record
 
     For example:
 
     >>> raise DNSNotARecordError()
     Traceback (most recent call last):
       ...
-    DNSNotARecordError: Host does not have corresponding DNS A record
+    DNSNotARecordError: Host does not have corresponding DNS A/AAAA record
 
     """
 
     errno = 4019
-    format = _('Host does not have corresponding DNS A record')
+    format = _('Host does not have corresponding DNS A/AAAA record')
 
 class ManagedGroupError(ExecutionError):
     """
@@ -1306,6 +1329,21 @@ class PosixGroupViolation(ExecutionError):
     errno = 4030
     format = _('This is already a posix group and cannot be converted to external one')
 
+class EmptyResult(NotFound):
+    """
+    **4031** Raised when a LDAP search returned no results.
+
+    For example:
+
+    >>> raise EmptyResult(reason='no matching entry found')
+    Traceback (most recent call last):
+      ...
+    EmptyResult: no matching entry found
+
+    """
+
+    errno = 4031
+
 class BuiltinError(ExecutionError):
     """
     **4100** Base class for builtin execution errors (*4100 - 4199*).
@@ -1512,6 +1550,40 @@ class DatabaseTimeout(DatabaseError):
     format = _('LDAP timeout')
 
 
+class DNSDataMismatch(ExecutionError):
+    """
+    **4212** Raised when an DNS query didn't return expected answer
+    in a configured time limit.
+
+    For example:
+
+    >>> raise DNSDataMismatch(expected="zone3.test. 86400 IN A 192.0.2.1", \
+                              got="zone3.test. 86400 IN A 192.168.1.1")
+    Traceback (most recent call last):
+      ...
+    DNSDataMismatch: DNS check failed: Expected {zone3.test. 86400 IN A 192.0.2.1} got {zone3.test. 86400 IN A 192.168.1.1}
+    """
+
+    errno = 4212
+    format = _('DNS check failed: Expected {%(expected)s} got {%(got)s}')
+
+
+class TaskTimeout(DatabaseError):
+    """
+    **4213** Raised when an LDAP task times out
+
+    For example:
+
+    >>> raise TaskTimeout(task='Automember', task_dn='')
+    Traceback (most recent call last):
+      ...
+    TaskTimeout: Automember LDAP task timeout, Task DN: ''
+    """
+
+    errno = 4213
+    format = _("%(task)s LDAP task timeout, Task DN: '%(task_dn)s'")
+
+
 class CertificateError(ExecutionError):
     """
     **4300** Base class for Certificate execution errors (*4300 - 4399*).
