@@ -157,6 +157,63 @@ class DNSSECWarning(PublicMessage):
     type = "warning"
     format = _("DNSSEC support is experimental.\n%(additional_info)s")
 
+
+class OptionDeprecatedWarning(PublicMessage):
+    """
+    **13004** Used when user uses a deprecated option
+    """
+
+    errno = 13004
+    type = "warning"
+    format = _(u"'%(option)s' option is deprecated. %(additional_info)s")
+
+
+class OptionSemanticChangedWarning(PublicMessage):
+    """
+    **13005** Used when option which recently changes its semantic is used
+    """
+
+    errno = 13005
+    type = "warning"
+    format = _(u"Semantic of %(label)s was changed. %(current_behavior)s\n"
+               u"%(hint)s")
+
+
+class DNSServerNotRespondingWarning(PublicMessage):
+    """
+    **13006**  Used when a DNS server is not responding to queries
+    """
+
+    errno = 13006
+    type = "warning"
+    format = _(u"DNS server %(server)s not responding.")
+
+
+class DNSServerDoesNotSupportDNSSECWarning(PublicMessage):
+    """
+    **13007** Used when a DNS server does not support DNSSEC validation
+    """
+
+    errno = 13007
+    type = "warning"
+    format = _(u"DNS server %(server)s does not support DNSSEC. "
+               u"If DNSSEC validation is enabled on IPA server(s), "
+               u"please disable it.")
+
+class ForwardzoneIsNotEffectiveWarning(PublicMessage):
+    """
+    **13008** Forwardzone is not effective, forwarding will not work because
+    there is authoritative parent zone, without proper NS delegation
+    """
+
+    errno = 13008
+    type = "warning"
+    format = _(u"forward zone \"%(fwzone)s\" is not effective because of "
+               u"missing proper NS delegation in authoritative zone "
+               u"\"%(authzone)s\". Please add NS record "
+               u"\"%(ns_rec)s\" to parent zone \"%(authzone)s\".")
+
+
 def iter_messages(variables, base):
     """Return a tuple with all subclasses
     """
