@@ -102,6 +102,7 @@ a more detailed description for clarity.
 import re
 import decimal
 import base64
+import datetime
 from xmlrpclib import MAXINT, MININT
 
 from types import NoneType
@@ -109,10 +110,16 @@ from text import _ as ugettext
 from plugable import ReadOnly, lock, check_name
 from errors import ConversionError, RequirementError, ValidationError
 from errors import PasswordMismatch, Base64DecodeError
-from constants import NULLS, TYPE_ERROR, CALLABLE_ERROR
+from constants import TYPE_ERROR, CALLABLE_ERROR, LDAP_GENERALIZED_TIME_FORMAT
 from text import Gettext, FixMe
-from util import json_serialize
+from util import json_serialize, validate_idna_domain
 from ipapython.dn import DN
+from ipapython.dnsutil import DNSName
+import dns.name
+import encodings.idna
+
+def _is_null(value):
+    return not value and value != 0 # NOTE: False == 0
 
 class DefaultFrom(ReadOnly):
     """
@@ -297,6 +304,7 @@ class Param(ReadOnly):
 
       - cli_name: option name in CLI
       - cli_short_name: one character version of cli_name
+      - deprecated_cli_aliases: deprecated CLI aliases
       - label: very short description of the parameter. This value is used in
         when the Command output is printed to CLI or in a Command help
       - doc: parameter long description used in help
@@ -328,7 +336,9 @@ class Param(ReadOnly):
             * no_display (Output parameters only): do not display the parameter
             * no_create: do not include the parameter for crud.Create based
               commands
-            * no_update: do not include the parameter for crud.update based
+            * no_update: do not include the parameter for crud.Update based
+              commands
+            * no_search: do not include the parameter for crud.Search based
               commands
             * no_option: this attribute is not displayed in the CLI, usually
               because there's a better way of setting it (for example, a
@@ -349,6 +359,10 @@ class Param(ReadOnly):
               not be given at all. All crud.Update commands automatically
               convert required parameters to `nonempty` ones, so the value
               can be unspecified (unchanged) but cannot be deleted.
+            * optional_create: do not require the parameter for crud.Create
+              based commands
+            * allow_mod_for_managed_permission: permission-mod allows changing
+              the parameter for managed permissions
       - hint: this attribute is currently not used
       - alwaysask: when enabled, CLI asks for parameter value even when the
         parameter is not `required`
@@ -359,7 +373,9 @@ class Param(ReadOnly):
 
     # This is a dummy type so that most of the functionality of Param can be
     # unit tested directly without always creating a subclass; however, a real
-    # (direct) subclass must *always* override this class attribute:
+    # (direct) subclass must *always* override this class attribute.
+    # If multiple types are permitted, set `type` to the canonical type and
+    # `allowed_types` to a tuple of all allowed types.
     type = NoneType  # Ouch, this wont be very useful in the real world!
 
     # Subclasses should override this with something more specific:
@@ -368,9 +384,12 @@ class Param(ReadOnly):
     # _convert_scalar operates only on scalar values
     scalar_error = _('Only one value is allowed')
 
+    password = False
+
     kwargs = (
         ('cli_name', str, None),
         ('cli_short_name', str, None),
+        ('deprecated_cli_aliases', frozenset, frozenset()),
         ('label', (basestring, Gettext), None),
         ('doc', (basestring, Gettext), None),
         ('required', bool, True),
@@ -394,16 +413,16 @@ class Param(ReadOnly):
         # ('default', self.type, None),
     )
 
+    @property
+    def allowed_types(self):
+        """The allowed datatypes for this Param"""
+        return (self.type,)
+
     def __init__(self, name, *rules, **kw):
         # We keep these values to use in __repr__():
         self.param_spec = name
         self.__kw = dict(kw)
 
-        if isinstance(self, Password):
-            self.password = True
-        else:
-            self.password = False
-
         # Merge in kw from parse_param_spec():
         (name, kw_from_spec) = parse_param_spec(name)
         if not 'required' in kw:
@@ -414,7 +433,7 @@ class Param(ReadOnly):
         self.nice = '%s(%r)' % (self.__class__.__name__, self.param_spec)
 
         # Add 'default' to self.kwargs and makes sure no unknown kw were given:
-        assert type(self.type) is type
+        assert all(type(t) is type for t in self.allowed_types)
         if kw.get('multivalue', True):
             self.kwargs += (('default', tuple, None),)
         else:
@@ -451,7 +470,7 @@ class Param(ReadOnly):
             value = kw.get(key, default)
             if value is not None:
                 if kind is frozenset:
-                    if type(value) in (list, tuple):
+                    if type(value) in (list, tuple, set):
                         value = frozenset(value)
                     elif type(value) is str:
                         value = frozenset([value])
@@ -540,7 +559,7 @@ class Param(ReadOnly):
         """
         One stop shopping.
         """
-        if value in NULLS:
+        if _is_null(value):
             value = self.get_default(**kw)
         else:
             value = self.convert(self.normalize(value))
@@ -633,9 +652,8 @@ class Param(ReadOnly):
         """
         Return a value safe for logging.
 
-        This is used so that passwords don't get logged.  If this is a
-        `Password` instance and ``value`` is not ``None``, a constant
-        ``u'********'`` is returned.  For example:
+        This is used so that sensitive values like passwords don't get logged.
+        For example:
 
         >>> p = Password('my_password')
         >>> p.safe_value(u'This is my password')
@@ -643,9 +661,6 @@ class Param(ReadOnly):
         >>> p.safe_value(None) is None
         True
 
-        If this is not a `Password` instance, ``value`` is returned unchanged.
-        For example:
-
         >>> s = Str('my_str')
         >>> s.safe_value(u'Some arbitrary value')
         u'Some arbitrary value'
@@ -734,16 +749,16 @@ class Param(ReadOnly):
 
         (Note that `Str` is a subclass of `Param`.)
 
-        All values in `constants.NULLS` will be converted to ``None``.  For
-        example:
+        All non-numeric, non-boolean values which evaluate to False will be
+        converted to None.  For example:
 
         >>> scalar.convert(u'') is None  # An empty string
         True
         >>> scalar.convert([]) is None  # An empty list
         True
 
-        Likewise, values in `constants.NULLS` will be filtered out of a
-        multivalue parameter.  For example:
+        Likewise, they will be filtered out of a multivalue parameter.
+        For example:
 
         >>> multi = Str('my_multi', multivalue=True)
         >>> multi.convert([1.5, '', 17, None, u'Hello'])
@@ -766,14 +781,14 @@ class Param(ReadOnly):
 
         :param value: A proposed value for this parameter.
         """
-        if value in NULLS:
+        if _is_null(value):
             return
         if self.multivalue:
             if type(value) not in (tuple, list):
                 value = (value,)
             values = tuple(
                 self._convert_scalar(v, i) for (i, v) in filter(
-                    lambda iv: iv[1] not in NULLS, enumerate(value)
+                    lambda iv: not _is_null(iv[1]), enumerate(value)
                 )
             )
             if len(values) == 0:
@@ -785,7 +800,7 @@ class Param(ReadOnly):
         """
         Convert a single scalar value.
         """
-        if type(value) is self.type:
+        if type(value) in self.allowed_types:
             return value
         raise ConversionError(name=self.name, index=index,
             error=ugettext(self.type_error),
@@ -819,7 +834,7 @@ class Param(ReadOnly):
             self._validate_scalar(value)
 
     def _validate_scalar(self, value, index=None):
-        if type(value) is not self.type:
+        if type(value) not in self.allowed_types:
             raise TypeError(
                 TYPE_ERROR % (self.name, self.type, value, type(value))
             )
@@ -899,6 +914,9 @@ class Param(ReadOnly):
                     pass
         return self.default
 
+    def sort_key(self, value):
+        return value
+
     json_exclude_attrs = (
         'alwaysask', 'autofill', 'cli_name', 'cli_short_name', 'csv',
         'sortorder', 'falsehoods', 'truths', 'version',
@@ -945,7 +963,7 @@ class Bool(Param):
         """
         Convert a single scalar value.
         """
-        if type(value) is self.type:
+        if type(value) in self.allowed_types:
             return value
         if isinstance(value, basestring):
             value = value.lower()
@@ -1012,7 +1030,7 @@ class Number(Param):
         """
         Convert a single scalar value.
         """
-        if type(value) is self.type:
+        if type(value) in self.allowed_types:
             return value
         if type(value) in (unicode, int, long, float):
             try:
@@ -1033,6 +1051,7 @@ class Int(Number):
     """
 
     type = int
+    allowed_types = int, long
     type_error = _('must be an integer')
 
     kwargs = Param.kwargs + (
@@ -1040,6 +1059,21 @@ class Int(Number):
         ('maxvalue', (int, long), int(MAXINT)),
     )
 
+    @staticmethod
+    def convert_int(value):
+        if type(value) in Int.allowed_types:
+            return value
+
+        if type(value) is float:
+            return int(value)
+
+        if type(value) is unicode:
+            if u'.' in value:
+                return int(float(value))
+            return int(value, 0)
+
+        raise ValueError(value)
+
     def __init__(self, name, *rules, **kw):
         super(Int, self).__init__(name, *rules, **kw)
 
@@ -1053,30 +1087,12 @@ class Int(Number):
         """
         Convert a single scalar value.
         """
-        if type(value) in (int, long):
-            return value
-        if type(value) is unicode:
-            # permit floating point strings
-            if value.find(u'.') >= 0:
-                try:
-                    return int(float(value))
-                except ValueError:
-                    pass
-            else:
-                try:
-                    # 2nd arg is radix base, 2nd arg only accepted for strings.
-                    # Zero means determine radix base from prefix (e.g. 0x for hex)
-                    return int(value, 0)
-                except ValueError:
-                    pass
-        if type(value) is float:
-            try:
-                return int(value)
-            except ValueError:
-                pass
-        raise ConversionError(name=self.get_param_name(), index=index,
-            error=ugettext(self.type_error),
-        )
+        try:
+            return Int.convert_int(value)
+        except ValueError:
+            raise ConversionError(name=self.get_param_name(),
+                                  index=index,
+                                  error=ugettext(self.type_error))
 
     def _rule_minvalue(self, _, value):
         """
@@ -1098,31 +1114,6 @@ class Int(Number):
                 maxvalue=self.maxvalue,
             )
 
-    def _validate_scalar(self, value, index=None):
-        """
-        This duplicates _validate_scalar in the Param class with
-        the exception that it allows both int and long types. The
-        min/max rules handle size enforcement.
-        """
-        if type(value) not in (int, long):
-            raise TypeError(
-                TYPE_ERROR % (self.name, self.type, value, type(value))
-            )
-        if index is not None and type(index) is not int:
-            raise TypeError(
-                TYPE_ERROR % ('index', int, index, type(index))
-            )
-        for rule in self.all_rules:
-            error = rule(ugettext, value)
-            if error is not None:
-                raise ValidationError(
-                    name=self.get_param_name(),
-                    value=value,
-                    index=index,
-                    error=error,
-                    rule=rule,
-                )
-
 
 class Decimal(Number):
     """
@@ -1318,7 +1309,7 @@ class Data(Param):
         """
         Check pattern (regex) contraint.
         """
-        assert type(value) is self.type
+        assert type(value) in self.allowed_types
         if self.re.match(value) is None:
             if self.re_errmsg:
                 return self.re_errmsg % dict(pattern=self.pattern,)
@@ -1421,7 +1412,7 @@ class Str(Data):
         """
         Convert a single scalar value.
         """
-        if type(value) is self.type:
+        if type(value) in self.allowed_types:
             return value
         if type(value) in (int, long, float, decimal.Decimal):
             return self.type(value)
@@ -1472,6 +1463,8 @@ class Str(Data):
                 length=self.length,
             )
 
+    def sort_key(self, value):
+        return value.lower()
 
 class IA5Str(Str):
     """
@@ -1498,6 +1491,8 @@ class Password(Str):
     A parameter for passwords (stored in the ``unicode`` type).
     """
 
+    password = True
+
     kwargs = Str.kwargs + (
         ('confirm', bool, True),
     )
@@ -1523,7 +1518,7 @@ class Enum(Param):
     def __init__(self, name, *rules, **kw):
         super(Enum, self).__init__(name, *rules, **kw)
         for (i, v) in enumerate(self.values):
-            if type(v) is not self.type:
+            if type(v) not in self.allowed_types:
                 n = '%s values[%d]' % (self.nice, i)
                 raise TypeError(
                     TYPE_ERROR % (n, self.type, v, type(v))
@@ -1567,6 +1562,27 @@ class StrEnum(Enum):
     type = unicode
 
 
+class IntEnum(Enum):
+    """
+    Enumerable for integer data (stored in the ``int`` type).
+    """
+
+    type = int
+    allowed_types = int, long
+    type_error = Int.type_error
+
+    def _convert_scalar(self, value, index=None):
+        """
+        Convert a single scalar value.
+        """
+        try:
+            return Int.convert_int(value)
+        except ValueError:
+            raise ConversionError(name=self.get_param_name(),
+                                  index=index,
+                                  error=ugettext(self.type_error))
+
+
 class Any(Param):
     """
     A parameter capable of holding values of any type. For internal use only.
@@ -1602,6 +1618,55 @@ class File(Str):
         ('noextrawhitespace', bool, False),
     )
 
+class DateTime(Param):
+    """
+    DateTime parameter type.
+
+    Accepts LDAP Generalized time without in the following format:
+       '%Y%m%d%H%M%SZ'
+
+    Accepts subset of values defined by ISO 8601:
+        '%Y-%m-%dT%H:%M:%SZ'
+        '%Y-%m-%dT%H:%MZ'
+        '%Y-%m-%dZ'
+
+    Also accepts above formats using ' ' (space) as a separator instead of 'T'.
+
+    Refer to the `man strftime` for the explanations for the %Y,%m,%d,%H.%M,%S.
+    """
+
+    accepted_formats = [LDAP_GENERALIZED_TIME_FORMAT,  # generalized time
+                        '%Y-%m-%dT%H:%M:%SZ',  # ISO 8601, second precision
+                        '%Y-%m-%dT%H:%MZ',     # ISO 8601, minute precision
+                        '%Y-%m-%dZ',           # ISO 8601, date only
+                        '%Y-%m-%d %H:%M:%SZ',  # non-ISO 8601, second precision
+                        '%Y-%m-%d %H:%MZ']     # non-ISO 8601, minute precision
+
+
+    type = datetime.datetime
+    type_error = _('must be datetime value')
+
+    def _convert_scalar(self, value, index=None):
+        if isinstance(value, basestring):
+            for date_format in self.accepted_formats:
+                try:
+                    time = datetime.datetime.strptime(value, date_format)
+                    return time
+                except ValueError:
+                    pass
+
+            # If we get here, the strptime call did not succeed for any
+            # the accepted formats, therefore raise error
+
+            error = (_("does not match any of accepted formats: ") +
+                      (', '.join(self.accepted_formats)))
+
+            raise ConversionError(name=self.get_param_name(),
+                                  index=index,
+                                  error=error)
+
+        return super(DateTime, self)._convert_scalar(value, index)
+
 
 class AccessTime(Str):
     """
@@ -1790,7 +1855,7 @@ class DNParam(Param):
         """
         Convert a single scalar value.
         """
-        if type(value) is self.type:
+        if type(value) in self.allowed_types:
             return value
 
         try:
@@ -1857,3 +1922,49 @@ def create_param(spec):
             TYPE_ERROR % ('spec', (str, Param), spec, type(spec))
         )
     return Str(spec)
+
+
+class DNSNameParam(Param):
+    """
+    Domain name parameter type.
+
+    :only_absolute a domain name has to be absolute
+        (makes it absolute from unicode input)
+    :only_relative a domain name has to be relative
+    """
+    type = DNSName
+    type_error = _('must be DNS name')
+    kwargs = Param.kwargs + (
+        ('only_absolute', bool, False),
+        ('only_relative', bool, False),
+    )
+
+    def __init__(self, name, *rules, **kw):
+        super(DNSNameParam, self).__init__(name, *rules, **kw)
+        if self.only_absolute and self.only_relative:
+            raise ValueError('%s: cannot be both absolute and relative' %
+                             self.nice)
+
+    def _convert_scalar(self, value, index=None):
+        if isinstance(value, unicode):
+            error = None
+
+            try:
+                validate_idna_domain(value)
+            except ValueError as e:
+                raise ConversionError(name=self.get_param_name(), index=index,
+                                      error=unicode(e))
+            value = DNSName(value)
+
+            if self.only_absolute and not value.is_absolute():
+                value = value.make_absolute()
+
+        return super(DNSNameParam, self)._convert_scalar(value, index)
+
+    def _rule_only_absolute(self, _, value):
+        if self.only_absolute and not value.is_absolute():
+            return _('must be absolute')
+
+    def _rule_only_relative(self, _, value):
+        if self.only_relative and value.is_absolute():
+            return _('must be relative')
