@@ -16,57 +16,61 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+import uuid
+import time
 
-from ipalib import api, errors
-from ipalib import Str, StrEnum
-from ipalib.plugins.baseldap import *
-from ipalib import _, ngettext
-from ipalib.request import context
 import ldap as _ldap
+
+from ipalib import api, errors, Str, StrEnum, DNParam, _, ngettext
+from ipalib.plugable import Registry
+from ipalib.plugins.baseldap import *
+from ipalib.request import context
 from ipapython.dn import DN
 
 __doc__ = _("""
 Auto Membership Rule.
-
+""") + _("""
 Bring clarity to the membership of hosts and users by configuring inclusive
 or exclusive regex patterns, you can automatically assign a new entries into
 a group or hostgroup based upon attribute information.
-
+""") + _("""
 A rule is directly associated with a group by name, so you cannot create
 a rule without an accompanying group or hostgroup.
-
+""") + _("""
 A condition is a regular expression used by 389-ds to match a new incoming
 entry with an automember rule. If it matches an inclusive rule then the
 entry is added to the appropriate group or hostgroup.
-
+""") + _("""
 A default group or hostgroup could be specified for entries that do not
 match any rule. In case of user entries this group will be a fallback group
 because all users are by default members of group specified in IPA config.
-
-
+""") + _("""
+The automember-rebuild command can be used to retroactively run automember rules
+against existing entries, thus rebuilding their membership.
+""") + _("""
 EXAMPLES:
-
+""") + _("""
  Add the initial group or hostgroup:
    ipa hostgroup-add --desc="Web Servers" webservers
    ipa group-add --desc="Developers" devel
-
+""") + _("""
  Add the initial rule:
    ipa automember-add --type=hostgroup webservers
    ipa automember-add --type=group devel
-
+""") + _("""
  Add a condition to the rule:
    ipa automember-add-condition --key=fqdn --type=hostgroup --inclusive-regex=^web[1-9]+\.example\.com webservers
    ipa automember-add-condition --key=manager --type=group --inclusive-regex=^uid=mscott devel
-
+""") + _("""
  Add an exclusive condition to the rule to prevent auto assignment:
    ipa automember-add-condition --key=fqdn --type=hostgroup --exclusive-regex=^web5\.example\.com webservers
-
+""") + _("""
  Add a host:
     ipa host-add web1.example.com
-
+""") + _("""
  Add a user:
     ipa user-add --first=Tim --last=User --password tuser1 --manager=mscott
-
+""") + _("""
  Verify automembership:
     ipa hostgroup-show webservers
       Host-group: webservers
@@ -78,41 +82,60 @@ EXAMPLES:
       Description: Developers
       GID: 1004200000
       Member users: tuser
-
+""") + _("""
  Remove a condition from the rule:
    ipa automember-remove-condition --key=fqdn --type=hostgroup --inclusive-regex=^web[1-9]+\.example\.com webservers
-
+""") + _("""
  Modify the automember rule:
     ipa automember-mod
-
+""") + _("""
  Set the default (fallback) target group:
     ipa automember-default-group-set --default-group=webservers --type=hostgroup
     ipa automember-default-group-set --default-group=ipausers --type=group
-
+""") + _("""
  Remove the default (fallback) target group:
     ipa automember-default-group-remove --type=hostgroup
     ipa automember-default-group-remove --type=group
-
+""") + _("""
  Show the default (fallback) target group:
     ipa automember-default-group-show --type=hostgroup
     ipa automember-default-group-show --type=group
-
+""") + _("""
  Find all of the automember rules:
     ipa automember-find
-
+""") + _("""
  Display a automember rule:
     ipa automember-show --type=hostgroup webservers
     ipa automember-show --type=group devel
-
+""") + _("""
  Delete an automember rule:
     ipa automember-del --type=hostgroup webservers
     ipa automember-del --type=group devel
+""") + _("""
+ Rebuild membership for all users:
+    ipa automember-rebuild --type=group
+""") + _("""
+ Rebuild membership for all hosts:
+    ipa automember-rebuild --type=hostgroup
+""") + _("""
+ Rebuild membership for specified users:
+    ipa automember-rebuild --users=tuser1 --users=tuser2
+""") + _("""
+ Rebuild membership for specified hosts:
+    ipa automember-rebuild --hosts=web1.example.com --hosts=web2.example.com
 """)
 
+register = Registry()
+
 # Options used by Condition Add and Remove.
 INCLUDE_RE = 'automemberinclusiveregex'
 EXCLUDE_RE = 'automemberexclusiveregex'
 
+REBUILD_TASK_CONTAINER = DN(('cn', 'automember rebuild membership'),
+                            ('cn', 'tasks'),
+                            ('cn', 'config'))
+
+
 regex_attrs = (
     Str('automemberinclusiveregex*',
         cli_name='inclusive_regex',
@@ -152,6 +175,8 @@ automember_rule = (
     ),
 )
 
+
+@register()
 class automember(LDAPObject):
 
     """
@@ -160,13 +185,49 @@ class automember(LDAPObject):
 
     container_dn = api.env.container_automember
 
-    object_name = 'auto_member_rule'
-    object_name_plural = 'auto_member_rules'
+    object_name = 'Automember rule'
+    object_name_plural = 'Automember rules'
     object_class = ['top', 'automemberregexrule']
+    permission_filter_objectclasses = ['automemberregexrule']
     default_attributes = [
         'automemberinclusiveregex', 'automemberexclusiveregex',
         'cn', 'automembertargetgroup', 'description', 'automemberdefaultgroup'
     ]
+    managed_permissions = {
+        'System: Read Automember Definitions': {
+            'non_object': True,
+            'ipapermlocation': DN(container_dn, api.env.basedn),
+            'ipapermtargetfilter': {'(objectclass=automemberdefinition)'},
+            'replaces_global_anonymous_aci': True,
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'objectclass', 'cn', 'automemberscope', 'automemberfilter',
+                'automembergroupingattr', 'automemberdefaultgroup',
+                'automemberdisabled',
+            },
+            'default_privileges': {'Automember Readers',
+                                   'Automember Task Administrator'},
+        },
+        'System: Read Automember Rules': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'cn', 'objectclass', 'automembertargetgroup', 'description',
+                'automemberexclusiveregex', 'automemberinclusiveregex',
+            },
+            'default_privileges': {'Automember Readers',
+                                   'Automember Task Administrator'},
+        },
+        'System: Read Automember Tasks': {
+            'non_object': True,
+            'ipapermlocation': DN('cn=tasks', 'cn=config'),
+            'ipapermtarget': DN('cn=*', REBUILD_TASK_CONTAINER),
+            'replaces_global_anonymous_aci': True,
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {'*'},
+            'default_privileges': {'Automember Task Administrator'},
+        },
+    }
 
     label = _('Auto Membership Rule')
 
@@ -184,20 +245,23 @@ class automember(LDAPObject):
         ),
     )
 
-    def dn_exists(self, grouptype, groupname, *keys):
+    def dn_exists(self, otype, oname):
         ldap = self.api.Backend.ldap2
-        dn = self.api.Object[grouptype].get_dn(groupname)
+        dn = self.api.Object[otype].get_dn(oname)
         try:
-            (gdn, entry_attrs) = ldap.get_entry(dn, [])
+            entry = ldap.get_entry(dn, [])
         except errors.NotFound:
-            raise errors.NotFound(reason=_(u'Group: %s not found!') % groupname)
-        return gdn
+            raise errors.NotFound(
+                reason=_(u'%(otype)s "%(oname)s" not found') %
+                dict(otype=otype, oname=oname)
+            )
+        return entry.dn
 
     def get_dn(self, *keys, **options):
         if self.parent_object:
             parent_dn = self.api.Object[self.parent_object].get_dn(*keys[:-1])
         else:
-            parent_dn = self.container_dn
+            parent_dn = DN(self.container_dn, api.env.basedn)
         grouptype = options['type']
         try:
             ndn = DN(('cn', keys[-1]), ('cn', grouptype), parent_dn)
@@ -216,16 +280,16 @@ class automember(LDAPObject):
         else:
             raise errors.NotFound(reason=_('%s is not a valid attribute.') % attr)
 
-api.register(automember)
-
 
 def automember_container_exists(ldap):
     try:
-        ldap.get_entry(api.env.container_automember, [])
+        ldap.get_entry(DN(api.env.container_automember, api.env.basedn), [])
     except errors.NotFound:
         return False
     return True
 
+
+@register()
 class automember_add(LDAPCreate):
     __doc__ = _("""
     Add an automember rule.
@@ -245,12 +309,11 @@ class automember_add(LDAPCreate):
 
     def execute(self, *keys, **options):
         result = super(automember_add, self).execute(*keys, **options)
-        result['value'] = keys[-1]
+        result['value'] = pkey_to_value(keys[-1], options)
         return result
 
-api.register(automember_add)
-
 
+@register()
 class automember_add_condition(LDAPUpdate):
     __doc__ = _("""
     Add conditions to an automember rule.
@@ -285,7 +348,7 @@ class automember_add_condition(LDAPUpdate):
         assert isinstance(dn, DN)
         # Check to see if the automember rule exists
         try:
-            (tdn, test_attrs) = ldap.get_entry(dn, [])
+            dn = ldap.get_entry(dn, []).dn
         except errors.NotFound:
             raise errors.NotFound(reason=_(u'Auto member rule: %s not found!') % keys[0])
         # Define container key
@@ -303,9 +366,8 @@ class automember_add_condition(LDAPUpdate):
                 entry_attrs[attr] = [key + condition for condition in options[attr]]
                 completed += len(entry_attrs[attr])
                 try:
-                    (dn, old_entry) = ldap.get_entry(
-                        dn, [attr], normalize=self.obj.normalize_dn)
-                    for regex in old_entry:
+                    old_entry = ldap.get_entry(dn, [attr])
+                    for regex in old_entry.keys():
                         if not isinstance(entry_attrs[regex], (list, tuple)):
                             entry_attrs[regex] = [entry_attrs[regex]]
                         duplicate = set(old_entry[regex]) & set(entry_attrs[regex])
@@ -316,6 +378,8 @@ class automember_add_condition(LDAPUpdate):
                 except errors.NotFound:
                     failed['failed'][attr].append(regex)
 
+        entry_attrs = entry_to_dict(entry_attrs, **options)
+
         # Set failed and completed to they can be harvested in the execute super
         setattr(context, 'failed', failed)
         setattr(context, 'completed', completed)
@@ -323,9 +387,7 @@ class automember_add_condition(LDAPUpdate):
 
         # Make sure to returned the failed results if there is nothing to remove
         if completed == 0:
-            (dn, entry_attrs) = ldap.get_entry(
-                dn, attrs_list, normalize=self.obj.normalize_dn
-            )
+            ldap.get_entry(dn, attrs_list)
             raise errors.EmptyModlist
         return dn
 
@@ -339,12 +401,11 @@ class automember_add_condition(LDAPUpdate):
             result =  {'result': getattr(context, 'entry_attrs'), 'value': keys[-1]}
         result['failed'] = getattr(context, 'failed')
         result['completed'] = getattr(context, 'completed')
-        result['value'] = keys[-1]
+        result['value'] = pkey_to_value(keys[-1], options)
         return result
 
-api.register(automember_add_condition)
-
 
+@register()
 class automember_remove_condition(LDAPUpdate):
     __doc__ = _("""
     Remove conditions from an automember rule.
@@ -372,7 +433,7 @@ class automember_remove_condition(LDAPUpdate):
         assert isinstance(dn, DN)
         # Check to see if the automember rule exists
         try:
-            (tdn, test_attrs) = ldap.get_entry(dn, [])
+            ldap.get_entry(dn, [])
         except errors.NotFound:
             raise errors.NotFound(reason=_(u'Auto member rule: %s not found!') % keys[0])
 
@@ -388,16 +449,13 @@ class automember_remove_condition(LDAPUpdate):
         failed = {'failed': {}}
 
         # Check to see if there are existing exclusive conditions present.
-        (dn, exclude_present) = ldap.get_entry(
-            dn, [EXCLUDE_RE], normalize=self.obj.normalize_dn)
+        dn = ldap.get_entry(dn, [EXCLUDE_RE]).dn
 
         for attr in (INCLUDE_RE, EXCLUDE_RE):
             failed['failed'][attr] = []
             if attr in options and options[attr]:
                 entry_attrs[attr] = [key + condition for condition in options[attr]]
-                (dn, entry_attrs_) = ldap.get_entry(
-                    dn, [attr], normalize=self.obj.normalize_dn
-                )
+                entry_attrs_ = ldap.get_entry(dn, [attr])
                 old_entry = entry_attrs_.get(attr, [])
                 for regex in entry_attrs[attr]:
                     if regex in old_entry:
@@ -406,6 +464,9 @@ class automember_remove_condition(LDAPUpdate):
                     else:
                         failed['failed'][attr].append(regex)
                 entry_attrs[attr] = old_entry
+
+        entry_attrs = entry_to_dict(entry_attrs, **options)
+
         # Set failed and completed to they can be harvested in the execute super
         setattr(context, 'failed', failed)
         setattr(context, 'completed', completed)
@@ -413,9 +474,7 @@ class automember_remove_condition(LDAPUpdate):
 
         # Make sure to returned the failed results if there is nothing to remove
         if completed == 0:
-            (dn, entry_attrs) = ldap.get_entry(
-                dn, attrs_list, normalize=self.obj.normalize_dn
-            )
+            ldap.get_entry(dn, attrs_list)
             raise errors.EmptyModlist
         return dn
 
@@ -429,12 +488,11 @@ class automember_remove_condition(LDAPUpdate):
             result =  {'result': getattr(context, 'entry_attrs'), 'value': keys[-1]}
         result['failed'] = getattr(context, 'failed')
         result['completed'] = getattr(context, 'completed')
-        result['value'] = keys[-1]
+        result['value'] = pkey_to_value(keys[-1], options)
         return result
 
-api.register(automember_remove_condition)
-
 
+@register()
 class automember_mod(LDAPUpdate):
     __doc__ = _("""
     Modify an automember rule.
@@ -445,12 +503,11 @@ class automember_mod(LDAPUpdate):
 
     def execute(self, *keys, **options):
         result = super(automember_mod, self).execute(*keys, **options)
-        result['value'] = keys[-1]
+        result['value'] = pkey_to_value(keys[-1], options)
         return result
 
-api.register(automember_mod)
-
 
+@register()
 class automember_del(LDAPDelete):
     __doc__ = _("""
     Delete an automember rule.
@@ -461,12 +518,11 @@ class automember_del(LDAPDelete):
 
     def execute(self, *keys, **options):
         result = super(automember_del, self).execute(*keys, **options)
-        result['value'] = keys[-1]
+        result['value'] = pkey_to_value([keys[-1]], options)
         return result
 
-api.register(automember_del)
-
 
+@register()
 class automember_find(LDAPSearch):
     __doc__ = _("""
     Search for automember rules.
@@ -484,9 +540,8 @@ class automember_find(LDAPSearch):
         ndn = DN(('cn', options['type']), base_dn)
         return (filters, ndn, scope)
 
-api.register(automember_find)
-
 
+@register()
 class automember_show(LDAPRetrieve):
     __doc__ = _("""
     Display information about an automember rule.
@@ -497,12 +552,11 @@ class automember_show(LDAPRetrieve):
 
     def execute(self, *keys, **options):
         result = super(automember_show, self).execute(*keys, **options)
-        result['value'] = keys[-1]
+        result['value'] = pkey_to_value(keys[-1], options)
         return result
 
-api.register(automember_show)
-
 
+@register()
 class automember_default_group_set(LDAPUpdate):
     __doc__ = _("""
     Set default (fallback) group for all unmatched entries.
@@ -519,18 +573,18 @@ class automember_default_group_set(LDAPUpdate):
     msg_summary = _('Set default (fallback) group for automember "%(value)s"')
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
-        dn = DN(('cn', options['type']), api.env.container_automember)
+        dn = DN(('cn', options['type']), api.env.container_automember,
+                api.env.basedn)
         entry_attrs['automemberdefaultgroup'] = self.obj.dn_exists(options['type'], options['automemberdefaultgroup'])
         return dn
 
     def execute(self, *keys, **options):
         result = super(automember_default_group_set, self).execute(*keys, **options)
-        result['value'] = options['type']
+        result['value'] = pkey_to_value(options['type'], options)
         return result
 
-api.register(automember_default_group_set)
-
 
+@register()
 class automember_default_group_remove(LDAPUpdate):
     __doc__ = _("""
     Remove default (fallback) group for all unmatched entries.
@@ -540,18 +594,17 @@ class automember_default_group_remove(LDAPUpdate):
     msg_summary = _('Removed default (fallback) group for automember "%(value)s"')
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
-        dn = DN(('cn', options['type']), api.env.container_automember)
+        dn = DN(('cn', options['type']), api.env.container_automember,
+                api.env.basedn)
         attr = 'automemberdefaultgroup'
 
-        (dn, entry_attrs_) = ldap.get_entry(
-            dn, [attr], normalize=self.obj.normalize_dn
-        )
+        entry_attrs_ = ldap.get_entry(dn, [attr])
 
         if attr not in entry_attrs_:
             raise errors.NotFound(reason=_(u'No default (fallback) group set'))
         else:
             entry_attrs[attr] = []
-        return dn
+        return entry_attrs_.dn
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
         assert isinstance(dn, DN)
@@ -561,12 +614,11 @@ class automember_default_group_remove(LDAPUpdate):
 
     def execute(self, *keys, **options):
         result = super(automember_default_group_remove, self).execute(*keys, **options)
-        result['value'] = options['type']
+        result['value'] = pkey_to_value(options['type'], options)
         return result
 
-api.register(automember_default_group_remove)
-
 
+@register()
 class automember_default_group_show(LDAPRetrieve):
     __doc__ = _("""
     Display information about the default (fallback) automember groups.
@@ -574,7 +626,8 @@ class automember_default_group_show(LDAPRetrieve):
     takes_options = group_type
 
     def pre_callback(self, ldap, dn, attrs_list, *keys, **options):
-        dn = DN(('cn', options['type']), api.env.container_automember)
+        dn = DN(('cn', options['type']), api.env.container_automember,
+                api.env.basedn)
         return dn
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
@@ -585,7 +638,154 @@ class automember_default_group_show(LDAPRetrieve):
 
     def execute(self, *keys, **options):
         result = super(automember_default_group_show, self).execute(*keys, **options)
-        result['value'] = options['type']
+        result['value'] = pkey_to_value(options['type'], options)
         return result
 
-api.register(automember_default_group_show)
+
+@register()
+class automember_rebuild(Command):
+    __doc__ = _('Rebuild auto membership.')
+    # TODO: Add a --dry-run option:
+    # https://fedorahosted.org/freeipa/ticket/3936
+    takes_options = (
+        group_type[0].clone(
+            required=False,
+            label=_('Rebuild membership for all members of a grouping')
+        ),
+        Str(
+            'users*',
+            label=_('Users'),
+            doc=_('Rebuild membership for specified users'),
+        ),
+        Str(
+            'hosts*',
+            label=_('Hosts'),
+            doc=_('Rebuild membership for specified hosts'),
+        ),
+        Flag(
+            'no_wait?',
+            default=False,
+            label=_('No wait'),
+            doc=_("Don't wait for rebuilding membership"),
+        ),
+    )
+    has_output = output.standard_entry
+    has_output_params = (
+        DNParam(
+            'dn',
+            label=_('Task DN'),
+            doc=_('DN of the started task'),
+        ),
+    )
+
+    def validate(self, **kw):
+        """
+        Validation rules:
+        - at least one of 'type', 'users', 'hosts' is required
+        - 'users' and 'hosts' cannot be combined together
+        - if 'users' and 'type' are specified, 'type' must be 'group'
+        - if 'hosts' and 'type' are specified, 'type' must be 'hostgroup'
+        """
+        super(automember_rebuild, self).validate(**kw)
+        users, hosts, gtype = kw.get('users'), kw.get('hosts'), kw.get('type')
+
+        if not (gtype or users or hosts):
+            raise errors.MutuallyExclusiveError(
+                reason=_('at least one of options: type, users, hosts must be '
+                         'specified')
+            )
+
+        if users and hosts:
+            raise errors.MutuallyExclusiveError(
+                reason=_("users and hosts cannot both be set")
+            )
+        if gtype == 'group' and hosts:
+            raise errors.MutuallyExclusiveError(
+                reason=_("hosts cannot be set when type is 'group'")
+            )
+        if gtype == 'hostgroup' and users:
+            raise errors.MutuallyExclusiveError(
+                reason=_("users cannot be set when type is 'hostgroup'")
+            )
+
+    def execute(self, *keys, **options):
+        ldap = self.api.Backend.ldap2
+        cn = str(uuid.uuid4())
+
+        gtype = options.get('type')
+        if not gtype:
+            gtype = 'group' if options.get('users') else 'hostgroup'
+
+        types = {
+            'group': (
+                'user',
+                'users',
+                DN(api.env.container_user, api.env.basedn)
+            ),
+            'hostgroup': (
+                'host',
+                'hosts',
+                DN(api.env.container_host, api.env.basedn)
+            ),
+        }
+
+        obj_name, opt_name, basedn = types[gtype]
+        obj = self.api.Object[obj_name]
+
+        names = options.get(opt_name)
+        if names:
+            for name in names:
+                try:
+                    obj.get_dn_if_exists(name)
+                except errors.NotFound:
+                    obj.handle_not_found(name)
+            search_filter = ldap.make_filter_from_attr(
+                obj.primary_key.name,
+                names,
+                rules=ldap.MATCH_ANY
+            )
+        else:
+            search_filter = '(%s=*)' % obj.primary_key.name
+
+        task_dn = DN(('cn', cn), REBUILD_TASK_CONTAINER)
+
+        entry = ldap.make_entry(
+            task_dn,
+            objectclass=['top', 'extensibleObject'],
+            cn=[cn],
+            basedn=[basedn],
+            filter=[search_filter],
+            scope=['sub'],
+            ttl=[3600])
+        ldap.add_entry(entry)
+
+        summary = _('Automember rebuild membership task started')
+        result = {'dn': task_dn}
+
+        if not options.get('no_wait'):
+            summary = _('Automember rebuild membership task completed')
+            result = {}
+            start_time = time.time()
+
+            while True:
+                try:
+                    task = ldap.get_entry(task_dn)
+                except errors.NotFound:
+                    break
+
+                if 'nstaskexitcode' in task:
+                    if str(task.single_value['nstaskexitcode']) == '0':
+                        summary=task.single_value['nstaskstatus']
+                        break
+                    else:
+                        raise errors.DatabaseError(
+                            desc=task.single_value['nstaskstatus'],
+                            info=_("Task DN = '%s'" % task_dn))
+                time.sleep(1)
+                if time.time() > (start_time + 60):
+                   raise errors.TaskTimeout(task=_('Automember'), task_dn=task_dn)
+
+        return dict(
+            result=result,
+            summary=unicode(summary),
+            value=pkey_to_value(None, options))
