@@ -494,6 +494,23 @@ def host_is_master(ldap, fqdn):
         return
 
 
+def add_missing_object_class(ldap, objectclass, dn, entry_attrs=None, update=True):
+    """
+    Add object class if missing into entry. Fetches entry if not passed. Updates
+    the entry by default.
+
+    Returns the entry
+    """
+
+    if not entry_attrs:
+        entry_attrs = ldap.get_entry(dn, ['objectclass'])
+    if (objectclass.lower() not in (o.lower() for o in entry_attrs['objectclass'])):
+        entry_attrs['objectclass'].append(objectclass)
+        if update:
+            ldap.update_entry(entry_attrs)
+    return entry_attrs
+
+
 class LDAPObject(Object):
     """
     Object representing a LDAP entry.
@@ -1398,16 +1415,23 @@ class LDAPUpdate(LDAPQuery, crud.Update):
                 entry_attrs[self.obj.primary_key.name] = options['rename']
 
             if self.obj.rdn_is_primary_key and self.obj.primary_key.name in entry_attrs:
-                # RDN change
-                self._exc_wrapper(keys, options, ldap.update_entry_rdn)(
-                    entry_attrs.dn,
-                    RDN((self.obj.primary_key.name,
-                         entry_attrs[self.obj.primary_key.name])))
-                rdnkeys = keys[:-1] + (entry_attrs[self.obj.primary_key.name], )
-                entry_attrs.dn = self.obj.get_dn(*rdnkeys)
-                del entry_attrs[self.obj.primary_key.name]
-                options['rdnupdate'] = True
-                rdnupdate = True
+                try:
+                    # RDN change
+                    self._exc_wrapper(keys, options, ldap.update_entry_rdn)(
+                        entry_attrs.dn,
+                        RDN((self.obj.primary_key.name,
+                             entry_attrs[self.obj.primary_key.name])))
+
+                    rdnkeys = keys[:-1] + (entry_attrs[self.obj.primary_key.name], )
+                    entry_attrs.dn = self.obj.get_dn(*rdnkeys)
+                    options['rdnupdate'] = True
+                    rdnupdate = True
+                except errors.EmptyModlist:
+                    # Attempt to rename to the current name, ignore
+                    pass
+                finally:
+                    # Delete the primary_key from entry_attrs either way
+                    del entry_attrs[self.obj.primary_key.name]
 
             # Exception callbacks will need to test for options['rdnupdate']
             # to decide what to do. An EmptyModlist in this context doesn't
@@ -1971,8 +1995,10 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
                 time_limit=options.get('timelimit', None),
                 size_limit=options.get('sizelimit', None)
             )
-        except errors.NotFound:
+        except errors.EmptyResult:
             (entries, truncated) = ([], False)
+        except errors.NotFound:
+            self.api.Object[self.obj.parent_object].handle_not_found(*args[:-1])
 
         for callback in self.get_callbacks('post'):
             truncated = callback(self, ldap, entries, truncated, *args, **options)
@@ -2095,9 +2121,6 @@ class LDAPAddReverseMember(LDAPModReverseMember):
                 attrs_list.difference_update(self.obj.attribute_members)
             attrs_list = list(attrs_list)
 
-        # Pull the record as it is now so we can know how many members
-        # there are.
-        entry_start = self.api.Command[self.show_command](keys[-1])['result']
         completed = 0
         failed = {'member': {self.reverse_attr: []}}
         for attr in options.get(self.reverse_attr) or []:
@@ -2199,9 +2222,6 @@ class LDAPRemoveReverseMember(LDAPModReverseMember):
                 attrs_list.difference_update(self.obj.attribute_members)
             attrs_list = list(attrs_list)
 
-        # Pull the record as it is now so we can know how many members
-        # there are.
-        entry_start = self.api.Command[self.show_command](keys[-1])['result']
         completed = 0
         failed = {'member': {self.reverse_attr: []}}
         for attr in options.get(self.reverse_attr) or []:
