@@ -18,26 +18,27 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import platform
-import os
-import sys
 from nss.error import NSPRError
-import nss.nss as nss
-import netaddr
 import string
 
 from ipalib import api, errors, util
 from ipalib import Str, Flag, Bytes
 from ipalib.plugable import Registry
-from ipalib.plugins.baseldap import *
+from ipalib.plugins.baseldap import (LDAPQuery, LDAPObject, LDAPCreate,
+                                     LDAPDelete, LDAPUpdate, LDAPSearch,
+                                     LDAPRetrieve, LDAPAddMember,
+                                     LDAPRemoveMember, host_is_master,
+                                     pkey_to_value, add_missing_object_class)
 from ipalib.plugins.service import (split_principal, validate_certificate,
     set_certificate_attrs, ticket_flags_params, update_krbticketflags,
-    set_kerberos_attrs)
+    set_kerberos_attrs, rename_ipaallowedtoperform_from_ldap,
+    rename_ipaallowedtoperform_to_ldap)
 from ipalib.plugins.dns import (dns_container_exists, _record_types,
         add_records_for_host_validation, add_records_for_host,
         get_reverse_zone)
 from ipalib import _, ngettext
 from ipalib import x509
+from ipalib import output
 from ipalib.request import context
 from ipalib.util import (normalize_sshpubkey, validate_sshpubkey_no_options,
     convert_sshpubkey_post, validate_hostname)
@@ -102,13 +103,17 @@ EXAMPLES:
 """) + _("""
  Add a host that can manage this host's keytab and certificate:
    ipa host-add-managedby --hosts=test2 test
+""") + _("""
+ Allow user to create a keytab:
+   ipa host-allow-create-keytab test2 --users=tuser1
 """)
 
 register = Registry()
 
 # Characters to be used by random password generator
 # The set was chosen to avoid the need for escaping the characters by user
-host_pwd_chars=string.digits + string.ascii_letters + '_,.@+-='
+host_pwd_chars = string.digits + string.ascii_letters + '_,.@+-='
+
 
 def remove_fwd_ptr(ipaddr, host, domain, recordtype):
     api.log.debug('deleting ipaddr %s' % ipaddr)
@@ -118,34 +123,36 @@ def remove_fwd_ptr(ipaddr, host, domain, recordtype):
         # in case domain is in FQDN form with a trailing dot, we needn't add
         # another one, in case it has no trailing dot, dnsrecord-del will
         # normalize the entry
-        delkw = { 'ptrrecord' : "%s.%s" % (host, domain) }
+        delkw = {'ptrrecord': "%s.%s" % (host, domain)}
 
         api.Command['dnsrecord_del'](revzone, revname, **delkw)
     except errors.NotFound:
         pass
 
     try:
-        delkw = { recordtype : ipaddr }
+        delkw = {recordtype: ipaddr}
         api.Command['dnsrecord_del'](domain, host, **delkw)
     except errors.NotFound:
         pass
 
+
 def update_sshfp_record(zone, record, entry_attrs):
     if 'ipasshpubkey' not in entry_attrs:
         return
 
     pubkeys = entry_attrs['ipasshpubkey'] or ()
-    sshfps=[]
+    sshfps = []
     for pubkey in pubkeys:
         try:
             sshfp = SSHPublicKey(pubkey).fingerprint_dns_sha1()
-        except ValueError, UnicodeDecodeError:
+        except (ValueError, UnicodeDecodeError):
             continue
         if sshfp is not None:
             sshfps.append(sshfp)
+
         try:
             sshfp = SSHPublicKey(pubkey).fingerprint_dns_sha256()
-        except ValueError, UnicodeDecodeError:
+        except (ValueError, UnicodeDecodeError):
             continue
         if sshfp is not None:
             sshfps.append(sshfp)
@@ -155,6 +162,17 @@ def update_sshfp_record(zone, record, entry_attrs):
     except errors.EmptyModlist:
         pass
 
+
+def convert_ipaassignedidview_post(entry_attrs, options):
+    """
+    Converts the ID View DN to its name for the better looking output.
+    """
+
+    if 'ipaassignedidview' in entry_attrs and not options.get('raw'):
+        idview_name = entry_attrs.single_value['ipaassignedidview'][0].value
+        entry_attrs.single_value['ipaassignedidview'] = idview_name
+
+
 host_output_params = (
     Flag('has_keytab',
         label=_('Keytab'),
@@ -198,18 +216,50 @@ host_output_params = (
     Str('sshpubkeyfp*',
         label=_('SSH public key fingerprint'),
     ),
+    Str('ipaallowedtoperform_read_keys_user',
+        label=_('Users allowed to retrieve keytab'),
+    ),
+    Str('ipaallowedtoperform_read_keys_group',
+        label=_('Groups allowed to retrieve keytab'),
+    ),
+    Str('ipaallowedtoperform_read_keys_host',
+        label=_('Hosts allowed to retrieve keytab'),
+    ),
+    Str('ipaallowedtoperform_read_keys_hostgroup',
+        label=_('Host Groups allowed to retrieve keytab'),
+    ),
+    Str('ipaallowedtoperform_write_keys_user',
+        label=_('Users allowed to create keytab'),
+    ),
+    Str('ipaallowedtoperform_write_keys_group',
+        label=_('Groups allowed to create keytab'),
+    ),
+    Str('ipaallowedtoperform_write_keys_host',
+        label=_('Hosts allowed to create keytab'),
+    ),
+    Str('ipaallowedtoperform_write_keys_hostgroup',
+        label=_('Host Groups allowed to create keytab'),
+    ),
+    Str('ipaallowedtoperform_read_keys',
+        label=_('Failed allowed to retrieve keytab'),
+    ),
+    Str('ipaallowedtoperform_write_keys',
+        label=_('Failed allowed to create keytab'),
+    ),
 )
 
+
 def validate_ipaddr(ugettext, ipaddr):
     """
     Verify that we have either an IPv4 or IPv6 address.
     """
     try:
-        ip = CheckedIPAddress(ipaddr, match_local=False)
+        CheckedIPAddress(ipaddr, match_local=False)
     except Exception, e:
         return unicode(e)
     return None
 
+
 def normalize_hostname(hostname):
     """Use common fqdn form without the trailing dot"""
     if hostname.endswith(u'.'):
@@ -217,15 +267,16 @@ def normalize_hostname(hostname):
     hostname = hostname.lower()
     return hostname
 
+
 def _hostname_validator(ugettext, value):
     try:
         validate_hostname(value)
     except ValueError, e:
-        return _('invalid domain-name: %s') \
-            % unicode(e)
+        return _('invalid domain-name: %s') % unicode(e)
 
     return None
 
+
 @register()
 class host(LDAPObject):
     """
@@ -235,17 +286,18 @@ class host(LDAPObject):
     object_name = _('host')
     object_name_plural = _('hosts')
     object_class = ['ipaobject', 'nshost', 'ipahost', 'pkiuser', 'ipaservice']
+    possible_objectclasses = ['ipaallowedoperations']
     permission_filter_objectclasses = ['ipahost']
     # object_class_config = 'ipahostobjectclasses'
     search_attributes = [
         'fqdn', 'description', 'l', 'nshostlocation', 'krbprincipalname',
-        'nshardwareplatform', 'nsosversion', 'managedby'
+        'nshardwareplatform', 'nsosversion', 'managedby', 'ipaallowedtoperform'
     ]
     default_attributes = [
         'fqdn', 'description', 'l', 'nshostlocation', 'krbprincipalname',
         'nshardwareplatform', 'nsosversion', 'usercertificate', 'memberof',
         'managedby', 'memberindirect', 'memberofindirect', 'macaddress',
-        'userclass'
+        'userclass', 'ipaallowedtoperform', 'ipaassignedidview',
     ]
     uuid_attribute = 'ipauniqueid'
     attribute_members = {
@@ -255,6 +307,8 @@ class host(LDAPObject):
         'managing': ['host'],
         'memberofindirect': ['hostgroup', 'netgroup', 'role', 'hbacrule',
         'sudorule'],
+        'ipaallowedtoperform_read_keys': ['user', 'group', 'host', 'hostgroup'],
+        'ipaallowedtoperform_write_keys': ['user', 'group', 'host', 'hostgroup'],
     }
     bindable = True
     relationships = {
@@ -262,6 +316,8 @@ class host(LDAPObject):
         'enrolledby': ('Enrolled by', 'enroll_by_', 'not_enroll_by_'),
         'managedby': ('Managed by', 'man_by_', 'not_man_by_'),
         'managing': ('Managing', 'man_', 'not_man_'),
+        'ipaallowedtoperform_read_keys': ('Allow to retrieve keytab by', 'retrieve_keytab_by_', 'not_retrieve_keytab_by_'),
+        'ipaallowedtoperform_write_keys': ('Allow to create keytab by', 'write_keytab_by_', 'not_write_keytab_by'),
     }
     password_attributes = [('userpassword', 'has_password'),
                            ('krbprincipalkey', 'has_keytab')]
@@ -276,7 +332,7 @@ class host(LDAPObject):
                 'krbprincipalname', 'l', 'macaddress', 'nshardwareplatform',
                 'nshostlocation', 'nsosversion', 'objectclass',
                 'serverhostname', 'usercertificate', 'userclass',
-                'enrolledby', 'managedby',
+                'enrolledby', 'managedby', 'ipaassignedidview',
                 'krbprincipalname', 'krbcanonicalname', 'krbprincipalaliases',
                 'krbprincipalexpiration', 'krbpasswordexpiration',
                 'krblastpwdchange',
@@ -338,11 +394,19 @@ class host(LDAPObject):
             ],
             'default_privileges': {'Host Administrators', 'Host Enrollment'},
         },
+        'System: Manage Host Keytab Permissions': {
+            'ipapermright': {'read', 'search', 'compare', 'write'},
+            'ipapermdefaultattr': {
+                'ipaallowedtoperform;write_keys',
+                'ipaallowedtoperform;read_keys', 'objectclass'
+            },
+            'default_privileges': {'Host Administrators'},
+        },
         'System: Modify Hosts': {
             'ipapermright': {'write'},
             'ipapermdefaultattr': {
                 'description', 'l', 'nshardwareplatform', 'nshostlocation',
-                'nsosversion', 'macaddress', 'userclass',
+                'nsosversion', 'macaddress', 'userclass', 'ipaassignedidview',
             },
             'replaces': [
                 '(targetattr = "description || l || nshostlocation || nshardwareplatform || nsosversion")(target = "ldap:///fqdn=*,cn=computers,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Hosts";allow (write) groupdn = "ldap:///cn=Modify Hosts,cn=permissions,cn=pbac,$SUFFIX";)',
@@ -441,7 +505,8 @@ class host(LDAPObject):
         Str('macaddress*',
             normalizer=lambda value: value.upper(),
             pattern='^([a-fA-F0-9]{2}[:|\-]?){5}[a-fA-F0-9]{2}$',
-            pattern_errmsg='Must be of the form HH:HH:HH:HH:HH:HH, where each H is a hexadecimal character.',
+            pattern_errmsg=('Must be of the form HH:HH:HH:HH:HH:HH, where '
+                            'each H is a hexadecimal character.'),
             csv=True,
             label=_('MAC address'),
             doc=_('Hardware MAC address(es) on this host'),
@@ -459,6 +524,10 @@ class host(LDAPObject):
             doc=_('Host category (semantics placed on this attribute are for '
                   'local interpretation)'),
         ),
+        Str('ipaassignedidview?',
+            label=_('Assigned ID View'),
+            flags=['no_option'],
+        ),
     ) + ticket_flags_params
 
     def get_dn(self, *keys, **options):
@@ -514,7 +583,6 @@ class host(LDAPObject):
                 entry_attrs['memberofindirect'].remove(member)
 
 
-
 @register()
 class host_add(LDAPCreate):
     __doc__ = _('Add a new host.')
@@ -620,6 +688,7 @@ class host_add(LDAPCreate):
             )
         set_certificate_attrs(entry_attrs)
         set_kerberos_attrs(entry_attrs, options)
+        rename_ipaallowedtoperform_from_ldap(entry_attrs, options)
 
         if options.get('all', False):
             entry_attrs['managing'] = self.obj.get_managed_hosts(dn)
@@ -634,7 +703,6 @@ class host_add(LDAPCreate):
         return dn
 
 
-
 @register()
 class host_del(LDAPDelete):
     __doc__ = _('Delete a host.')
@@ -642,7 +710,7 @@ class host_del(LDAPDelete):
     msg_summary = _('Deleted host "%(value)s"')
     member_attributes = ['managedby']
 
-    takes_options = (
+    takes_options = LDAPDelete.takes_options + (
         Flag('updatedns?',
             doc=_('Remove entries from DNS'),
             default=False,
@@ -700,7 +768,8 @@ class host_del(LDAPDelete):
                                    domain, 'aaaarecord')
                 else:
                     # Try to delete all other record types too
-                    _attribute_types = [str('%srecord' % t.lower()) for t in _record_types]
+                    _attribute_types = [str('%srecord' % t.lower())
+                                        for t in _record_types]
                     for attr in _attribute_types:
                         if attr not in ['arecord', 'aaaarecord'] and attr in record:
                             for i in xrange(len(record[attr])):
@@ -712,7 +781,7 @@ class host_del(LDAPDelete):
                                             **delkw)
                             break
 
-        if self.api.env.enable_ra:
+        if self.api.Command.ca_is_enabled()['result']:
             try:
                 entry_attrs = ldap.get_entry(dn, ['usercertificate'])
             except errors.NotFound:
@@ -746,7 +815,6 @@ class host_del(LDAPDelete):
         return dn
 
 
-
 @register()
 class host_mod(LDAPUpdate):
     __doc__ = _('Modify information about a host.')
@@ -776,7 +844,9 @@ class host_mod(LDAPUpdate):
             entry = {}
             self.obj.get_password_attributes(ldap, dn, entry)
             if not entry['has_password'] and entry['has_keytab']:
-                raise errors.ValidationError(name='password', error=_('Password cannot be set on enrolled host.'))
+                raise errors.ValidationError(
+                    name='password',
+                    error=_('Password cannot be set on enrolled host.'))
 
         # Once a principal name is set it cannot be changed
         if 'cn' in entry_attrs:
@@ -796,7 +866,7 @@ class host_mod(LDAPUpdate):
                 entry_attrs['objectclass'] = obj_classes
         cert = x509.normalize_certificate(entry_attrs.get('usercertificate'))
         if cert:
-            if self.api.env.enable_ra:
+            if self.api.Command.ca_is_enabled()['result']:
                 x509.verify_cert_subject(ldap, keys[-1], cert)
                 entry_attrs_old = ldap.get_entry(dn, ['usercertificate'])
                 oldcert = entry_attrs_old.single_value.get('usercertificate')
@@ -878,6 +948,7 @@ class host_mod(LDAPUpdate):
             entry_attrs['randompassword'] = unicode(getattr(context, 'randompassword'))
         set_certificate_attrs(entry_attrs)
         set_kerberos_attrs(entry_attrs, options)
+        rename_ipaallowedtoperform_from_ldap(entry_attrs, options)
         self.obj.get_password_attributes(ldap, dn, entry_attrs)
         if entry_attrs['has_password']:
             # If an OTP is set there is no keytab, at least not one
@@ -890,11 +961,11 @@ class host_mod(LDAPUpdate):
         self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
 
         convert_sshpubkey_post(ldap, dn, entry_attrs)
+        convert_ipaassignedidview_post(entry_attrs, options)
 
         return dn
 
 
-
 @register()
 class host_find(LDAPSearch):
     __doc__ = _('Search for hosts.')
@@ -949,7 +1020,8 @@ class host_find(LDAPSearch):
             for target_hosts, filter_op in ((hosts, ldap.MATCH_ANY),
                                             (not_hosts, ldap.MATCH_NONE)):
                 hosts_avas = [DN(host)[0][0] for host in target_hosts]
-                hosts_filters = [ldap.make_filter_from_attr(ava.attr, ava.value) for ava in hosts_avas]
+                hosts_filters = [ldap.make_filter_from_attr(ava.attr, ava.value)
+                                 for ava in hosts_avas]
                 hosts_filter = ldap.combine_filters(hosts_filters, filter_op)
 
                 filter = ldap.combine_filters(
@@ -964,6 +1036,7 @@ class host_find(LDAPSearch):
         for entry_attrs in entries:
             set_certificate_attrs(entry_attrs)
             set_kerberos_attrs(entry_attrs, options)
+            rename_ipaallowedtoperform_from_ldap(entry_attrs, options)
             self.obj.get_password_attributes(ldap, entry_attrs.dn, entry_attrs)
             self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
             if entry_attrs['has_password']:
@@ -975,11 +1048,11 @@ class host_find(LDAPSearch):
                 entry_attrs['managing'] = self.obj.get_managed_hosts(entry_attrs.dn)
 
             convert_sshpubkey_post(ldap, entry_attrs.dn, entry_attrs)
+            convert_ipaassignedidview_post(entry_attrs, options)
 
         return truncated
 
 
-
 @register()
 class host_show(LDAPRetrieve):
     __doc__ = _('Display information about a host.')
@@ -1003,6 +1076,7 @@ class host_show(LDAPRetrieve):
 
         set_certificate_attrs(entry_attrs)
         set_kerberos_attrs(entry_attrs, options)
+        rename_ipaallowedtoperform_from_ldap(entry_attrs, options)
 
         if options.get('all', False):
             entry_attrs['managing'] = self.obj.get_managed_hosts(dn)
@@ -1010,6 +1084,7 @@ class host_show(LDAPRetrieve):
         self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
 
         convert_sshpubkey_post(ldap, dn, entry_attrs)
+        convert_ipaassignedidview_post(entry_attrs, options)
 
         return dn
 
@@ -1027,7 +1102,6 @@ class host_show(LDAPRetrieve):
             return super(host_show, self).forward(*keys, **options)
 
 
-
 @register()
 class host_disable(LDAPQuery):
     __doc__ = _('Disable the Kerberos key, SSL certificate and all services of a host.')
@@ -1076,7 +1150,7 @@ class host_disable(LDAPQuery):
             self.obj.handle_not_found(*keys)
         cert = entry_attrs.single_value.get('usercertificate')
         if cert:
-            if self.api.env.enable_ra:
+            if self.api.Command.ca_is_enabled()['result']:
                 cert = x509.normalize_certificate(cert)
                 try:
                     serial = unicode(x509.get_serial_number(cert, x509.DER))
@@ -1139,7 +1213,6 @@ class host_add_managedby(LDAPAddMember):
         return (completed, dn)
 
 
-
 @register()
 class host_remove_managedby(LDAPRemoveMember):
     __doc__ = _('Remove hosts that can manage this host.')
@@ -1152,3 +1225,76 @@ class host_remove_managedby(LDAPRemoveMember):
         self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
         return (completed, dn)
 
+
+@register()
+class host_allow_retrieve_keytab(LDAPAddMember):
+    __doc__ = _('Allow users, groups, hosts or host groups to retrieve a keytab'
+                ' of this host.')
+    member_attributes = ['ipaallowedtoperform_read_keys']
+    has_output_params = LDAPAddMember.has_output_params + host_output_params
+
+    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
+        rename_ipaallowedtoperform_to_ldap(found)
+        rename_ipaallowedtoperform_to_ldap(not_found)
+        add_missing_object_class(ldap, u'ipaallowedoperations', dn)
+        return dn
+
+    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
+        rename_ipaallowedtoperform_from_ldap(entry_attrs, options)
+        rename_ipaallowedtoperform_from_ldap(failed, options)
+        return (completed, dn)
+
+
+@register()
+class host_disallow_retrieve_keytab(LDAPRemoveMember):
+    __doc__ = _('Disallow users, groups, hosts or host groups to retrieve a '
+                'keytab of this host.')
+    member_attributes = ['ipaallowedtoperform_read_keys']
+    has_output_params = LDAPRemoveMember.has_output_params + host_output_params
+
+    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
+        rename_ipaallowedtoperform_to_ldap(found)
+        rename_ipaallowedtoperform_to_ldap(not_found)
+        return dn
+
+    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
+        rename_ipaallowedtoperform_from_ldap(entry_attrs, options)
+        rename_ipaallowedtoperform_from_ldap(failed, options)
+        return (completed, dn)
+
+
+@register()
+class host_allow_create_keytab(LDAPAddMember):
+    __doc__ = _('Allow users, groups, hosts or host groups to create a keytab '
+                'of this host.')
+    member_attributes = ['ipaallowedtoperform_write_keys']
+    has_output_params = LDAPAddMember.has_output_params + host_output_params
+
+    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
+        rename_ipaallowedtoperform_to_ldap(found)
+        rename_ipaallowedtoperform_to_ldap(not_found)
+        add_missing_object_class(ldap, u'ipaallowedoperations', dn)
+        return dn
+
+    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
+        rename_ipaallowedtoperform_from_ldap(entry_attrs, options)
+        rename_ipaallowedtoperform_from_ldap(failed, options)
+        return (completed, dn)
+
+
+@register()
+class host_disallow_create_keytab(LDAPRemoveMember):
+    __doc__ = _('Disallow users, groups, hosts or host groups to create a '
+                'keytab of this host.')
+    member_attributes = ['ipaallowedtoperform_write_keys']
+    has_output_params = LDAPRemoveMember.has_output_params + host_output_params
+
+    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
+        rename_ipaallowedtoperform_to_ldap(found)
+        rename_ipaallowedtoperform_to_ldap(not_found)
+        return dn
+
+    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
+        rename_ipaallowedtoperform_from_ldap(entry_attrs, options)
+        rename_ipaallowedtoperform_from_ldap(failed, options)
+        return (completed, dn)
