@@ -20,8 +20,9 @@
 import re
 
 from ipalib import api, errors, output
-from ipalib import Command, Password, Str, Flag, StrEnum, DNParam
+from ipalib import Command, Password, Str, Flag, StrEnum, DNParam, File
 from ipalib.cli import to_cli
+from ipalib.plugable import Registry
 from ipalib.plugins.user import NO_UPG_MAGIC
 if api.env.in_server and api.env.context in ['lite', 'server']:
     try:
@@ -30,7 +31,9 @@ if api.env.in_server and api.env.context in ['lite', 'server']:
         raise e
 from ipalib import _
 from ipapython.dn import DN
+from ipapython.ipautil import write_tmp_file
 import datetime
+from ipaplatform.paths import paths
 
 __doc__ = _("""
 Migration to IPA
@@ -125,6 +128,8 @@ for each user added plus a summary when the default user group is
 updated.
 """)
 
+register = Registry()
+
 # USER MIGRATION CALLBACKS AND VARS
 
 _krb_err_msg = _('Kerberos principal %s already exists. Use \'ipa user-mod\' to set it manually.')
@@ -135,7 +140,6 @@ _dn_err_msg = _('Malformed DN')
 
 _supported_schemas = (u'RFC2307bis', u'RFC2307')
 
-_compat_dn = DN(('cn', 'Schema Compatibility'), ('cn', 'plugins'), ('cn', 'config'))
 
 def _pre_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwargs):
     assert isinstance(dn, DN)
@@ -157,7 +161,7 @@ def _pre_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwargs
                          % (entry_attrs['gidnumber'][0], pkey))
         elif entry_attrs['gidnumber'][0] not in valid_gids:
             try:
-                (remote_dn, remote_entry) = ds_ldap.find_entry_by_attr(
+                remote_entry = ds_ldap.find_entry_by_attr(
                     'gidnumber', entry_attrs['gidnumber'][0], 'posixgroup',
                     [''], search_bases['group']
                 )
@@ -181,19 +185,18 @@ def _pre_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwargs
     # fill in required attributes by IPA
     entry_attrs['ipauniqueid'] = 'autogenerate'
     if 'homedirectory' not in entry_attrs:
-        homes_root = config.get('ipahomesrootdir', ('/home', ))[0]
+        homes_root = config.get('ipahomesrootdir', (paths.HOME_DIR, ))[0]
         home_dir = '%s/%s' % (homes_root, pkey)
         home_dir = home_dir.replace('//', '/').rstrip('/')
         entry_attrs['homedirectory'] = home_dir
 
     if 'loginshell' not in entry_attrs:
-        default_shell = config.get('ipadefaultloginshell', ['/bin/sh'])[0]
+        default_shell = config.get('ipadefaultloginshell', [paths.SH])[0]
         entry_attrs.setdefault('loginshell', default_shell)
 
     # do not migrate all attributes
-    for attr in entry_attrs.keys():
-        if attr in attr_blacklist:
-            del entry_attrs[attr]
+    for attr in attr_blacklist:
+        entry_attrs.pop(attr, None)
 
     # do not migrate all object classes
     if 'objectclass' in entry_attrs:
@@ -237,7 +240,7 @@ def _pre_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwargs
                                 pkey, value, type(value), attr, e)
                         continue
                 try:
-                    (remote_dn, remote_entry) = ds_ldap.get_entry(value, [api.Object.user.primary_key.name, api.Object.group.primary_key.name])
+                    remote_entry = ds_ldap.get_entry(value, [api.Object.user.primary_key.name, api.Object.group.primary_key.name])
                 except errors.NotFound:
                     api.log.warn('%s: attribute %s refers to non-existent entry %s' % (pkey, attr, value))
                     continue
@@ -265,17 +268,18 @@ def _pre_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwargs
 def _post_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx):
     assert isinstance(dn, DN)
 
-    _update_default_group(ldap, pkey, config, ctx, False)
+    _update_default_group(ldap, ctx, False)
 
     if 'description' in entry_attrs and NO_UPG_MAGIC in entry_attrs['description']:
         entry_attrs['description'].remove(NO_UPG_MAGIC)
-        update_attrs = dict(description = entry_attrs['description'])
         try:
-            ldap.update_entry(dn, update_attrs)
+            update_attrs = ldap.get_entry(dn, ['description'])
+            update_attrs['description'] = entry_attrs['description']
+            ldap.update_entry(update_attrs)
         except (errors.EmptyModlist, errors.NotFound):
             pass
 
-def _update_default_group(ldap, pkey, config, ctx, force):
+def _update_default_group(ldap, ctx, force):
     migrate_cnt = ctx['migrate_cnt']
     group_dn = ctx['def_group_dn']
 
@@ -286,29 +290,32 @@ def _update_default_group(ldap, pkey, config, ctx, force):
         searchfilter = "(&(objectclass=posixAccount)(!(memberof=%s)))" % group_dn
         try:
             (result, truncated) = ldap.find_entries(searchfilter,
-                [''], api.env.container_user, scope=ldap.SCOPE_SUBTREE,
-                time_limit = -1)
+                [''], DN(api.env.container_user, api.env.basedn),
+                scope=ldap.SCOPE_SUBTREE, time_limit = -1)
         except errors.NotFound:
+            api.log.debug('All users have default group set')
             return
         new_members = []
-        (group_dn, group_entry_attrs) = ldap.get_entry(group_dn, ['member'])
+        group_entry_attrs = ldap.get_entry(group_dn, ['member'])
+        existing_members = set(group_entry_attrs.get('member', []))
         for m in result:
-            if m[0] not in group_entry_attrs.get('member', []):
-                new_members.append(m[0])
-        if len(new_members) > 0:
-            members = group_entry_attrs.get('member', [])
+            if m.dn not in existing_members:
+                new_members.append(m.dn)
+
+        if new_members:
+            members = group_entry_attrs.setdefault('member', [])
             members.extend(new_members)
-            group_entry_attrs['member'] = members
 
             try:
-                ldap.update_entry(group_dn, group_entry_attrs)
+                ldap.update_entry(group_entry_attrs)
             except errors.EmptyModlist:
                 pass
 
         e = datetime.datetime.now()
         d = e - s
         mode = " (forced)" if force else ""
-        api.log.debug('Adding %d users to group%s duration %s' % (len(new_members), mode, d))
+        api.log.debug('Adding %d users to group%s duration %s',
+                len(new_members), mode, d)
 
 # GROUP MIGRATION CALLBACKS AND VARS
 
@@ -384,9 +391,8 @@ def _pre_migrate_group(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwarg
         raise ValueError('Schema %s not supported' % schema)
 
     # do not migrate all attributes
-    for attr in entry_attrs.keys():
-        if attr in attr_blacklist:
-            del entry_attrs[attr]
+    for attr in attr_blacklist:
+        entry_attrs.pop(attr, None)
 
     # do not migrate all object classes
     if 'objectclass' in entry_attrs:
@@ -405,8 +411,9 @@ def _group_exc_callback(ldap, dn, entry_attrs, exc, options):
         if options.get('groupoverwritegid', False) and \
            entry_attrs.get('gidnumber') is not None:
             try:
-                new_entry_attrs = {'gidnumber':entry_attrs['gidnumber']}
-                ldap.update_entry(dn, new_entry_attrs)
+                new_entry_attrs = ldap.get_entry(dn, ['gidnumber'])
+                new_entry_attrs['gidnumber'] = entry_attrs['gidnumber']
+                ldap.update_entry(new_entry_attrs)
             except errors.EmptyModlist:
                 # no change to the GID
                 pass
@@ -435,6 +442,7 @@ def validate_ldapuri(ugettext, ldapuri):
         raise errors.ValidationError(name='ldap_uri', error=err_msg)
 
 
+@register()
 class migrate_ds(Command):
     __doc__ = _('Migrate users and groups from DS to IPA.')
 
@@ -593,6 +601,12 @@ class migrate_ds(Command):
             doc=_('Allows migration despite the usage of compat plugin'),
             default=False,
         ),
+        File('cacertfile?',
+            cli_name='ca_cert_file',
+            label=_('CA certificate'),
+            doc=_('Load CA certificate of LDAP server from FILE'),
+            default=None
+        ),
     )
 
     has_output = (
@@ -740,7 +754,7 @@ can use their Kerberos accounts.''')
                 def_group = config.get('ipadefaultprimarygroup')
                 context['def_group_dn'] = api.Object.group.get_dn(def_group)
                 try:
-                    (g_dn, g_attrs) = ldap.get_entry(context['def_group_dn'], ['gidnumber', 'cn'])
+                    g_attrs = ldap.get_entry(context['def_group_dn'], ['gidnumber', 'cn'])
                 except errors.NotFound:
                     error_msg = _('Default group for new users not found')
                     raise errors.NotFound(reason=error_msg)
@@ -753,20 +767,11 @@ can use their Kerberos accounts.''')
             invalid_gids = []
             migrate_cnt = 0
             context['migrate_cnt'] = 0
-            for (dn, entry_attrs) in entries:
+            for entry_attrs in entries:
                 context['migrate_cnt'] = migrate_cnt
                 s = datetime.datetime.now()
-                if dn is None:  # LDAP search reference
-                    failed[ldap_obj_name][entry_attrs[0]] = unicode(_ref_err_msg)
-                    continue
 
-                try:
-                    dn = DN(dn)
-                except ValueError:
-                    failed[ldap_obj_name][dn] = unicode(_dn_err_msg)
-                    continue
-
-                ava = dn[0][0]
+                ava = entry_attrs.dn[0][0]
                 if ava.attr == ldap_obj.primary_key.name:
                     # In case if pkey attribute is in the migrated object DN
                     # and the original LDAP is multivalued, make sure that
@@ -778,8 +783,7 @@ can use their Kerberos accounts.''')
                 if pkey in exclude:
                     continue
 
-                dn = ldap_obj.get_dn(pkey)
-                assert isinstance(dn, DN)
+                entry_attrs.dn = ldap_obj.get_dn(pkey)
                 entry_attrs['objectclass'] = list(
                     set(
                         config.get(
@@ -792,28 +796,29 @@ can use their Kerberos accounts.''')
                 callback = self.migrate_objects[ldap_obj_name]['pre_callback']
                 if callable(callback):
                     try:
-                        dn = callback(
-                            ldap, pkey, dn, entry_attrs, failed[ldap_obj_name],
-                            config, context, schema = options['schema'],
-                            search_bases = search_bases,
-                            valid_gids = valid_gids,
-                            invalid_gids = invalid_gids,
+                        entry_attrs.dn = callback(
+                            ldap, pkey, entry_attrs.dn, entry_attrs,
+                            failed[ldap_obj_name], config, context,
+                            schema=options['schema'],
+                            search_bases=search_bases,
+                            valid_gids=valid_gids,
+                            invalid_gids=invalid_gids,
                             **blacklists
                         )
-                        assert isinstance(dn, DN)
-                        if not dn:
+                        if not entry_attrs.dn:
                             continue
                     except errors.NotFound, e:
                         failed[ldap_obj_name][pkey] = unicode(e.reason)
                         continue
 
                 try:
-                    ldap.add_entry(dn, entry_attrs)
+                    ldap.add_entry(entry_attrs)
                 except errors.ExecutionError, e:
                     callback = self.migrate_objects[ldap_obj_name]['exc_callback']
                     if callable(callback):
                         try:
-                            callback(ldap, dn, entry_attrs, e, options)
+                            callback(
+                                ldap, entry_attrs.dn, entry_attrs, e, options)
                         except errors.ExecutionError, e:
                             failed[ldap_obj_name][pkey] = unicode(e)
                             continue
@@ -826,9 +831,8 @@ can use their Kerberos accounts.''')
                 callback = self.migrate_objects[ldap_obj_name]['post_callback']
                 if callable(callback):
                     callback(
-                        ldap, pkey, dn, entry_attrs, failed[ldap_obj_name],
-                        config, context,
-                    )
+                        ldap, pkey, entry_attrs.dn, entry_attrs,
+                        failed[ldap_obj_name], config, context)
                 e = datetime.datetime.now()
                 d = e - s
                 total_dur = e - migration_start
@@ -837,15 +841,14 @@ can use their Kerberos accounts.''')
                     api.log.info("%d %ss migrated. %s elapsed." % (migrate_cnt, ldap_obj_name, total_dur))
                 api.log.debug("%d %ss migrated, duration: %s (total %s)" % (migrate_cnt, ldap_obj_name, d, total_dur))
 
-        _update_default_group(ldap, pkey, config, context, True)
+        _update_default_group(ldap, context, True)
 
         return (migrated, failed)
 
     def execute(self, ldapuri, bindpw, **options):
         ldap = self.api.Backend.ldap2
         self.normalize_options(options)
-
-        config = ldap.get_ipa_config()[1]
+        config = ldap.get_ipa_config()
 
         ds_base_dn = options.get('basedn')
         if ds_base_dn is not None:
@@ -857,16 +860,26 @@ can use their Kerberos accounts.''')
 
         # connect to DS
         ds_ldap = ldap2(shared_instance=False, ldap_uri=ldapuri, base_dn='')
-        ds_ldap.connect(bind_dn=options['binddn'], bind_pw=bindpw)
+
+        cacert = None
+        if options.get('cacertfile') is not None:
+            #store CA cert into file
+            tmp_ca_cert_f = write_tmp_file(options['cacertfile'])
+            cacert = tmp_ca_cert_f.name
+
+            #start TLS connection
+            ds_ldap.connect(bind_dn=options['binddn'], bind_pw=bindpw,
+                tls_cacertfile=cacert)
+
+            tmp_ca_cert_f.close()
+        else:
+            ds_ldap.connect(bind_dn=options['binddn'], bind_pw=bindpw)
 
         #check whether the compat plugin is enabled
         if not options.get('compat'):
             try:
-                (dn,check_compat) = ldap.get_entry(_compat_dn)
-                assert isinstance(dn, DN)
-                if check_compat is not None and \
-                        check_compat.get('nsslapd-pluginenabled', [''])[0].lower() == 'on':
-                    return dict(result={}, failed={}, enabled=True, compat=False)
+                ldap.get_entry(DN(('cn', 'compat'), (api.env.basedn)))
+                return dict(result={}, failed={}, enabled=True, compat=False)
             except errors.NotFound:
                 pass
 
@@ -876,12 +889,12 @@ can use their Kerberos accounts.''')
                 '', ['namingcontexts', 'defaultnamingcontext'], DN(''),
                 ds_ldap.SCOPE_BASE, size_limit=-1, time_limit=0,
             )
-            if 'defaultnamingcontext' in entries[0][1]:
-                ds_base_dn = DN(entries[0][1]['defaultnamingcontext'][0])
+            if 'defaultnamingcontext' in entries[0]:
+                ds_base_dn = DN(entries[0]['defaultnamingcontext'][0])
                 assert isinstance(ds_base_dn, DN)
             else:
                 try:
-                    ds_base_dn = DN(entries[0][1]['namingcontexts'][0])
+                    ds_base_dn = DN(entries[0]['namingcontexts'][0])
                     assert isinstance(ds_base_dn, DN)
                 except (IndexError, KeyError), e:
                     raise StandardError(str(e))
@@ -901,6 +914,7 @@ can use their Kerberos accounts.''')
         if not result['compat']:
             textui.print_plain("The compat plug-in is enabled. This can increase the memory requirements during migration. Disable the compat plug-in with \'ipa-compat-manage disable\' or re-run this script with \'--with-compat\' option.")
             return 1
+        any_migrated = any(result['result'].values())
         textui.print_plain('Migrated:')
         textui.print_entry1(
             result['result'], attr_order=self.migrate_order,
@@ -913,6 +927,9 @@ can use their Kerberos accounts.''')
                 one_value_per_line=True,
             )
         textui.print_plain('-' * len(self.name))
+        if not any_migrated:
+            textui.print_plain('No users/groups were migrated from %s' %
+                               ldapuri)
+            return 1
         textui.print_plain(unicode(self.pwd_migration_msg))
 
-api.register(migrate_ds)
