@@ -61,6 +61,8 @@ EXAMPLES:
 
 register = Registry()
 
+topic = ('otp', _('One time password commands'))
+
 TOKEN_TYPES = {
     u'totp': ['ipatokentotpclockoffset', 'ipatokentotptimestep'],
     u'hotp': ['ipatokenhotpcounter']
@@ -100,14 +102,26 @@ def _convert_owner(userobj, entry_attrs, options):
 
 def _normalize_owner(userobj, entry_attrs):
     owner = entry_attrs.get('ipatokenowner', None)
-    if owner is not None:
-        entry_attrs['ipatokenowner'] = userobj.get_dn(owner)
+    if owner:
+        try:
+            entry_attrs['ipatokenowner'] = userobj._normalize_manager(owner)[0]
+        except NotFound:
+            userobj.handle_not_found(owner)
 
 def _check_interval(not_before, not_after):
     if not_before and not_after:
         return not_before <= not_after
     return True
 
+def _set_token_type(entry_attrs, **options):
+    klasses = [x.lower() for x in entry_attrs.get('objectclass', [])]
+    for ttype in TOKEN_TYPES.keys():
+        cls = 'ipatoken' + ttype
+        if cls.lower() in klasses:
+            entry_attrs['type'] = ttype.upper()
+
+    if not options.get('all', False) or options.get('pkey_only', False):
+        entry_attrs.pop('objectclass', None)
 
 @register()
 class otptoken(LDAPObject):
@@ -144,56 +158,61 @@ class otptoken(LDAPObject):
         ),
         StrEnum('type?',
             label=_('Type'),
+            doc=_('Type of the token'),
             default=u'totp',
             autofill=True,
-            values=tuple(TOKEN_TYPES.keys()),
+            values=tuple(TOKEN_TYPES.keys() + [x.upper() for x in TOKEN_TYPES]),
             flags=('virtual_attribute', 'no_update'),
         ),
         Str('description?',
             cli_name='desc',
             label=_('Description'),
+            doc=_('Token description (informational only)'),
         ),
         Str('ipatokenowner?',
             cli_name='owner',
             label=_('Owner'),
+            doc=_('Assigned user of the token (default: self)'),
         ),
         Str('managedby_user?',
             label=_('Manager'),
+            doc=_('Assigned manager of the token (default: self)'),
             flags=['no_create', 'no_update', 'no_search'],
         ),
         Bool('ipatokendisabled?',
             cli_name='disabled',
-            label=_('Disabled state')
+            label=_('Disabled'),
+            doc=_('Mark the token as disabled (default: false)')
         ),
         DateTime('ipatokennotbefore?',
             cli_name='not_before',
             label=_('Validity start'),
+            doc=_('First date/time the token can be used'),
         ),
         DateTime('ipatokennotafter?',
             cli_name='not_after',
             label=_('Validity end'),
+            doc=_('Last date/time the token can be used'),
         ),
         Str('ipatokenvendor?',
             cli_name='vendor',
             label=_('Vendor'),
-            default=u'FreeIPA',
-            autofill=True,
+            doc=_('Token vendor name (informational only)'),
         ),
         Str('ipatokenmodel?',
             cli_name='model',
             label=_('Model'),
-            default_from=lambda type: type,
-            autofill=True,
+            doc=_('Token model (informational only)'),
         ),
         Str('ipatokenserial?',
             cli_name='serial',
             label=_('Serial'),
-            default_from=lambda id: id,
-            autofill=True,
+            doc=_('Token serial (informational only)'),
         ),
         OTPTokenKey('ipatokenotpkey?',
             cli_name='key',
             label=_('Key'),
+            doc=_('Token secret (Base32; default: random)'),
             default_from=lambda: os.urandom(KEY_LENGTH),
             autofill=True,
             flags=('no_display', 'no_update', 'no_search'),
@@ -201,6 +220,7 @@ class otptoken(LDAPObject):
         StrEnum('ipatokenotpalgorithm?',
             cli_name='algo',
             label=_('Algorithm'),
+            doc=_('Token hash algorithm'),
             default=u'sha1',
             autofill=True,
             flags=('no_update'),
@@ -209,6 +229,7 @@ class otptoken(LDAPObject):
         IntEnum('ipatokenotpdigits?',
             cli_name='digits',
             label=_('Digits'),
+            doc=_('Number of digits each token code will have'),
             values=(6, 8),
             default=6,
             autofill=True,
@@ -217,6 +238,7 @@ class otptoken(LDAPObject):
         Int('ipatokentotpclockoffset?',
             cli_name='offset',
             label=_('Clock offset'),
+            doc=_('TOTP token / FreeIPA server time difference'),
             default=0,
             autofill=True,
             flags=('no_update'),
@@ -224,6 +246,7 @@ class otptoken(LDAPObject):
         Int('ipatokentotptimestep?',
             cli_name='interval',
             label=_('Clock interval'),
+            doc=_('Length of TOTP token code validity'),
             default=30,
             autofill=True,
             minvalue=5,
@@ -232,6 +255,7 @@ class otptoken(LDAPObject):
         Int('ipatokenhotpcounter?',
             cli_name='counter',
             label=_('Counter'),
+            doc=_('Initial counter for the HOTP token'),
             default=0,
             autofill=True,
             minvalue=0,
@@ -246,7 +270,8 @@ class otptoken_add(LDAPCreate):
     msg_summary = _('Added OTP token "%(value)s"')
 
     takes_options = LDAPCreate.takes_options + (
-        Flag('qrcode?', label=_('Display QR code')),
+        Flag('qrcode?', label=_('(deprecated)'), flags=('no_option')),
+        Flag('no_qrcode', label=_('Do not display QR code'), default=False),
     )
 
     has_output_params = LDAPCreate.has_output_params + (
@@ -265,6 +290,7 @@ class otptoken_add(LDAPCreate):
                                   error='is before the validity start')
 
         # Set the object class and defaults for specific token types
+        options['type'] = options['type'].lower()
         entry_attrs['objectclass'] = otptoken.object_class + ['ipatoken' + options['type']]
         for ttype, tattrs in TOKEN_TYPES.items():
             if ttype != options['type']:
@@ -311,10 +337,12 @@ class otptoken_add(LDAPCreate):
         uri = u'otpauth://%s/%s:%s?%s' % (options['type'], issuer, label, parameters)
         setattr(context, 'uri', uri)
 
+        attrs_list.append("objectclass")
         return dn
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
         entry_attrs['uri'] = getattr(context, 'uri')
+        _set_token_type(entry_attrs, **options)
         _convert_owner(self.api.Object.user, entry_attrs, options)
         return super(otptoken_add, self).post_callback(ldap, dn, entry_attrs, *keys, **options)
 
@@ -323,7 +351,7 @@ class otptoken_add(LDAPCreate):
         rv = super(otptoken_add, self).output_for_cli(textui, output, *args, **options)
 
         # Print QR code to terminal if specified
-        if uri and options.get('qrcode', False):
+        if uri and not options.get('no_qrcode', False):
             print "\n"
             qr = qrcode.QRCode()
             qr.add_data(uri)
@@ -366,9 +394,25 @@ class otptoken_mod(LDAPUpdate):
                 raise ValidationError(name='not_before',
                                       error='is after the validity end')
         _normalize_owner(self.api.Object.user, entry_attrs)
+
+        # ticket #4681: if the owner of the token is changed and the
+        # user also manages this token, then we should automatically
+        # set the 'managedby' attribute to the new owner
+        if 'ipatokenowner' in entry_attrs and 'managedby' not in entry_attrs:
+            new_owner = entry_attrs.get('ipatokenowner', None)
+            prev_entry = ldap.get_entry(dn, attrs_list=['ipatokenowner',
+                                                        'managedby'])
+            prev_owner = prev_entry.get('ipatokenowner', None)
+            prev_managedby = prev_entry.get('managedby', None)
+
+            if (new_owner != prev_owner) and (prev_owner == prev_managedby):
+                entry_attrs.setdefault('managedby', new_owner)
+
+        attrs_list.append("objectclass")
         return dn
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
+        _set_token_type(entry_attrs, **options)
         _convert_owner(self.api.Object.user, entry_attrs, options)
         return super(otptoken_mod, self).post_callback(ldap, dn, entry_attrs, *keys, **options)
 
@@ -378,7 +422,7 @@ class otptoken_find(LDAPSearch):
     __doc__ = _('Search for OTP token.')
     msg_summary = ngettext('%(count)d OTP token matched', '%(count)d OTP tokens matched', 0)
 
-    def pre_callback(self, ldap, filters, *args, **kwargs):
+    def pre_callback(self, ldap, filters, attrs_list, *args, **kwargs):
         # This is a hack, but there is no other way to
         # replace the objectClass when searching
         type = kwargs.get('type', '')
@@ -387,7 +431,8 @@ class otptoken_find(LDAPSearch):
         filters = filters.replace("(objectclass=ipatoken)",
                                   "(objectclass=ipatoken%s)" % type)
 
-        return super(otptoken_find, self).pre_callback(ldap, filters, *args, **kwargs)
+        attrs_list.append("objectclass")
+        return super(otptoken_find, self).pre_callback(ldap, filters, attrs_list, *args, **kwargs)
 
     def args_options_2_entry(self, *args, **options):
         entry = super(otptoken_find, self).args_options_2_entry(*args, **options)
@@ -396,6 +441,7 @@ class otptoken_find(LDAPSearch):
 
     def post_callback(self, ldap, entries, truncated, *args, **options):
         for entry in entries:
+            _set_token_type(entry, **options)
             _convert_owner(self.api.Object.user, entry, options)
         return super(otptoken_find, self).post_callback(ldap, entries, truncated, *args, **options)
 
@@ -404,7 +450,12 @@ class otptoken_find(LDAPSearch):
 class otptoken_show(LDAPRetrieve):
     __doc__ = _('Display information about an OTP token.')
 
+    def pre_callback(self, ldap, dn, attrs_list, *keys, **options):
+        attrs_list.append("objectclass")
+        return super(otptoken_show, self).pre_callback(ldap, dn, attrs_list, *keys, **options)
+
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
+        _set_token_type(entry_attrs, **options)
         _convert_owner(self.api.Object.user, entry_attrs, options)
         return super(otptoken_show, self).post_callback(ldap, dn, entry_attrs, *keys, **options)
 
