@@ -18,6 +18,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib.plugins.dns import dns_container_exists
 from ipapython.ipautil import realm_to_suffix
@@ -129,6 +130,8 @@ particular type.
    ipa trustconfig-mod --type ad --fallback-primary-group "Default SMB Group"
 """)
 
+register = Registry()
+
 trust_output_params = (
     Str('trustdirection',
         label=_('Trust direction')),
@@ -296,6 +299,7 @@ def add_range(self, range_name, dom_sid, *keys, **options):
     return range_type, range_size, base_id
 
 
+@register()
 class trust(LDAPObject):
     """
     Trust object.
@@ -306,12 +310,38 @@ class trust(LDAPObject):
     object_name_plural = _('trusts')
     object_class = ['ipaNTTrustedDomain']
     default_attributes = ['cn', 'ipantflatname', 'ipanttrusteddomainsid',
-        'ipanttrusttype', 'ipanttrustattributes', 'ipanttrustdirection', 'ipanttrustpartner',
-        'ipantauthtrustoutgoing', 'ipanttrustauthincoming', 'ipanttrustforesttrustinfo',
+        'ipanttrusttype', 'ipanttrustattributes', 'ipanttrustdirection',
+        'ipanttrustpartner', 'ipanttrustforesttrustinfo',
         'ipanttrustposixoffset', 'ipantsupportedencryptiontypes' ]
     search_display_attributes = ['cn', 'ipantflatname',
-                                 'ipanttrusteddomainsid', 'ipanttrusttype',
-                                 'ipantsidblacklistincoming', 'ipantsidblacklistoutgoing' ]
+                                 'ipanttrusteddomainsid', 'ipanttrusttype']
+    managed_permissions = {
+        'System: Read Trust Information': {
+            # Allow reading of attributes needed for SSSD subdomains support
+            'non_object': True,
+            'ipapermlocation': DN(container_dn, api.env.basedn),
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'cn', 'objectclass',
+                'ipantflatname', 'ipantsecurityidentifier',
+                'ipanttrusteddomainsid', 'ipanttrustpartner',
+                'ipantsidblacklistincoming', 'ipantsidblacklistoutgoing'
+            },
+        },
+
+        'System: Read system trust accounts': {
+            'non_object': True,
+            'ipapermlocation': DN(container_dn, api.env.basedn),
+            'replaces_global_anonymous_aci': True,
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'uidnumber', 'gidnumber', 'krbprincipalname'
+            },
+            'default_privileges': {'ADTrust Agents'},
+        },
+    }
 
     label = _('Trusts')
     label_singular = _('Trust')
@@ -365,7 +395,7 @@ class trust(LDAPObject):
             ldap = self.backend
             filter = ldap.make_filter({'objectclass': ['ipaNTTrustedDomain'], 'cn': [keys[-1]] },
                                       rules=ldap.MATCH_ALL)
-            filter = ldap.combine_filters((filter, "ipaNTSIDBlacklistIncoming=*"), rules=ldap.MATCH_ALL)
+            filter = ldap.combine_filters((filter, "ipaNTSecurityIdentifier=*"), rules=ldap.MATCH_ALL)
             result = ldap.get_entries(DN(self.container_dn, self.env.basedn),
                                       ldap.SCOPE_SUBTREE, filter, [''])
             if len(result) > 1:
@@ -375,6 +405,7 @@ class trust(LDAPObject):
         dn=make_trust_dn(self.env, trust_type, DN(*sdn))
         return dn
 
+@register()
 class trust_add(LDAPCreate):
     __doc__ = _('''
 Add new trust to use.
@@ -404,7 +435,7 @@ sides.
         ),
         Password('realm_passwd?',
             cli_name='password',
-            label=_("Active directory domain administrator's password"),
+            label=_("Active Directory domain administrator's password"),
             confirm=False,
         ),
         Str('realm_server?',
@@ -434,6 +465,7 @@ sides.
     )
 
     msg_summary = _('Added Active Directory trust for realm "%(value)s"')
+    msg_summary_existing = _('Re-established trust to domain "%(value)s"')
     has_output_params = LDAPCreate.has_output_params + trust_output_params
 
     def execute(self, *keys, **options):
@@ -456,40 +488,16 @@ sides.
                          base_dn=DN(api.env.container_trusts, api.env.basedn),
                          filter=trust_filter)
 
-        result['result'] = entry_to_dict(trusts[0][1], **options)
+        result['result'] = entry_to_dict(trusts[0], **options)
 
-        # For AD trusts with algorithmic mapping, we need to add a separate
-        # range for each subdomain.
+        # Fetch topology of the trust forest -- we need always to do it
+        # for AD trusts, regardless of the type of idranges associated with it
+        # Note that fetch_domains_from_trust will add needed ranges for
+        # the algorithmic ID mapping case.
         if (options.get('trust_type') == u'ad' and
-            created_range_type != u'ipa-ad-trust-posix'):
-
+            options.get('trust_secret') is None):
             domains = fetch_domains_from_trust(self, self.trustinstance,
                                                result['result'], **options)
-            if domains and len(domains) > 0:
-                for dom in domains:
-                    range_name = dom['cn'][0].upper() + '_id_range'
-                    dom_sid = dom['ipanttrusteddomainsid'][0]
-
-                    # Enforce the same range type as the range for the root
-                    # level domain.
-
-                    # This will skip the detection of the POSIX attributes if
-                    # they are not available, since it has been already
-                    # detected when creating the range for the root level domain
-                    passed_options = options
-                    passed_options.update(range_type=created_range_type)
-
-                    # Do not pass the base id to the subdomains since it would
-                    # clash with the root level domain
-                    if 'base_id' in passed_options:
-                        del passed_options['base_id']
-
-                    # Try to add the range for each subdomain
-                    try:
-                        add_range(self, range_name, dom_sid, *keys,
-                                  **passed_options)
-                    except errors.DuplicateEntry:
-                        pass
 
         # Format the output into human-readable values
         result['result']['trusttype'] = [trust_type_string(
@@ -500,9 +508,35 @@ sides.
             result['verified'])]
 
         del result['verified']
+        result['result'].pop('ipanttrustauthoutgoing', None)
+        result['result'].pop('ipanttrustauthincoming', None)
 
         return result
 
+    def interactive_prompt_callback(self, kw):
+        """
+        Also ensure that realm_admin is prompted for if --admin or
+        --trust-secret is not specified when 'ipa trust-add' is run on the
+        system.
+
+        Also ensure that realm_passwd is prompted for if --password or
+        --trust-secret is not specified when 'ipa trust-add' is run on the
+        system.
+        """
+
+        trust_secret = kw.get('trust_secret')
+        realm_admin = kw.get('realm_admin')
+        realm_passwd = kw.get('realm_passwd')
+
+        if trust_secret is None:
+            if realm_admin is None:
+                kw['realm_admin'] = self.prompt_param(
+                           self.params['realm_admin'])
+
+            if realm_passwd is None:
+                kw['realm_passwd'] = self.Backend.textui.prompt_password(
+                           self.params['realm_passwd'].label, confirm=False)
+
     def validate_options(self, *keys, **options):
         if not _bindings_installed:
             raise errors.NotFound(
@@ -664,11 +698,6 @@ sides.
         except errors.NotFound:
             dn = None
 
-        if dn:
-            summary = _('Re-established trust to domain "%(value)s"')
-        else:
-            summary = self.msg_summary
-
         # 1. Full access to the remote domain. Use admin credentials and
         # generate random trustdom password to do work on both sides
         if full_join:
@@ -709,36 +738,46 @@ sides.
                                              error=_('Unable to verify write permissions to the AD'))
 
             ret = dict(
-                value=self.trustinstance.remote_domain.info['dns_domain'],
+                value=pkey_to_value(
+                    self.trustinstance.remote_domain.info['dns_domain'],
+                    options),
                 verified=result['verified']
             )
-            ret['summary'] = summary % ret
+            if dn:
+                ret['summary'] = self.msg_summary_existing % ret
             return ret
 
-
         # 2. We don't have access to the remote domain and trustdom password
         # is provided. Do the work on our side and inform what to do on remote
         # side.
-        if 'trust_secret' in options:
+        if options.get('trust_secret'):
             result = self.trustinstance.join_ad_ipa_half(
                 keys[-1],
                 self.realm_server,
                 options['trust_secret']
             )
             ret = dict(
-                value=self.trustinstance.remote_domain.info['dns_domain'],
+                value=pkey_to_value(
+                    self.trustinstance.remote_domain.info['dns_domain'],
+                    options),
                 verified=result['verified']
             )
-            ret['summary'] = summary % ret
+            if dn:
+                ret['summary'] = self.msg_summary_existing % ret
             return ret
-        raise errors.ValidationError(name=_('AD Trust setup'),
-                                     error=_('Not enough arguments specified to perform trust setup'))
+        else:
+            raise errors.ValidationError(
+                name=_('AD Trust setup'),
+                error=_('Not enough arguments specified to perform trust '
+                        'setup'))
 
+@register()
 class trust_del(LDAPDelete):
     __doc__ = _('Delete a trust.')
 
     msg_summary = _('Deleted trust "%(value)s"')
 
+@register()
 class trust_mod(LDAPUpdate):
     __doc__ = _("""
     Modify a trust (for future use).
@@ -757,6 +796,7 @@ class trust_mod(LDAPUpdate):
 
         return dn
 
+@register()
 class trust_find(LDAPSearch):
     __doc__ = _('Search for trusts.')
     has_output_params = LDAPSearch.has_output_params + trust_output_params +\
@@ -770,7 +810,7 @@ class trust_find(LDAPSearch):
     # search needs to be done on a sub-tree scope
     def pre_callback(self, ldap, filters, attrs_list, base_dn, scope, *args, **options):
         # list only trust, not trust domains
-        trust_filter = '(ipaNTSIDBlacklistIncoming=*)'
+        trust_filter = '(ipaNTSecurityIdentifier=*)'
         filter = ldap.combine_filters((filters, trust_filter), rules=ldap.MATCH_ALL)
         return (filter, base_dn, ldap.SCOPE_SUBTREE)
 
@@ -778,16 +818,16 @@ class trust_find(LDAPSearch):
         if options.get('pkey_only', False):
             return truncated
 
-        for entry in entries:
-            (dn, attrs) = entry
-
+        for attrs in entries:
             # Translate ipanttrusttype to trusttype if --raw not used
-            if not options.get('raw', False):
+            trust_type = attrs.get('ipanttrusttype', [None])[0]
+            if not options.get('raw', False) and trust_type is not None:
                 attrs['trusttype'] = trust_type_string(attrs['ipanttrusttype'][0])
                 del attrs['ipanttrusttype']
 
         return truncated
 
+@register()
 class trust_show(LDAPRetrieve):
     __doc__ = _('Display information about a trust.')
     has_output_params = LDAPRetrieve.has_output_params + trust_output_params +\
@@ -801,28 +841,25 @@ class trust_show(LDAPRetrieve):
         # if --raw not used
 
         if not options.get('raw', False):
-            type_str = trust_type_string(entry_attrs['ipanttrusttype'][0])
-            dir_str = trust_direction_string(entry_attrs['ipanttrustdirection']
-                                                        [0])
-            entry_attrs['trusttype'] = [type_str]
-            entry_attrs['trustdirection'] = [dir_str]
-            del entry_attrs['ipanttrusttype']
-            del entry_attrs['ipanttrustdirection']
+            trust_type = entry_attrs.get('ipanttrusttype', [None])[0]
+            if trust_type is not None:
+                entry_attrs['trusttype'] = trust_type_string(trust_type)
+                del entry_attrs['ipanttrusttype']
+
+            dir_str = entry_attrs.get('ipanttrustdirection', [None])[0]
+            if dir_str is not None:
+                entry_attrs['trustdirection'] = [trust_direction_string(dir_str)]
+                del entry_attrs['ipanttrustdirection']
 
         return dn
 
-api.register(trust)
-api.register(trust_add)
-api.register(trust_mod)
-api.register(trust_del)
-api.register(trust_find)
-api.register(trust_show)
 
 _trustconfig_dn = {
     u'ad': DN(('cn', api.env.domain), api.env.container_cifsdomains, api.env.basedn),
 }
 
 
+@register()
 class trustconfig(LDAPObject):
     """
     Trusts global configuration object
@@ -897,7 +934,7 @@ class trustconfig(LDAPObject):
             # and not "ipausergroup" so that it can also match groups like
             # "Default SMB Group" which does not have this objectclass.
             try:
-                (dn, group_entry) = self.backend.find_entry_by_attr(
+                group_entry = self.backend.find_entry_by_attr(
                     self.api.Object['group'].primary_key.name,
                     group,
                     ['posixgroup'],
@@ -906,7 +943,7 @@ class trustconfig(LDAPObject):
             except errors.NotFound:
                 self.api.Object['group'].handle_not_found(group)
             else:
-                entry_attrs['ipantfallbackprimarygroup'] = [dn]
+                entry_attrs['ipantfallbackprimarygroup'] = [group_entry.dn]
 
     def _convert_groupdn(self, entry_attrs, options):
         """
@@ -927,8 +964,8 @@ class trustconfig(LDAPObject):
 
         entry_attrs['ipantfallbackprimarygroup'] = [groupdn[0][0].value]
 
-api.register(trustconfig)
 
+@register()
 class trustconfig_mod(LDAPUpdate):
     __doc__ = _('Modify global trust configuration.')
 
@@ -941,16 +978,16 @@ class trustconfig_mod(LDAPUpdate):
 
     def execute(self, *keys, **options):
         result = super(trustconfig_mod, self).execute(*keys, **options)
-        result['value'] = options['trust_type']
+        result['value'] = pkey_to_value(options['trust_type'], options)
         return result
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
         self.obj._convert_groupdn(entry_attrs, options)
         return dn
 
-api.register(trustconfig_mod)
 
 
+@register()
 class trustconfig_show(LDAPRetrieve):
     __doc__ = _('Show global trust configuration.')
 
@@ -958,14 +995,13 @@ class trustconfig_show(LDAPRetrieve):
 
     def execute(self, *keys, **options):
         result = super(trustconfig_show, self).execute(*keys, **options)
-        result['value'] = options['trust_type']
+        result['value'] = pkey_to_value(options['trust_type'], options)
         return result
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
         self.obj._convert_groupdn(entry_attrs, options)
         return dn
 
-api.register(trustconfig_show)
 
 if _nss_idmap_installed:
     _idmap_type_dict = {
@@ -977,6 +1013,7 @@ if _nss_idmap_installed:
         string = _idmap_type_dict.get(int(level), 'unknown')
         return unicode(string)
 
+@register()
 class trust_resolve(Command):
     NO_CLI = True
     __doc__ = _('Resolve security identifiers of users and groups in trusted domains')
@@ -1015,9 +1052,9 @@ class trust_resolve(Command):
 
         return dict(result=result)
 
-api.register(trust_resolve)
 
 
+@register()
 class adtrust_is_enabled(Command):
     NO_CLI = True
 
@@ -1042,9 +1079,9 @@ class adtrust_is_enabled(Command):
 
         return dict(result=True)
 
-api.register(adtrust_is_enabled)
 
 
+@register()
 class compat_is_enabled(Command):
     NO_CLI = True
 
@@ -1086,9 +1123,9 @@ class compat_is_enabled(Command):
 
         return dict(result=True)
 
-api.register(compat_is_enabled)
 
 
+@register()
 class sidgen_was_run(Command):
     """
     This command tries to determine whether the sidgen task was run during
@@ -1130,8 +1167,8 @@ class sidgen_was_run(Command):
 
         return dict(result=True)
 
-api.register(sidgen_was_run)
 
+@register()
 class trustdomain(LDAPObject):
     """
     Object representing a domain of the AD trust.
@@ -1179,8 +1216,8 @@ class trustdomain(LDAPObject):
 
         dn=make_trust_dn(self.env, trust_type, DN(*sdn))
         return dn
-api.register(trustdomain)
 
+@register()
 class trustdomain_find(LDAPSearch):
     __doc__ = _('Search domains of the trust')
 
@@ -1191,26 +1228,33 @@ class trustdomain_find(LDAPSearch):
         return (filters, base_dn, ldap.SCOPE_SUBTREE)
 
     def post_callback(self, ldap, entries, truncated, *args, **options):
+        if options.get('pkey_only', False):
+            return truncated
         trust_dn = self.obj.get_dn(args[0], trust_type=u'ad')
         trust_entry = ldap.get_entry(trust_dn)
         for entry in entries:
             sid = entry['ipanttrusteddomainsid'][0]
-            if sid in trust_entry['ipantsidblacklistincoming']:
+
+            blacklist = trust_entry.get('ipantsidblacklistincoming')
+            if blacklist is None:
+                continue
+
+            if sid in blacklist:
                 entry['domain_enabled'] = [False]
             else:
                 entry['domain_enabled'] = [True]
         return truncated
 
 
-api.register(trustdomain_find)
 
+@register()
 class trustdomain_mod(LDAPUpdate):
     __doc__ = _('Modify trustdomain of the trust')
 
     NO_CLI = True
     takes_options = LDAPUpdate.takes_options + (_trust_type_option,)
-api.register(trustdomain_mod)
 
+@register()
 class trustdomain_add(LDAPCreate):
     __doc__ = _('Allow access from the trusted domain')
     NO_CLI = True
@@ -1220,8 +1264,8 @@ class trustdomain_add(LDAPCreate):
         if 'ipanttrustpartner' in options:
             entry_attrs['ipanttrustpartner'] = [options['ipanttrustpartner']]
         return dn
-api.register(trustdomain_add)
 
+@register()
 class trustdomain_del(LDAPDelete):
     __doc__ = _('Remove infromation about the domain associated with the trust.')
 
@@ -1240,11 +1284,10 @@ class trustdomain_del(LDAPDelete):
             except errors.AlreadyActive:
                 pass
         result = super(trustdomain_del, self).execute(*keys, **options)
-        result['value'] = u','.join(keys[1])
+        result['value'] = pkey_to_value(keys[1], options)
         return result
 
 
-api.register(trustdomain_del)
 
 
 def fetch_domains_from_trust(self, trustinstance, trust_entry, **options):
@@ -1268,7 +1311,7 @@ def fetch_domains_from_trust(self, trustinstance, trust_entry, **options):
     # trust range must exist by the time fetch_domains_from_trust is called
     range_name = trust_name.upper() + '_id_range'
     old_range = api.Command.idrange_show(range_name, raw=True)['result']
-    idrange_type = old_range['iparangetype']
+    idrange_type = old_range['iparangetype'][0]
 
     for dom in domains:
         dom['trust_type'] = u'ad'
@@ -1293,6 +1336,7 @@ def fetch_domains_from_trust(self, trustinstance, trust_entry, **options):
             pass
     return result
 
+@register()
 class trust_fetch_domains(LDAPRetrieve):
     __doc__ = _('Refresh list of the domains associated with the trust')
 
@@ -1333,8 +1377,8 @@ class trust_fetch_domains(LDAPRetrieve):
         result['truncated'] = False
         return result
 
-api.register(trust_fetch_domains)
 
+@register()
 class trustdomain_enable(LDAPQuery):
     __doc__ = _('Allow use of IPA resources by the domain of the trust')
 
@@ -1370,11 +1414,11 @@ class trustdomain_enable(LDAPQuery):
 
         return dict(
             result=True,
-            value=keys[1],
+            value=pkey_to_value(keys[1], options),
         )
 
-api.register(trustdomain_enable)
 
+@register()
 class trustdomain_disable(LDAPQuery):
     __doc__ = _('Disable use of IPA resources by the domain of the trust')
 
@@ -1410,7 +1454,6 @@ class trustdomain_disable(LDAPQuery):
 
         return dict(
             result=True,
-            value=keys[1],
+            value=pkey_to_value(keys[1], options),
         )
 
-api.register(trustdomain_disable)
