@@ -68,6 +68,7 @@ from ipalib.krb_utils import KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN, KRB5KRB_AP_ERR_TKT
                              KRB5_FCC_PERM, KRB5_FCC_NOFILE, KRB5_CC_FORMAT, KRB5_REALM_CANT_RESOLVE
 from ipapython.dn import DN
 from ipalib.capabilities import VERSION_WITHOUT_CAPABILITIES
+from ipalib import api
 
 COOKIE_NAME = 'ipa_session'
 KEYRING_COOKIE_NAME = '%s_cookie:%%s' % COOKIE_NAME
@@ -483,12 +484,14 @@ class SSLTransport(LanguageAwareTransport):
             if self._connection and host == self._connection[0]:
                 return self._connection[1]
 
-        dbdir = paths.NSS_DB_DIR
+        dbdir = getattr(context, 'nss_dir', paths.IPA_NSSDB_DIR)
         no_init = self.__nss_initialized(dbdir)
         if sys.version_info < (2, 7):
             conn = NSSHTTPS(host, 443, dbdir=dbdir, no_init=no_init)
         else:
-            conn = NSSConnection(host, 443, dbdir=dbdir, no_init=no_init)
+            conn = NSSConnection(host, 443, dbdir=dbdir, no_init=no_init,
+                                 tls_version_min=api.env.tls_version_min,
+                                 tls_version_max=api.env.tls_version_max)
         self.dbdir=dbdir
 
         conn.connect()
@@ -761,8 +764,8 @@ class RPCClient(Connectible):
 
         return session_url
 
-    def create_connection(self, ccache=None, verbose=False, fallback=True,
-                          delegate=False):
+    def create_connection(self, ccache=None, verbose=0, fallback=True,
+                          delegate=False, nss_dir=None):
         try:
             rpc_uri = self.env[self.env_rpc_uri_key]
             principal = get_current_principal()
@@ -774,6 +777,9 @@ class RPCClient(Connectible):
         except ValueError:
             # No session key, do full Kerberos auth
             pass
+        # This might be dangerous. Use at your own risk!
+        if nss_dir:
+            context.nss_dir = nss_dir
         urls = self.get_url_list(rpc_uri)
         serverproxy = None
         for url in urls:
@@ -961,12 +967,17 @@ class JSONServerProxy(object):
     def __request(self, name, args):
         payload = {'method': unicode(name), 'params': args, 'id': 0}
         version = args[1].get('version', VERSION_WITHOUT_CAPABILITIES)
+        payload = json_encode_binary(payload, version)
+
+        if self.__verbose >= 2:
+            root_logger.info('Request: %s',
+                             json.dumps(payload, sort_keys=True, indent=4))
 
         response = self.__transport.request(
             self.__host,
             self.__handler,
-            json.dumps(json_encode_binary(payload, version)),
-            verbose=self.__verbose,
+            json.dumps(payload),
+            verbose=self.__verbose >= 3,
         )
 
         try:
@@ -974,6 +985,12 @@ class JSONServerProxy(object):
         except ValueError, e:
             raise JSONError(str(e))
 
+        if self.__verbose >= 2:
+            root_logger.info(
+                'Response: %s',
+                json.dumps(json_encode_binary(response, version),
+                           sort_keys=True, indent=4)
+            )
         error = response.get('error')
         if error:
             try:
