@@ -27,6 +27,8 @@ import time
 import socket
 import re
 import decimal
+import dns
+import encodings
 import netaddr
 from types import NoneType
 from weakref import WeakKeyDictionary
@@ -219,9 +221,6 @@ def normalize_zonemgr(zonemgr):
         name = name.replace('.', '\\.')
         zonemgr = u''.join((name, u'.', domain))
 
-    if not zonemgr.endswith('.'):
-        zonemgr = zonemgr + u'.'
-
     return zonemgr
 
 def normalize_zone(zone):
@@ -273,18 +272,14 @@ def validate_domain_name(domain_name, allow_underscore=False, allow_slash=False)
 
 def validate_zonemgr(zonemgr):
     assert isinstance(zonemgr, DNSName)
-    assert zonemgr.is_absolute()
-    """ See RFC 1033, 1035 """
     if any('@' in label for label in zonemgr.labels):
         raise ValueError(_('too many \'@\' characters'))
-    if len(zonemgr.labels) < 3:
-        raise ValueError(_('missing address domain'))
-    if not zonemgr.labels[0]:
-        raise ValueError(_('missing mail account'))
+
 
 def validate_zonemgr_str(zonemgr):
     zonemgr = normalize_zonemgr(zonemgr)
-    zonemgr = DNSName(zonemgr).make_absolute()
+    validate_idna_domain(zonemgr)
+    zonemgr = DNSName(zonemgr)
     return validate_zonemgr(zonemgr)
 
 def validate_hostname(hostname, check_fqdn=True, allow_underscore=False, allow_slash=False):
@@ -562,3 +557,80 @@ def validate_hostmask(ugettext, hostmask):
         netaddr.IPNetwork(hostmask)
     except (ValueError, AddrFormatError):
         return _('invalid hostmask')
+
+
+def validate_dnssec_forwarder(ip_addr):
+    """Test DNS forwarder properties.
+
+    :returns:
+     True if forwarder works as expected and supports DNSSEC.
+     False if forwarder does not support DNSSEC.
+     None if forwarder does not respond.
+    """
+    ip_addr = str(ip_addr)
+    res = dns.resolver.Resolver()
+    res.nameservers = [ip_addr]
+    res.lifetime = 10  # wait max 10 seconds for reply
+
+    # enable Authenticated Data + Checking Disabled flags
+    res.set_flags(dns.flags.AD | dns.flags.CD)
+
+    # enable EDNS v0 + enable DNSSEC-Ok flag
+    res.use_edns(0, dns.flags.DO, 0)
+
+    # DNS root has to be signed
+    try:
+        ans = res.query('.', 'NS')
+    except DNSException:
+        return None
+
+    try:
+        ans.response.find_rrset(ans.response.answer, dns.name.root,
+                dns.rdataclass.IN, dns.rdatatype.RRSIG, dns.rdatatype.NS)
+    except KeyError:
+        return False
+
+    return True
+
+
+def validate_idna_domain(value):
+    """
+    Validate if value is valid IDNA domain.
+
+    If domain is not valid, raises ValueError
+    :param value:
+    :return:
+    """
+    error = None
+
+    try:
+        DNSName(value)
+    except dns.name.BadEscape:
+        error = _('invalid escape code in domain name')
+    except dns.name.EmptyLabel:
+        error = _('empty DNS label')
+    except dns.name.NameTooLong:
+        error = _('domain name cannot be longer than 255 characters')
+    except dns.name.LabelTooLong:
+        error = _('DNS label cannot be longer than 63 characters')
+    except dns.exception.SyntaxError:
+        error = _('invalid domain name')
+    else:
+        #compare if IDN normalized and original domain match
+        #there is N:1 mapping between unicode and IDNA names
+        #user should use normalized names to avoid mistakes
+        labels = re.split(u'[.\uff0e\u3002\uff61]', value, flags=re.UNICODE)
+        try:
+            map(lambda label: label.encode("ascii"), labels)
+        except UnicodeError:
+            # IDNA
+            is_nonnorm = any(encodings.idna.nameprep(x) != x for x in labels)
+            if is_nonnorm:
+                error = _("domain name '%(domain)s' should be normalized to"
+                          ": %(normalized)s") % {
+                          'domain': value,
+                          'normalized': '.'.join([encodings.idna.nameprep(x)
+                                                  for x in labels])}
+
+    if error:
+        raise ValueError(error)
