@@ -39,7 +39,7 @@ wellknownservices = ['certmonger', 'dirsrv', 'httpd', 'ipa', 'krb5kdc',
                      'messagebus', 'nslcd', 'nscd', 'ntpd', 'portmap',
                      'rpcbind', 'kadmin', 'sshd', 'autofs', 'rpcgssd',
                      'rpcidmapd', 'pki_tomcatd', 'pki_cad', 'chronyd',
-                     'domainname']
+                     'domainname', 'named', 'ods_enforcerd', 'ods_signerd']
 
 # The common ports for these services. This is used to wait for the
 # service to become available.
@@ -134,12 +134,21 @@ class PlatformService(object):
     def is_enabled(self, instance_name=""):
         return False
 
+    def is_masked(self, instance_name=""):
+        return False
+
     def enable(self, instance_name=""):
         return
 
     def disable(self, instance_name=""):
         return
 
+    def mask(self, instance_name=""):
+        return
+
+    def unmask(self, instance_name=""):
+        return
+
     def install(self, instance_name=""):
         return
 
@@ -149,6 +158,18 @@ class PlatformService(object):
     def get_config_dir(self, instance_name=""):
         return
 
+    def get_user_name(self, instance_name=""):
+        return
+
+    def get_group_name(self, instance_name=""):
+        return
+
+    def get_binary_path(self):
+        return
+
+    def get_package_name(self):
+        return
+
 
 class SystemdService(PlatformService):
     SYSTEMD_SRV_TARGET = "%s.target.wants"
@@ -166,8 +187,7 @@ class SystemdService(PlatformService):
         elements = self.systemd_name.split("@")
 
         # Make sure the correct DS instance is returned
-        if (elements[0] == 'dirsrv' and not instance_name and
-            operation == 'is-active'):
+        if elements[0] == 'dirsrv' and not instance_name:
 
             return ('dirsrv@%s.service'
                     % str(self.api.env.realm.replace('.', '-')))
@@ -327,6 +347,21 @@ class SystemdService(PlatformService):
                 enabled = False
         return enabled
 
+    def is_masked(self, instance_name=""):
+        masked = False
+        try:
+            (sout, serr, rcode) = ipautil.run(
+                                      [paths.SYSTEMCTL,
+                                       "is-enabled",
+                                        self.service_instance(instance_name)])
+
+            if rcode == 1 and sout == 'masked':
+                masked = True
+
+        except ipautil.CalledProcessError:
+                pass
+        return masked
+
     def enable(self, instance_name=""):
         if self.lib_path_exists is None:
             self.lib_path_exists = os.path.exists(self.lib_path)
@@ -400,18 +435,32 @@ class SystemdService(PlatformService):
             except:
                 pass
         else:
-            self.__disable(instance_name)
+            try:
+                ipautil.run([paths.SYSTEMCTL, "disable",
+                             self.service_instance(instance_name)])
+            except ipautil.CalledProcessError:
+                pass
 
-    def __enable(self, instance_name=""):
+    def mask(self, instance_name=""):
+        srv_tgt = os.path.join(paths.ETC_SYSTEMD_SYSTEM_DIR, self.service_instance(instance_name))
+        if os.path.exists(srv_tgt):
+            os.unlink(srv_tgt)
         try:
-            ipautil.run([paths.SYSTEMCTL, "enable",
+            ipautil.run([paths.SYSTEMCTL, "mask",
                          self.service_instance(instance_name)])
         except ipautil.CalledProcessError:
             pass
 
-    def __disable(self, instance_name=""):
+    def unmask(self, instance_name=""):
         try:
-            ipautil.run([paths.SYSTEMCTL, "disable",
+            ipautil.run([paths.SYSTEMCTL, "unmask",
+                         self.service_instance(instance_name)])
+        except ipautil.CalledProcessError:
+            pass
+
+    def __enable(self, instance_name=""):
+        try:
+            ipautil.run([paths.SYSTEMCTL, "enable",
                          self.service_instance(instance_name)])
         except ipautil.CalledProcessError:
             pass
