@@ -22,7 +22,13 @@
 This module contains default platform-specific implementations of system tasks.
 '''
 
+import pwd
+import grp
 from ipaplatform.paths import paths
+from ipapython.ipa_log_manager import log_mgr
+from ipapython import ipautil
+
+log = log_mgr.get_logger(__name__)
 
 
 class BaseTaskNamespace(object):
@@ -49,9 +55,18 @@ class BaseTaskNamespace(object):
 
         return
 
-    def insert_ca_cert_into_systemwide_ca_store(self, path):
+    def reload_systemwide_ca_store(self):
         """
-        Adds the CA certificate located at 'path' to the systemwide CA store
+        Reloads the systemwide CA store.
+
+        Returns True if the operation succeeded, False otherwise.
+        """
+
+        return True
+
+    def insert_ca_certs_into_systemwide_ca_store(self, ca_certs):
+        """
+        Adds CA certificates from 'ca_certs' to the systemwide CA store
         (if available on the platform).
 
         Returns True if the operation succeeded, False otherwise.
@@ -59,10 +74,10 @@ class BaseTaskNamespace(object):
 
         return True
 
-    def remove_ca_cert_from_systemwide_ca_store(self, path):
+    def remove_ca_certs_from_systemwide_ca_store(self):
         """
-        Removes the CA certificate located at 'path' from the systemwide CA
-        store (if available on the platform).
+        Removes IPA CA certificates from the systemwide CA store
+        (if available on the platform).
 
         Returns True if the operation succeeded, False otherwise.
         """
@@ -132,4 +147,66 @@ class BaseTaskNamespace(object):
 
         return
 
+    def set_selinux_booleans(self, required_settings, backup_func=None):
+        """Set the specified SELinux booleans
+
+        :param required_settings: A dictionary mapping the boolean names
+                                  to desired_values.
+                                  The desired value can be 'on' or 'off',
+                                  or None to leave the setting unchanged.
+
+        :param backup_func: A function called for each boolean with two
+                            arguments: the name and the previous value
+
+        If SELinux is disabled, return False; on success returns True.
+
+        If setting the booleans fails,
+        an ipapython.errors.SetseboolError is raised.
+        """
+
+        return
+
+    def create_system_user(self, name, group, homedir, shell, uid = None, gid = None, comment = None):
+        """Create a system user with a corresponding group"""
+        try:
+            grp.getgrnam(group)
+        except KeyError:
+            log.debug('Adding group %s', group)
+            args = [paths.GROUPADD, '-r', group]
+            if gid:
+                args += ['-g', str(gid)]
+            try:
+                ipautil.run(args)
+                log.debug('Done adding group')
+            except ipautil.CalledProcessError as e:
+                log.critical('Failed to add group: %s', e)
+                raise
+        else:
+            log.debug('group %s exists', group)
+
+        try:
+            pwd.getpwnam(name)
+        except KeyError:
+            log.debug('Adding user %s', name)
+            args = [
+                paths.USERADD,
+                '-g', group,
+                '-d', homedir,
+                '-s', shell,
+                '-M', '-r', name,
+            ]
+            if uid:
+                args += ['-u', str(uid)]
+            if comment:
+                args += ['-c', comment]
+            try:
+                ipautil.run(args)
+                log.debug('Done adding user')
+            except ipautil.CalledProcessError as e:
+                log.critical('Failed to add user: %s', e)
+                raise
+        else:
+            log.debug('user %s exists', name)
+
+
 task_namespace = BaseTaskNamespace()
