@@ -52,7 +52,7 @@ def setup_package():
     try:
         setup(
             name = "ipapython",
-            version = "4.0.5",
+            version = "4.1.4",
             license = "GPL",
             author = "Karl MacMillan, et.al.",
             author_email = "kmacmill@redhat.com",
@@ -65,7 +65,7 @@ def setup_package():
             classifiers=filter(None, CLASSIFIERS.split('\n')),
             platforms = ["Linux", "Solaris", "Unix"],
             package_dir = {'ipapython': ''},
-            packages = [ "ipapython" ],
+            packages = [ "ipapython", "ipapython.dnssec" ],
         )
     finally:
         del sys.path[0]
