@@ -37,6 +37,8 @@ import stat
 import syslog
 import ConfigParser
 import dbus
+import shlex
+import pipes
 
 from ipapython import dogtag
 from ipapython.certdb import get_ca_nickname
@@ -259,6 +261,16 @@ def is_step_one_done():
     return False
 
 
+def create_ca_user():
+    """Create PKI user/group if it doesn't exist yet."""
+    tasks.create_system_user(
+        name=PKI_USER,
+        group=PKI_USER,
+        homedir=paths.VAR_LIB,
+        shell=paths.NOLOGIN,
+    )
+
+
 class CADSInstance(service.Service):
     """Certificate Authority DS instance
 
@@ -270,7 +282,7 @@ class CADSInstance(service.Service):
             service_desc="directory server for the CA",
             dm_password=dm_password,
             ldapi=False,
-            autobind=service.DISABLED)
+            autobind=ipaldap.AUTOBIND_DISABLED)
 
         self.serverid = "PKI-IPA"
         self.realm = realm_name
@@ -320,9 +332,10 @@ def stop_tracking_certificates(dogtag_constants):
     cmonger.start()
 
     for nickname in ['Server-Cert cert-pki-ca',
-                        'auditSigningCert cert-pki-ca',
-                        'ocspSigningCert cert-pki-ca',
-                        'subsystemCert cert-pki-ca']:
+                     'auditSigningCert cert-pki-ca',
+                     'ocspSigningCert cert-pki-ca',
+                     'subsystemCert cert-pki-ca',
+                     'caSigningCert cert-pki-ca']:
         try:
             certmonger.stop_tracking(
                 dogtag_constants.ALIAS_DIR, nickname=nickname)
@@ -355,20 +368,21 @@ class CAInstance(service.Service):
        2 = have signed cert, continue installation
     """
 
-    def __init__(self, realm, ra_db, dogtag_constants=None):
+    def __init__(self, realm=None, ra_db=None, dogtag_constants=None,
+                 host_name=None, dm_password=None, ldapi=True):
         if dogtag_constants is None:
             dogtag_constants = dogtag.configured_constants()
 
         service.Service.__init__(self,
-                '%sd' % dogtag_constants.PKI_INSTANCE_NAME,
-                service_desc="certificate server"
-                )
+                                 '%sd' % dogtag_constants.PKI_INSTANCE_NAME,
+                                 service_desc="certificate server",
+                                 dm_password=dm_password,
+                                 ldapi=ldapi)
 
         self.dogtag_constants = dogtag_constants
         self.realm = realm
-        self.dm_password = None
         self.admin_password = None
-        self.fqdn = None
+        self.fqdn = host_name
         self.domain = None
         self.pkcs12_info = None
         self.clone = False
@@ -383,11 +397,15 @@ class CAInstance(service.Service):
         # The same database is used for mod_nss because the NSS context
         # will already have been initialized by Apache by the time
         # mod_python wants to do things.
-        self.canickname = get_ca_nickname(realm)
+        self.canickname = None
+        if realm:
+            self.canickname = get_ca_nickname(realm)
         self.basedn = DN(('o', 'ipaca'))
         self.ca_agent_db = tempfile.mkdtemp(prefix = "tmp-")
         self.ra_agent_db = ra_db
-        self.ra_agent_pwd = self.ra_agent_db + "/pwdfile.txt"
+        self.ra_agent_pwd = None
+        if ra_db:
+            self.ra_agent_pwd = ra_db + "/pwdfile.txt"
         self.ds_port = DEFAULT_DSPORT
         self.security_domain_name = "IPA"
         self.server_root = dogtag_constants.SERVER_ROOT
@@ -412,7 +430,8 @@ class CAInstance(service.Service):
                            pkcs12_info=None, master_host=None, csr_file=None,
                            cert_file=None, cert_chain_file=None,
                            master_replication_port=None,
-                           subject_base=None):
+                           subject_base=None, ca_signing_algorithm=None,
+                           ca_type=None):
         """Create a CA instance.
 
            For Dogtag 9, this may involve creating the pki-ca instance.
@@ -438,6 +457,14 @@ class CAInstance(service.Service):
             self.subject_base = DN(('O', self.realm))
         else:
             self.subject_base = subject_base
+        if ca_signing_algorithm is None:
+            self.ca_signing_algorithm = 'SHA256withRSA'
+        else:
+            self.ca_signing_algorithm = ca_signing_algorithm
+        if ca_type is not None:
+            self.ca_type = ca_type
+        else:
+            self.ca_type = 'generic'
 
         # Determine if we are installing as an externally-signed CA and
         # what stage we're in.
@@ -449,7 +476,7 @@ class CAInstance(service.Service):
             self.cert_chain_file=cert_chain_file
             self.external=2
 
-        self.step("creating certificate server user", self.__create_ca_user)
+        self.step("creating certificate server user", create_ca_user)
         if self.dogtag_constants.DOGTAG_VERSION >= 10:
             self.step("configuring certificate server instance", self.__spawn_instance)
         else:
@@ -460,6 +487,7 @@ class CAInstance(service.Service):
         self.step("backing up CS.cfg", self.backup_config)
         self.step("disabling nonces", self.__disable_nonce)
         self.step("set up CRL publishing", self.__enable_crl_publish)
+        self.step("enable PKIX certificate path discovery and validation", self.enable_pkix)
         self.step("starting certificate server instance", self.__start)
         # Step 1 of external is getting a CSR so we don't need to do these
         # steps until we get a cert back from the external CA.
@@ -564,6 +592,9 @@ class CAInstance(service.Service):
         config.set("CA", "pki_audit_signing_nickname", "auditSigningCert cert-pki-ca")
         config.set("CA", "pki_ca_signing_nickname", "caSigningCert cert-pki-ca")
 
+        # CA key algorithm
+        config.set("CA", "pki_ca_signing_key_algorithm", self.ca_signing_algorithm)
+
         if (self.clone):
             cafile = self.pkcs12_info[0]
             shutil.copy(cafile, paths.TMP_CA_P12)
@@ -591,6 +622,13 @@ class CAInstance(service.Service):
             config.set("CA", "pki_external", "True")
             config.set("CA", "pki_external_csr_path", self.csr_file)
 
+            if self.ca_type == 'ms-cs':
+                # Include MS template name extension in the CSR
+                config.set("CA", "pki_req_ext_add", "True")
+                config.set("CA", "pki_req_ext_oid", "1.3.6.1.4.1.311.20.2")
+                config.set("CA", "pki_req_ext_critical", "False")
+                config.set("CA", "pki_req_ext_data", "1E0A00530075006200430041")
+
         elif self.external == 2:
             cert = x509.load_certificate_from_file(self.cert_file)
             cert_file = tempfile.NamedTemporaryFile()
@@ -637,8 +675,8 @@ class CAInstance(service.Service):
             os.remove(cfg_file)
 
         if self.external == 1:
-            print "The next step is to get %s signed by your CA and re-run ipa-server-install as:" % self.csr_file
-            print "ipa-server-install --external_cert_file=/path/to/signed_certificate --external_ca_file=/path/to/external_ca_certificate"
+            print "The next step is to get %s signed by your CA and re-run %s as:" % (self.csr_file, sys.argv[0])
+            print "%s --external-cert-file=/path/to/signed_certificate --external-cert-file=/path/to/external_ca_certificate" % sys.argv[0]
             sys.exit(0)
         else:
             shutil.move(paths.CA_BACKUP_KEYS_P12, \
@@ -683,22 +721,6 @@ class CAInstance(service.Service):
         # We need to install DS before we can actually ldap_enable a service.
         # so actual enablement is delayed.
 
-    def __create_ca_user(self):
-        try:
-            pwd.getpwnam(PKI_USER)
-            root_logger.debug("ca user %s exists" % PKI_USER)
-        except KeyError:
-            root_logger.debug("adding ca user %s" % PKI_USER)
-            args = [paths.USERADD, "-c", "CA System User",
-                                         "-d", paths.VAR_LIB,
-                                         "-s", paths.NOLOGIN,
-                                         "-M", "-r", PKI_USER]
-            try:
-                ipautil.run(args)
-                root_logger.debug("done adding user")
-            except ipautil.CalledProcessError, e:
-                root_logger.critical("failed to add user %s" % e)
-
     def __configure_instance(self):
         # Only used for Dogtag 9
         preop_pin = get_preop_pin(
@@ -727,7 +749,8 @@ class CAInstance(service.Service):
                     "-db_name", "ipaca",
                     "-key_size", "2048",
                     "-key_type", "rsa",
-                    "-key_algorithm", "SHA256withRSA",
+                    "-key_algorithm", self.ca_signing_algorithm,
+                    "-signing_algorithm", "SHA256withRSA",
                     "-save_p12", "true",
                     "-backup_pwd", self.admin_password,
                     "-subsystem_name", self.service_name,
@@ -801,8 +824,8 @@ class CAInstance(service.Service):
             raise RuntimeError('Configuration of CA failed')
 
         if self.external == 1:
-            print "The next step is to get %s signed by your CA and re-run ipa-server-install as:" % self.csr_file
-            print "ipa-server-install --external_cert_file=/path/to/signed_certificate --external_ca_file=/path/to/external_ca_certificate"
+            print "The next step is to get %s signed by your CA and re-run %s as:" % (self.csr_file, sys.argv[0])
+            print "%s --external-cert-file=/path/to/signed_certificate --external-cert-file=/path/to/external_ca_certificate" % sys.argv[0]
             sys.exit(0)
 
         # pkisilent makes a copy of the CA PKCS#12 file for us but gives
@@ -837,6 +860,11 @@ class CAInstance(service.Service):
         os.chown(self.dogtag_constants.CS_CFG_PATH,
                  pent.pw_uid, pent.pw_gid)
 
+    def enable_pkix(self):
+        installutils.set_directive(self.dogtag_constants.SYSCONFIG_FILE_PATH,
+                                   'NSS_ENABLE_PKIX_VERIFY', '1',
+                                   quotes=False, separator='=')
+
     def __issue_ra_cert(self):
         # The CA certificate is in the agent DB but isn't trusted
         (admin_fd, admin_name) = tempfile.mkstemp()
@@ -1073,10 +1101,12 @@ class CAInstance(service.Service):
                     (rdn, subject_dn) = certs.get_cert_nickname(certlist[st:en+25])
                     if subject_dn == ca_dn:
                         nick = get_ca_nickname(self.realm)
+                        trust_flags = 'CT,C,C'
                     else:
                         nick = str(subject_dn)
+                        trust_flags = ',,'
                     self.__run_certutil(
-                        ['-A', '-t', 'CT,C,C', '-n', nick, '-a',
+                        ['-A', '-t', trust_flags, '-n', nick, '-a',
                          '-i', chain_name]
                     )
                 finally:
@@ -1349,6 +1379,9 @@ class CAInstance(service.Service):
             installutils.set_directive(caconfig,
                  'internaldb.ldapconn.secureConn', 'true', quotes=False,
                  separator='=')
+            # Remove internaldb password as is not needed anymore
+            installutils.set_directive(self.dogtag_constants.PASSWORD_CONF_PATH,
+                                       'internaldb', None)
 
     def uninstall(self):
         if self.is_configured():
@@ -1391,6 +1424,16 @@ class CAInstance(service.Service):
         if path:
             iface.remove_known_ca(path)
 
+        helper = self.restore_state('certmonger_dogtag_helper')
+        if helper:
+            path = iface.find_ca_by_nickname('dogtag-ipa-renew-agent')
+            if path:
+                ca_obj = bus.get_object('org.fedorahosted.certmonger', path)
+                ca_iface = dbus.Interface(ca_obj,
+                                          'org.freedesktop.DBus.Properties')
+                ca_iface.Set('org.fedorahosted.certmonger.ca',
+                             'external-helper', helper)
+
         cmonger.stop()
 
         # remove CRL files
@@ -1450,6 +1493,32 @@ class CAInstance(service.Service):
                 'dogtag-ipa-ca-renew-agent',
                 paths.DOGTAG_IPA_CA_RENEW_AGENT_SUBMIT, [])
 
+        self.configure_certmonger_renewal_guard()
+
+    def configure_certmonger_renewal_guard(self):
+        if not self.is_configured():
+            return
+
+        bus = dbus.SystemBus()
+        obj = bus.get_object('org.fedorahosted.certmonger',
+                             '/org/fedorahosted/certmonger')
+        iface = dbus.Interface(obj, 'org.fedorahosted.certmonger')
+        path = iface.find_ca_by_nickname('dogtag-ipa-renew-agent')
+        if path:
+            ca_obj = bus.get_object('org.fedorahosted.certmonger', path)
+            ca_iface = dbus.Interface(ca_obj,
+                                      'org.freedesktop.DBus.Properties')
+            helper = ca_iface.Get('org.fedorahosted.certmonger.ca',
+                                  'external-helper')
+            if helper:
+                args = shlex.split(helper)
+                if args[0] != paths.IPA_SERVER_GUARD:
+                    self.backup_state('certmonger_dogtag_helper', helper)
+                    args = [paths.IPA_SERVER_GUARD] + args
+                    helper = ' '.join(pipes.quote(a) for a in args)
+                    ca_iface.Set('org.fedorahosted.certmonger.ca',
+                                 'external-helper', helper)
+
     def configure_agent_renewal(self):
         try:
             certmonger.dogtag_start_tracking(
@@ -1473,12 +1542,16 @@ class CAInstance(service.Service):
                 'Unable to determine PIN for CA instance: %s' % e)
 
     def configure_renewal(self):
+        reqs = (
+            ('auditSigningCert cert-pki-ca', None),
+            ('ocspSigningCert cert-pki-ca',  None),
+            ('subsystemCert cert-pki-ca',    None),
+            ('caSigningCert cert-pki-ca',    'ipaCACertRenewal'),
+        )
         pin = self.__get_ca_pin()
 
         # Server-Cert cert-pki-ca is renewed per-server
-        for nickname in ['auditSigningCert cert-pki-ca',
-                         'ocspSigningCert cert-pki-ca',
-                         'subsystemCert cert-pki-ca']:
+        for nickname, profile in reqs:
             try:
                 certmonger.dogtag_start_tracking(
                     ca='dogtag-ipa-ca-renew-agent',
@@ -1487,7 +1560,8 @@ class CAInstance(service.Service):
                     pinfile=None,
                     secdir=self.dogtag_constants.ALIAS_DIR,
                     pre_command='stop_pkicad',
-                    post_command='renew_ca_cert "%s"' % nickname)
+                    post_command='renew_ca_cert "%s"' % nickname,
+                    profile=profile)
             except RuntimeError, e:
                 root_logger.error(
                     "certmonger failed to start tracking certificate: %s" % e)
@@ -1498,16 +1572,17 @@ class CAInstance(service.Service):
         done by the renewal script, renew_ca_cert once all the subsystem
         certificates are renewed.
         """
+        nickname = 'Server-Cert cert-pki-ca'
         pin = self.__get_ca_pin()
         try:
             certmonger.dogtag_start_tracking(
                 ca='dogtag-ipa-renew-agent',
-                nickname='Server-Cert cert-pki-ca',
+                nickname=nickname,
                 pin=pin,
                 pinfile=None,
                 secdir=self.dogtag_constants.ALIAS_DIR,
-                pre_command=None,
-                post_command=None)
+                pre_command='stop_pkicad',
+                post_command='renew_ca_cert "%s"' % nickname)
         except RuntimeError, e:
             root_logger.error(
                 "certmonger failed to start tracking certificate: %s" % e)
@@ -1825,7 +1900,8 @@ def backup_config(dogtag_constants=None):
     if dogtag_constants is None:
         dogtag_constants = dogtag.configured_constants()
 
-    if services.knownservices.dogtag.is_running():
+    if services.knownservices[dogtag_constants.SERVICE_NAME].is_running(
+        dogtag_constants.PKI_INSTANCE_NAME):
         raise RuntimeError("Dogtag must be stopped when creating backup of %s"
                            % dogtag_constants.CS_CFG_PATH)
     shutil.copy(dogtag_constants.CS_CFG_PATH,
@@ -1879,23 +1955,15 @@ def update_people_entry(dercert):
     issuer = x509.get_issuer(dercert, datatype=x509.DER)
 
     attempts = 0
-    configured_constants = dogtag.configured_constants(api)
-    dogtag_uri = 'ldap://localhost:%d' % configured_constants.DS_PORT
+    server_id = dsinstance.realm_to_serverid(api.env.realm)
+    dogtag_uri = 'ldapi://%%2fvar%%2frun%%2fslapd-%s.socket' % server_id
     updated = False
 
-    try:
-        dm_password = certmonger.get_pin('internaldb')
-    except IOError, e:
-        syslog.syslog(
-            syslog.LOG_ERR, 'Unable to determine PIN for CA instance: %s' % e)
-        return False
-
     while attempts < 10:
         conn = None
         try:
             conn = ldap2.ldap2(shared_instance=False, ldap_uri=dogtag_uri)
-            conn.connect(
-                bind_dn=DN(('cn', 'directory manager')), bind_pw=dm_password)
+            conn.connect(autobind=True)
 
             filter = conn.make_filter(
                 {'description': ';%s;%s' % (issuer, subject)},
