@@ -25,9 +25,7 @@ import os
 import re
 import time
 import tempfile
-import base64
 import stat
-import grp
 
 from ipapython.ipa_log_manager import *
 from ipapython import ipautil, sysrestore, ipaldap
@@ -38,7 +36,7 @@ import ldap
 from ipaserver.install import ldapupdate
 from ipaserver.install import replication
 from ipaserver.install import sysupgrade
-from ipalib import errors
+from ipalib import errors, certstore
 from ipaplatform.tasks import tasks
 from ipalib.constants import CACERT
 from ipapython.dn import DN
@@ -56,10 +54,13 @@ IPA_SCHEMA_FILES = ("60kerberos.ldif",
                     "60ipaconfig.ldif",
                     "60basev2.ldif",
                     "60basev3.ldif",
+                    "60ipapk11.ldif",
                     "60ipadns.ldif",
                     "61kerberos-ipav3.ldif",
+                    "65ipacertstore.ldif",
                     "65ipasudo.ldif",
                     "70ipaotp.ldif",
+                    "71idviews.ldif",
                     "15rfc2307bis.ldif",
                     "15rfc4876.ldif")
 
@@ -150,50 +151,15 @@ def is_ds_running(server_id=''):
 
 
 def create_ds_user():
-    """
-    Create DS user if it doesn't exist yet.
-    """
-    try:
-        pwd.getpwnam(DS_USER)
-        root_logger.debug('DS user %s exists', DS_USER)
-    except KeyError:
-        root_logger.debug('Adding DS user %s', DS_USER)
-        args = [
-            paths.USERADD,
-            '-g', DS_GROUP,
-            '-c', 'DS System User',
-            '-d', paths.VAR_LIB_DIRSRV,
-            '-s', paths.NOLOGIN,
-            '-M', '-r', DS_USER
-        ]
-        try:
-            ipautil.run(args)
-            root_logger.debug('Done adding DS user')
-        except ipautil.CalledProcessError, e:
-            root_logger.critical('Failed to add DS user: %s', e)
+    """Create DS user/group if it doesn't exist yet."""
+    tasks.create_system_user(
+        name=DS_USER,
+        group=DS_USER,
+        homedir=paths.VAR_LIB_DIRSRV,
+        shell=paths.NOLOGIN,
+    )
 
 
-def create_ds_group():
-    """
-    Create DS group if it doesn't exist yet.
-    Returns True if the group already exists.
-    """
-    try:
-        grp.getgrnam(DS_GROUP)
-        root_logger.debug('DS group %s exists', DS_GROUP)
-        group_exists = True
-    except KeyError:
-        group_exists = False
-        root_logger.debug('Adding DS group %s', DS_GROUP)
-        args = [paths.GROUPADD, '-r', DS_GROUP]
-        try:
-            ipautil.run(args)
-            root_logger.debug('Done adding DS group')
-        except ipautil.CalledProcessError, e:
-            root_logger.critical('Failed to add DS group: %s', e)
-
-    return group_exists
-
 INF_TEMPLATE = """
 [General]
 FullMachineName=   $FQDN
@@ -226,7 +192,7 @@ class DsInstance(service.Service):
             service_desc="directory server",
             dm_password=dm_password,
             ldapi=False,
-            autobind=service.DISABLED
+            autobind=ipaldap.AUTOBIND_DISABLED
             )
         self.nickname = 'Server-Cert'
         self.dm_password = dm_password
@@ -234,6 +200,7 @@ class DsInstance(service.Service):
         self.sub_dict = None
         self.domain = domain_name
         self.serverid = None
+        self.master_fqdn = None
         self.pkcs12_info = None
         self.cacert_name = None
         self.ca_is_configured = True
@@ -381,6 +348,7 @@ class DsInstance(service.Service):
         # See LDIFs for automember configuration during replica install
         self.step("setting Auto Member configuration", self.__add_replica_automember_config)
         self.step("enabling S4U2Proxy delegation", self.__setup_s4u2proxy)
+        self.step("importing CA certificates from LDAP", self.__import_ca_certs)
 
         self.__common_post_setup()
 
@@ -569,7 +537,7 @@ class DsInstance(service.Service):
 
     def __config_uuid_module(self):
         self._ldap_mod("uuid-conf.ldif")
-        self._ldap_mod("uuid-ipauniqueid.ldif", self.sub_dict)
+        self._ldap_mod("uuid.ldif", self.sub_dict)
 
     def __config_modrdn_module(self):
         self._ldap_mod("modrdn-conf.ldif")
@@ -621,8 +589,13 @@ class DsInstance(service.Service):
         dirname = config_dirname(self.serverid)
         dsdb = certs.CertDB(self.realm, nssdir=dirname, subject_base=self.subject_base)
         if self.pkcs12_info:
+            if self.ca_is_configured:
+                trust_flags = 'CT,C,C'
+            else:
+                trust_flags = None
             dsdb.create_from_pkcs12(self.pkcs12_info[0], self.pkcs12_info[1],
-                                    ca_file=self.ca_file)
+                                    ca_file=self.ca_file,
+                                    trust_flags=trust_flags)
             server_certs = dsdb.find_server_certs()
             if len(server_certs) == 0:
                 raise RuntimeError("Could not find a suitable server cert in import in %s" % self.pkcs12_info[0])
@@ -679,28 +652,37 @@ class DsInstance(service.Service):
         """
 
         dirname = config_dirname(self.serverid)
-        certdb = certs.CertDB(self.realm, nssdir=dirname,
-                              subject_base=self.subject_base)
-
-        dercert = certdb.get_cert_from_db(self.cacert_name, pem=False)
+        dsdb = certs.CertDB(self.realm, nssdir=dirname,
+                            subject_base=self.subject_base)
+        trust_flags = dict(reversed(dsdb.list_certs()))
 
         conn = ipaldap.IPAdmin(self.fqdn)
         conn.do_simple_bind(DN(('cn', 'directory manager')), self.dm_password)
 
-        dn = DN(('cn', 'CAcert'), ('cn', 'ipa'), ('cn', 'etc'), self.suffix)
-        try:
-            entry = conn.get_entry(dn, attrs_list=['cACertificate;binary'])
-            entry['cACertificate;binary'] = [dercert]
-            conn.update_entry(entry)
-        except errors.NotFound:
-            entry = conn.make_entry(
-                dn,
-                {'objectClass': ['nsContainer', 'pkiCA'],
-                 'cn': ['CAcert'],
-                 'cACertificate;binary': [dercert]})
-            conn.add_entry(entry)
-        except errors.EmptyModlist:
-            pass
+        nicknames = dsdb.find_root_cert(self.cacert_name)[:-1]
+        for nickname in nicknames:
+            cert = dsdb.get_cert_from_db(nickname, pem=False)
+            certstore.put_ca_cert_nss(conn, self.suffix, cert, nickname,
+                                      trust_flags[nickname])
+
+        nickname = self.cacert_name
+        cert = dsdb.get_cert_from_db(nickname, pem=False)
+        certstore.put_ca_cert_nss(conn, self.suffix, cert, nickname,
+                                  trust_flags[nickname],
+                                  config_ipa=self.ca_is_configured,
+                                  config_compat=self.master_fqdn is None)
+
+        conn.unbind()
+
+    def __import_ca_certs(self):
+        dirname = config_dirname(self.serverid)
+        dsdb = certs.CertDB(self.realm, nssdir=dirname,
+                            subject_base=self.subject_base)
+
+        conn = ipaldap.IPAdmin(self.fqdn)
+        conn.do_simple_bind(DN(('cn', 'directory manager')), self.dm_password)
+
+        self.import_ca_certs(dsdb, self.ca_is_configured, conn)
 
         conn.unbind()
 
@@ -859,7 +841,7 @@ class DsInstance(service.Service):
         certdb.cacert_name = cacert_name
         status = True
         try:
-            certdb.load_cacert(cacert_fname)
+            certdb.load_cacert(cacert_fname, 'C,,')
         except ipautil.CalledProcessError, e:
             root_logger.critical("Error importing CA cert file named [%s]: %s" %
                                          (cacert_fname, str(e)))
