@@ -113,6 +113,7 @@ class Backup(admintool.AdminTool):
         paths.VAR_LIB_PKI_CA_DIR,
         paths.SYSRESTORE,
         paths.IPA_CLIENT_SYSRESTORE,
+        paths.IPA_DNSSEC_DIR,
         paths.SSSD_PUBCONF_KRB5_INCLUDE_D_DIR,
         paths.AUTHCONFIG_LAST,
         paths.VAR_LIB_CERTMONGER_DIR,
@@ -131,10 +132,14 @@ class Backup(admintool.AdminTool):
         paths.SYSCONFIG_NTPD,
         paths.SYSCONFIG_KRB5KDC_DIR,
         paths.SYSCONFIG_PKI_CA_PKI_CA_DIR,
+        paths.SYSCONFIG_IPA_DNSKEYSYNCD,
+        paths.SYSOCNFIG_IPA_ODS_EXPORTER,
+        paths.SYSCONFIG_NAMED,
+        paths.SYSCONFIG_ODS,
         paths.ETC_SYSCONFIG_AUTHCONFIG,
-        paths.NSSDB_CERT8_DB,
-        paths.NSSDB_KEY3_DB,
-        paths.NSSDB_SECMOD_DB,
+        paths.IPA_NSSDB_PWDFILE_TXT,
+        paths.IPA_P11_KIT,
+        paths.SYSTEMWIDE_IPA_CA_CRT,
         paths.NSSWITCH_CONF,
         paths.KRB5_KEYTAB,
         paths.SSSD_CONF,
@@ -149,8 +154,6 @@ class Backup(admintool.AdminTool):
         paths.SSHD_CONFIG,
         paths.SSH_CONFIG,
         paths.KRB5_CONF,
-        paths.GROUP,
-        paths.PASSWD,
         CACERT,
         paths.IPA_DEFAULT_CONF,
         paths.DS_KEYTAB,
@@ -165,6 +168,16 @@ class Backup(admintool.AdminTool):
         paths.SYSTEMD_CERTMONGER_SERVICE,
         paths.SYSTEMD_PKI_TOMCAT_SERVICE,
         paths.SVC_LIST_FILE,
+        paths.OPENDNSSEC_CONF_FILE,
+        paths.OPENDNSSEC_KASP_FILE,
+        paths.DNSSEC_SOFTHSM2_CONF,
+        paths.DNSSEC_SOFTHSM_PIN_SO,
+        paths.IPA_ODS_EXPORTER_KEYTAB,
+        paths.IPA_DNSKEYSYNCD_KEYTAB,
+    ) + tuple(
+        os.path.join(base, file)
+        for base in (paths.NSS_DB_DIR, paths.IPA_NSSDB_DIR)
+        for file in ('cert8.db', 'key3.db', 'secmod.db')
     )
 
     logs=(
@@ -279,7 +292,7 @@ class Backup(admintool.AdminTool):
 
             for instance in [realm_to_serverid(api.env.realm), 'PKI-IPA']:
                 if os.path.exists(paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE % instance):
-                    if os.path.exists(paths.IPACA_DIRSRV_INSTANCE_DB_TEMPLATE % instance):
+                    if os.path.exists(paths.SLAPD_INSTANCE_DB_DIR_TEMPLATE % (instance, 'ipaca')):
                         self.db2ldif(instance, 'ipaca', online=options.online)
                     self.db2ldif(instance, 'userRoot', online=options.online)
                     self.db2bak(instance, online=options.online)
@@ -555,6 +568,8 @@ class Backup(admintool.AdminTool):
 
         shutil.move(self.header, backup_dir)
 
+        self.log.info('Backed up to %s', backup_dir)
+
     def __find_scripts_dir(self, instance):
         """
         IPA stores its 389-ds scripts in a different directory than dogtag
