@@ -24,24 +24,33 @@ import tempfile
 import time
 import pwd
 from ConfigParser import SafeConfigParser
+import ldif
+import itertools
 
-from ipalib import api, errors
-from ipapython import version
+from ipalib import api, errors, constants
+from ipapython import version, ipautil, certdb, dogtag
 from ipapython.ipautil import run, user_input
 from ipapython import admintool
 from ipapython.dn import DN
-from ipaserver.install.dsinstance import (realm_to_serverid, create_ds_group,
+from ipaserver.install.dsinstance import (realm_to_serverid,
                                           create_ds_user, DS_USER)
-from ipaserver.install.cainstance import PKI_USER
+from ipaserver.install.cainstance import PKI_USER, create_ca_user
 from ipaserver.install.replication import (wait_for_task, ReplicationManager,
                                            get_cs_replication_manager)
 from ipaserver.install import installutils
+from ipaserver.install import dsinstance, httpinstance, cainstance
 from ipapython import ipaldap
+import ipapython.errors
 from ipaplatform.tasks import tasks
 from ipaserver.install.ipa_backup import BACKUP_DIR
 from ipaplatform import services
 from ipaplatform.paths import paths
 
+try:
+    from ipaserver.install import adtrustinstance
+except ImportError:
+    adtrustinstance = None
+
 
 def recursive_chown(path, uid, gid):
     '''
@@ -87,6 +96,32 @@ def decrypt_file(tmpdir, filename, keyring):
     return dest
 
 
+class RemoveRUVParser(ldif.LDIFParser):
+    def __init__(self, input_file, writer, logger):
+        ldif.LDIFParser.__init__(self, input_file)
+        self.writer = writer
+        self.log = logger
+
+    def handle(self, dn, entry):
+        objectclass = None
+        nsuniqueid = None
+
+        for name, value in entry.iteritems():
+            name = name.lower()
+            if name == 'objectclass':
+                objectclass = [x.lower() for x in value]
+            elif name == 'nsuniqueid':
+                nsuniqueid = [x.lower() for x in value]
+
+        if (objectclass and nsuniqueid and
+            'nstombstone' in objectclass and
+            'ffffffff-ffffffff-ffffffff-ffffffff' in nsuniqueid):
+            self.log.debug("Removing RUV entry %s", dn)
+            return
+
+        self.writer.unparse(dn, entry)
+
+
 class Restore(admintool.AdminTool):
     command_name = 'ipa-restore'
     log_file_name = paths.IPARESTORE_LOG
@@ -127,29 +162,25 @@ class Restore(admintool.AdminTool):
 
 
     def validate_options(self):
+        parser = self.option_parser
         options = self.options
         super(Restore, self).validate_options(needs_root=True)
-        if options.data_only:
-            installutils.check_server_configuration()
 
         if len(self.args) < 1:
-            self.option_parser.error(
-                "must provide the backup to restore")
+            parser.error("must provide the backup to restore")
         elif len(self.args) > 1:
-            self.option_parser.error(
-                "must provide exactly one name for the backup")
+            parser.error("must provide exactly one name for the backup")
 
         dirname = self.args[0]
         if not os.path.isabs(dirname):
-            self.backup_dir = os.path.join(BACKUP_DIR, dirname)
-        else:
-            self.backup_dir = dirname
+            dirname = os.path.join(BACKUP_DIR, dirname)
+        if not os.path.isdir(dirname):
+            parser.error("must provide path to backup directory")
 
         if options.gpg_keyring:
             if (not os.path.exists(options.gpg_keyring + '.pub') or
-               not os.path.exists(options.gpg_keyring + '.sec')):
-                raise admintool.ScriptError('No such key %s' %
-                    options.gpg_keyring)
+                    not os.path.exists(options.gpg_keyring + '.sec')):
+                parser.error("no such key %s" % options.gpg_keyring)
 
 
     def ask_for_options(self):
@@ -172,23 +203,89 @@ class Restore(admintool.AdminTool):
         options = self.options
         super(Restore, self).run()
 
-        api.bootstrap(in_server=False, context='restore')
-        api.finalize()
+        self.backup_dir = self.args[0]
+        if not os.path.isabs(self.backup_dir):
+            self.backup_dir = os.path.join(BACKUP_DIR, self.backup_dir)
 
         self.log.info("Preparing restore from %s on %s",
-            self.backup_dir, api.env.host)
+                      self.backup_dir, constants.FQDN)
 
-        if not options.instance:
-            instances = []
-            for instance in [realm_to_serverid(api.env.realm), 'PKI-IPA']:
-                if os.path.exists(paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE % instance):
-                    instances.append(instance)
+        self.header = os.path.join(self.backup_dir, 'header')
+
+        try:
+            self.read_header()
+        except IOError as e:
+            raise admintool.ScriptError("Cannot read backup metadata: %s" % e)
+
+        if options.data_only:
+            restore_type = 'DATA'
         else:
-            instances = [options.instance]
-        if options.data_only and not instances:
-            raise admintool.ScriptError('No instances to restore to')
+            restore_type = self.backup_type
+
+        # These checks would normally be in the validate method but
+        # we need to know the type of backup we're dealing with.
+        if restore_type == 'FULL':
+            if options.online:
+                raise admintool.ScriptError(
+                    "File restoration cannot be done online")
+            if options.instance or options.backend:
+                raise admintool.ScriptError(
+                    "Restore must be in data-only mode when restoring a "
+                    "specific instance or backend")
+        else:
+            installutils.check_server_configuration()
+
+            self.init_api()
+
+            if options.instance:
+                instance_dir = (paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE %
+                                options.instance)
+                if not os.path.exists(instance_dir):
+                    raise admintool.ScriptError(
+                        "Instance %s does not exist" % options.instance)
+
+                self.instances = [options.instance]
+
+            if options.backend:
+                for instance in self.instances:
+                    db_dir = (paths.SLAPD_INSTANCE_DB_DIR_TEMPLATE %
+                              (instance, options.backend))
+                    if os.path.exists(db_dir):
+                        break
+                else:
+                    raise admintool.ScriptError(
+                        "Backend %s does not exist" % options.backend)
+
+                self.backends = [options.backend]
+
+            for instance, backend in itertools.product(self.instances,
+                                                       self.backends):
+                db_dir = (paths.SLAPD_INSTANCE_DB_DIR_TEMPLATE %
+                          (instance, backend))
+                if os.path.exists(db_dir):
+                    break
+            else:
+                raise admintool.ScriptError(
+                    "Cannot restore a data backup into an empty system")
+
+        self.log.info("Performing %s restore from %s backup" %
+                      (restore_type, self.backup_type))
+
+        if self.backup_host != constants.FQDN:
+            raise admintool.ScriptError(
+                "Host name %s does not match backup name %s" %
+                (constants.FQDN, self.backup_host))
+
+        if self.backup_ipa_version != str(version.VERSION):
+            self.log.warning(
+                "Restoring data from a different release of IPA.\n"
+                "Data is version %s.\n"
+                "Server is running %s." %
+                (self.backup_ipa_version, str(version.VERSION)))
+            if (not options.unattended and
+                    not user_input("Continue to restore?", False)):
+                raise admintool.ScriptError("Aborted")
 
-        create_ds_group()
         create_ds_user()
         pent = pwd.getpwnam(DS_USER)
 
@@ -201,36 +298,39 @@ class Restore(admintool.AdminTool):
 
         os.chown(self.dir, pent.pw_uid, pent.pw_gid)
 
-        self.header = os.path.join(self.backup_dir, 'header')
-
         cwd = os.getcwd()
         try:
             dirsrv = services.knownservices.dirsrv
 
-            self.read_header()
-            # These two checks would normally be in the validate method but
-            # we need to know the type of backup we're dealing with.
-            if (self.backup_type != 'FULL' and not options.data_only and
-                not instances):
-                raise admintool.ScriptError('Cannot restore a data backup into an empty system')
-            if (self.backup_type == 'FULL' and not options.data_only and
-                (options.instance or options.backend)):
-                raise admintool.ScriptError('Restore must be in data-only mode when restoring a specific instance or backend.')
-            if self.backup_host != api.env.host:
-                self.log.warning('Host name %s does not match backup name %s' %
-                    (api.env.host, self.backup_host))
-                if (not options.unattended and
-                    not user_input("Continue to restore?", False)):
-                    raise admintool.ScriptError("Aborted")
-            if self.backup_ipa_version != str(version.VERSION):
-                self.log.warning(
-                    "Restoring data from a different release of IPA.\n"
-                    "Data is version %s.\n"
-                    "Server is running %s." %
-                    (self.backup_ipa_version, str(version.VERSION)))
-                if (not options.unattended and
-                    not user_input("Continue to restore?", False)):
-                    raise admintool.ScriptError("Aborted")
+            self.extract_backup(options.gpg_keyring)
+
+            if restore_type == 'FULL':
+                self.restore_default_conf()
+                self.init_api(confdir=self.dir + paths.ETC_IPA)
+
+            databases = []
+            for instance in self.instances:
+                for backend in self.backends:
+                    database = (instance, backend)
+                    ldiffile = os.path.join(self.dir, '%s-%s.ldif' % database)
+                    if os.path.exists(ldiffile):
+                        databases.append(database)
+
+            if options.instance:
+                for instance, backend in databases:
+                    if instance == options.instance:
+                        break
+                else:
+                    raise admintool.ScriptError(
+                        "Instance %s not found in backup" % options.instance)
+
+            if options.backend:
+                for instance, backend in databases:
+                    if backend == options.backend:
+                        break
+                else:
+                    raise admintool.ScriptError(
+                        "Backend %s not found in backup" % options.backend)
 
             # Big fat warning
             if  (not options.unattended and
@@ -249,8 +349,7 @@ class Restore(admintool.AdminTool):
             self.log.info("Disabling all replication.")
             self.disable_agreements()
 
-            self.extract_backup(options.gpg_keyring)
-            if options.data_only:
+            if restore_type != 'FULL':
                 if not options.online:
                     self.log.info('Stopping Directory Server')
                     dirsrv.stop(capture_output=False)
@@ -263,12 +362,16 @@ class Restore(admintool.AdminTool):
                 if rc not in [0, 6]:
                     self.log.warn('Stopping IPA failed: %s' % stderr)
 
+                self.restore_selinux_booleans()
+
 
             # We do either a full file restore or we restore data.
-            if self.backup_type == 'FULL' and not options.data_only:
-                if options.online:
-                    raise admintool.ScriptError('File restoration cannot be done online.')
+            if restore_type == 'FULL':
+                if 'CA' in self.backup_services:
+                    create_ca_user()
+                self.cert_restore_prepare()
                 self.file_restore(options.no_logs)
+                self.cert_restore()
                 if 'CA' in self.backup_services:
                     self.__create_dogtag_log_dirs()
 
@@ -277,18 +380,10 @@ class Restore(admintool.AdminTool):
             # userRoot backend in it and the main IPA instance. If we
             # have a unified instance we need to restore both userRoot and
             # ipaca.
-            for instance in instances:
-                if os.path.exists(paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE % instance):
-                    if options.backend is None:
-                        self.ldif2db(instance, 'userRoot', online=options.online)
-                        if os.path.exists(paths.IPACA_DIRSRV_INSTANCE_DB_TEMPLATE % instance):
-                            self.ldif2db(instance, 'ipaca', online=options.online)
-                    else:
-                        self.ldif2db(instance, options.backend, online=options.online)
-                else:
-                    raise admintool.ScriptError('389-ds instance %s does not exist' % instance)
+            for instance, backend in databases:
+                self.ldif2db(instance, backend, online=options.online)
 
-            if options.data_only:
+            if restore_type != 'FULL':
                 if not options.online:
                     self.log.info('Starting Directory Server')
                     dirsrv.start(capture_output=False)
@@ -303,6 +398,8 @@ class Restore(admintool.AdminTool):
                 self.log.info('Restarting SSSD')
                 sssd = services.service('sssd')
                 sssd.restart()
+                http = httpinstance.HTTPInstance()
+                http.remove_httpd_ccache()
         finally:
             try:
                 os.chdir(cwd)
@@ -362,6 +459,7 @@ class Restore(admintool.AdminTool):
                                           self.dirman_password)
             except Exception, e:
                 self.log.critical("Unable to disable agreement on %s: %s" % (master, e))
+                continue
 
             master_dn = DN(('cn', master), ('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'), api.env.basedn)
             try:
@@ -386,6 +484,7 @@ class Restore(admintool.AdminTool):
                                                       self.dirman_password)
                 except Exception, e:
                     self.log.critical("Unable to disable agreement on %s: %s" % (master, e))
+                    continue
 
                 host_entries = repl.find_ipa_replication_agreements()
                 hosts = [rep.single_value.get('nsds5replicahost')
@@ -408,8 +507,22 @@ class Restore(admintool.AdminTool):
         cn = time.strftime('import_%Y_%m_%d_%H_%M_%S')
         dn = DN(('cn', cn), ('cn', 'import'), ('cn', 'tasks'), ('cn', 'config'))
 
+        ldifdir = paths.SLAPD_INSTANCE_LDIF_DIR_TEMPLATE % instance
         ldifname = '%s-%s.ldif' % (instance, backend)
-        ldiffile = os.path.join(self.dir, ldifname)
+        ldiffile = os.path.join(ldifdir, ldifname)
+        srcldiffile = os.path.join(self.dir, ldifname)
+
+        if not os.path.exists(ldifdir):
+            pent = pwd.getpwnam(DS_USER)
+            os.mkdir(ldifdir, 0770)
+            os.chown(ldifdir, pent.pw_uid, pent.pw_gid)
+
+        ipautil.backup_file(ldiffile)
+        with open(ldiffile, 'wb') as out_file:
+            ldif_writer = ldif.LDIFWriter(out_file)
+            with open(srcldiffile, 'rb') as in_file:
+                ldif_parser = RemoveRUVParser(in_file, ldif_writer, self.log)
+                ldif_parser.parse()
 
         if online:
             conn = self.get_connection()
@@ -427,20 +540,15 @@ class Restore(admintool.AdminTool):
             try:
                 conn.add_entry(ent)
             except Exception, e:
-                raise admintool.ScriptError(
-                    'Unable to bind to LDAP server: %s' % e)
+                self.log.error("Unable to bind to LDAP server: %s" % e)
+                return
 
             self.log.info("Waiting for LDIF to finish")
             wait_for_task(conn, dn)
         else:
             args = ['%s/ldif2db' % self.__find_scripts_dir(instance),
-                    '-i', ldiffile]
-            if backend is not None:
-                args.append('-n')
-                args.append(backend)
-            else:
-                args.append('-n')
-                args.append('userRoot')
+                    '-i', ldiffile,
+                    '-n', backend]
             (stdout, stderr, rc) = run(args, raiseonerr=False)
             if rc != 0:
                 self.log.critical("ldif2db failed: %s" % stderr)
@@ -500,6 +608,30 @@ class Restore(admintool.AdminTool):
                 self.log.critical("bak2db failed: %s" % stderr)
 
 
+    def restore_default_conf(self):
+        '''
+        Restore paths.IPA_DEFAULT_CONF to temporary directory.
+
+        Primary purpose of this method is to get cofiguration for api
+        finalization when restoring ipa after uninstall.
+        '''
+        cwd = os.getcwd()
+        os.chdir(self.dir)
+        args = ['tar',
+                '--xattrs',
+                '--selinux',
+                '-xzf',
+                os.path.join(self.dir, 'files.tar'),
+                paths.IPA_DEFAULT_CONF[1:],
+               ]
+
+        (stdout, stderr, rc) = run(args, raiseonerr=False)
+        if rc != 0:
+            self.log.critical('Restoring %s failed: %s' %
+                              (paths.IPA_DEFAULT_CONF, stderr))
+        os.chdir(cwd)
+
+
     def file_restore(self, nologs=False):
         '''
         Restore all the files in the tarball.
@@ -511,6 +643,8 @@ class Restore(admintool.AdminTool):
         cwd = os.getcwd()
         os.chdir('/')
         args = ['tar',
+                '--xattrs',
+                '--selinux',
                 '-xzf',
                 os.path.join(self.dir, 'files.tar')
                ]
@@ -530,16 +664,16 @@ class Restore(admintool.AdminTool):
         Read the backup file header that contains the meta data about
         this particular backup.
         '''
-        fd = open(self.header)
-        config = SafeConfigParser()
-        config.readfp(fd)
+        with open(self.header) as fd:
+            config = SafeConfigParser()
+            config.readfp(fd)
 
         self.backup_type = config.get('ipa', 'type')
         self.backup_time = config.get('ipa', 'time')
         self.backup_host = config.get('ipa', 'host')
         self.backup_ipa_version = config.get('ipa', 'ipa_version')
         self.backup_version = config.get('ipa', 'version')
-        self.backup_services = config.get('ipa', 'services')
+        self.backup_services = config.get('ipa', 'services').split(',')
 
 
     def extract_backup(self, keyring=None):
@@ -569,6 +703,8 @@ class Restore(admintool.AdminTool):
         os.chdir(self.dir)
 
         args = ['tar',
+                '--xattrs',
+                '--selinux',
                 '-xzf',
                 filename,
                 '.'
@@ -604,22 +740,21 @@ class Restore(admintool.AdminTool):
         not exist then tomcat will fail to start.
 
         The directory is different depending on whether we have a d9-based
-        or a d10-based installation. We can tell based on whether there is
-        a PKI-IPA 389-ds instance.
+        or a d10-based installation.
         """
-        if os.path.exists(paths.ETC_SLAPD_PKI_IPA_DIR): # dogtag 9
-            topdir = paths.PKI_CA_LOG_DIR
-            dirs = [topdir,
-                    '/var/log/pki-ca/signedAudit,']
-        else: # dogtag 10
-            topdir = paths.TOMCAT_TOPLEVEL_DIR
-            dirs = [topdir,
-                    paths.TOMCAT_CA_DIR,
-                    paths.TOMCAT_CA_ARCHIVE_DIR,
-                    paths.TOMCAT_SIGNEDAUDIT_DIR,]
-
-        if os.path.exists(topdir):
-            return
+        dirs = []
+        # dogtag 9
+        if (os.path.exists(paths.VAR_LIB_PKI_CA_DIR) and
+                not os.path.exists(paths.PKI_CA_LOG_DIR)):
+            dirs += [paths.PKI_CA_LOG_DIR,
+                     os.path.join(paths.PKI_CA_LOG_DIR, 'signedAudit')]
+        # dogtag 10
+        if (os.path.exists(paths.VAR_LIB_PKI_TOMCAT_DIR) and
+                not os.path.exists(paths.TOMCAT_TOPLEVEL_DIR)):
+            dirs += [paths.TOMCAT_TOPLEVEL_DIR,
+                     paths.TOMCAT_CA_DIR,
+                     paths.TOMCAT_CA_ARCHIVE_DIR,
+                     paths.TOMCAT_SIGNEDAUDIT_DIR]
 
         try:
             pent = pwd.getpwnam(PKI_USER)
@@ -636,3 +771,67 @@ class Restore(admintool.AdminTool):
             except Exception, e:
                 # This isn't so fatal as to side-track the restore
                 self.log.error('Problem with %s: %s' % (dir, e))
+
+    def restore_selinux_booleans(self):
+        bools = dict(httpinstance.SELINUX_BOOLEAN_SETTINGS)
+        if 'ADTRUST' in self.backup_services:
+            if adtrustinstance:
+                bools.update(adtrustinstance.SELINUX_BOOLEAN_SETTINGS)
+            else:
+                self.log.error(
+                    'The AD trust package was not found, '
+                    'not setting SELinux booleans.')
+        try:
+            tasks.set_selinux_booleans(bools)
+        except ipapython.errors.SetseboolError as e:
+            self.log.error('%s', e)
+
+    def cert_restore_prepare(self):
+        cainstance.stop_tracking_certificates(
+            dogtag.configured_constants())
+        httpinstance.HTTPInstance().stop_tracking_certificates()
+        try:
+            dsinstance.DsInstance().stop_tracking_certificates(
+                realm_to_serverid(api.env.realm))
+        except OSError:
+            # When IPA is not installed, DS NSS DB does not exist
+            pass
+
+        for basename in ('cert8.db', 'key3.db', 'secmod.db', 'pwdfile.txt'):
+            filename = os.path.join(paths.IPA_NSSDB_DIR, basename)
+            try:
+                ipautil.backup_file(filename)
+            except OSError as e:
+                self.log.error("Failed to backup %s: %s" % (filename, e))
+
+        tasks.remove_ca_certs_from_systemwide_ca_store()
+
+    def cert_restore(self):
+        if not os.path.exists(os.path.join(paths.IPA_NSSDB_DIR, 'cert8.db')):
+            certdb.create_ipa_nssdb()
+            ipa_db = certdb.NSSDatabase(paths.IPA_NSSDB_DIR)
+            sys_db = certdb.NSSDatabase(paths.NSS_DB_DIR)
+            for nickname, trust_flags in (('IPA CA', 'CT,C,C'),
+                                          ('External CA cert', 'C,,')):
+                try:
+                    cert = sys_db.get_cert(nickname)
+                except RuntimeError:
+                    pass
+                else:
+                    try:
+                        ipa_db.add_cert(cert, nickname, trust_flags)
+                    except ipautil.CalledProcessError as e:
+                        self.log.error(
+                            "Failed to add %s to %s: %s" %
+                            (nickname, paths.IPA_NSSDB_DIR, e))
+
+        tasks.reload_systemwide_ca_store()
+
+        services.knownservices.certmonger.restart()
+
+    def init_api(self, **overrides):
+        api.bootstrap(in_server=False, context='restore', **overrides)
+        api.finalize()
+
+        self.instances = [realm_to_serverid(api.env.realm), 'PKI-IPA']
+        self.backends = ['userRoot', 'ipaca']
