@@ -22,8 +22,6 @@
 # TODO
 # save undo files?
 
-UPDATES_DIR="/usr/share/ipa/updates/"
-
 import sys
 import uuid
 import platform
@@ -32,22 +30,53 @@ import os
 import pwd
 import fnmatch
 import csv
-import inspect
 import re
 
 import krbV
 import ldap
-from ldap.schema.models import ObjectClass, AttributeType
 
 from ipaserver.install import installutils
 from ipapython import ipautil, ipaldap
 from ipalib import errors
 from ipalib import api
+from ipaplatform.paths import paths
 from ipapython.dn import DN
 from ipapython.ipa_log_manager import *
-from ipaserver.install.plugins import PRE_UPDATE, POST_UPDATE
+from ipaserver.install.plugins import (PRE_UPDATE, POST_UPDATE,
+                                       PRE_SCHEMA_UPDATE)
 from ipaserver.plugins import ldap2
 
+UPDATES_DIR=paths.UPDATES_DIR
+
+
+def connect(ldapi=False, realm=None, fqdn=None, dm_password=None, pw_name=None):
+    """Create a connection for updates"""
+    if ldapi:
+        conn = ipaldap.IPAdmin(ldapi=True, realm=realm, decode_attrs=False)
+    else:
+        conn = ipaldap.IPAdmin(fqdn, ldapi=False, realm=realm, decode_attrs=False)
+    try:
+        if dm_password:
+            conn.do_simple_bind(binddn=DN(('cn', 'directory manager')),
+                                bindpw=dm_password)
+        elif os.getegid() == 0:
+            try:
+                # autobind
+                conn.do_external_bind(pw_name)
+            except errors.NotFound:
+                # Fall back
+                conn.do_sasl_gssapi_bind()
+        else:
+            conn.do_sasl_gssapi_bind()
+    except (ldap.CONNECT_ERROR, ldap.SERVER_DOWN):
+        raise RuntimeError("Unable to connect to LDAP server %s" % fqdn)
+    except ldap.INVALID_CREDENTIALS:
+        raise RuntimeError(
+            "The password provided is incorrect for LDAP server %s" % fqdn)
+    except ldap.LOCAL_ERROR, e:
+        raise RuntimeError('%s' % e.args[0].get('info', '').strip())
+    return conn
+
 
 class BadSyntax(installutils.ScriptError):
     def __init__(self, value):
@@ -187,26 +216,10 @@ class LDAPUpdate:
 
         if online:
             # Try out the connection/password
-            try:
-                conn = ipaldap.IPAdmin(fqdn, ldapi=self.ldapi, realm=self.realm)
-                if self.dm_password:
-                    conn.do_simple_bind(binddn=DN(('cn', 'directory manager')), bindpw=self.dm_password)
-                elif os.getegid() == 0:
-                    try:
-                        # autobind
-                        conn.do_external_bind(self.pw_name)
-                    except errors.NotFound:
-                        # Fall back
-                        conn.do_sasl_gssapi_bind()
-                else:
-                    conn.do_sasl_gssapi_bind()
-                conn.unbind()
-            except (ldap.CONNECT_ERROR, ldap.SERVER_DOWN):
-                raise RuntimeError("Unable to connect to LDAP server %s" % fqdn)
-            except ldap.INVALID_CREDENTIALS:
-                raise RuntimeError("The password provided is incorrect for LDAP server %s" % fqdn)
-            except ldap.LOCAL_ERROR, e:
-                raise RuntimeError('%s' % e.args[0].get('info', '').strip())
+            # (This will raise if the server is not available)
+            self.create_connection()
+            self.conn.unbind()
+            self.conn = None
         else:
             raise RuntimeError("Offline updates are not supported.")
 
@@ -224,8 +237,7 @@ class LDAPUpdate:
                                 skipinitialspace=True,
                                 **kwargs)
         for row in csv_reader:
-            # decode UTF-8 back to Unicode, cell by cell:
-            yield [unicode(cell, 'utf-8') for cell in row]
+            yield row
 
     def _identify_arch(self):
         """On multi-arch systems some libraries may be in /lib64, /usr/lib64,
@@ -268,11 +280,6 @@ class LDAPUpdate:
         if fd != sys.stdin: fd.close()
         return text
 
-    def _entry_to_entity(self, ent):
-        entry = ent.copy()
-        entry.commit()
-        return entry
-
     def _combine_updates(self, all_updates, update):
         'Combine a new update with the list of total updates'
         dn = update.get('dn')
@@ -482,7 +489,7 @@ class LDAPUpdate:
                 self.error("Task lookup failure %s", e)
                 return
 
-            status = entry.single_value('nstaskstatus', None)
+            status = entry.single_value.get('nstaskstatus')
             if status is None:
                 # task doesn't have a status yet
                 time.sleep(1)
@@ -507,7 +514,7 @@ class LDAPUpdate:
 
         if not default:
             # This means that the entire entry needs to be created with add
-            return self._entry_to_entity(entry)
+            return entry
 
         for item in default:
             # We already do syntax-parsing so this is safe
@@ -520,8 +527,9 @@ class LDAPUpdate:
             else:
                 e = [value]
             entry[attr] = e
+        entry.reset_modlist()
 
-        return self._entry_to_entity(entry)
+        return entry
 
     def _get_entry(self, dn):
         """Retrieve an object from LDAP.
@@ -550,49 +558,7 @@ class LDAPUpdate:
             # We already do syntax-parsing so this is safe
             (action, attr, update_values) = update.split(':',2)
             update_values = self._parse_values(update_values)
-
-            # If the attribute is known to be a DN convert it to a DN object.
-            # This has to be done after _parse_values() due to quoting and comma separated lists.
-            if self.conn.has_dn_syntax(attr):
-                update_values = [DN(x) for x in update_values]
-
-            entry_values = entry.get(attr)
-            if not isinstance(entry_values, list):
-                if entry_values is None:
-                    entry_values = []
-                else:
-                    entry_values = [entry_values]
-
-            # Replacing objectClassess needs a special handling and
-            # normalization of OC definitions to avoid update failures for
-            # example when X-ORIGIN is the only difference
-            schema_update = False
-            schema_elem_class = None
-            schema_elem_name = None
-            if action == "replace" and entry.dn == DN(('cn', 'schema')):
-                if attr.lower() == "objectclasses":
-                    schema_elem_class = ObjectClass
-                    schema_elem_name = "ObjectClass"
-                elif attr.lower() == "attributetypes":
-                    schema_elem_class = AttributeType
-                    schema_elem_name = "AttributeType"
-
-                if schema_elem_class is not None:
-                    schema_update = True
-                    oid_index = {}
-                    # build the OID index for replacing
-                    for schema_elem in entry_values:
-                        try:
-                            schema_elem_object = schema_elem_class(str(schema_elem))
-                        except Exception, e:
-                            self.error('replace: cannot parse %s "%s": %s',
-                                            schema_elem_name, schema_elem, e)
-                            continue
-                        # In a corner case, there may be more representations of
-                        # the same objectclass/attributetype due to the previous updates
-                        # We want to replace them all
-                        oid_index.setdefault(schema_elem_object.oid, []).append(schema_elem)
-
+            entry_values = entry.get(attr, [])
             for update_value in update_values:
                 if action == 'remove':
                     self.debug("remove: '%s' from %s, current value %s", safe_output(attr, update_value), attr, safe_output(attr,entry_values))
@@ -660,36 +626,13 @@ class LDAPUpdate:
                     except ValueError:
                         raise BadSyntax, "bad syntax in replace, needs to be in the format old::new in %s" % update_value
                     try:
-                        if schema_update:
-                            try:
-                                schema_elem_old = schema_elem_class(str(old))
-                            except Exception, e:
-                                self.error('replace: cannot parse replaced %s "%s": %s',
-                                        schema_elem_name, old, e)
-                                continue
-                            replaced_values = []
-                            for schema_elem in oid_index.get(schema_elem_old.oid, []):
-                                schema_elem_object = schema_elem_class(str(schema_elem))
-                                if str(schema_elem_old).lower() == str(schema_elem_object).lower():
-                                    # compare normalized values
-                                    replaced_values.append(schema_elem)
-                                    self.debug('replace: replace %s "%s" with "%s"',
-                                            schema_elem_name,
-                                            safe_output(attr, old),
-                                            safe_output(attr, new))
-                            if not replaced_values:
-                                self.debug('replace: no match for replaced %s "%s"',
-                                        schema_elem_name, safe_output(attr, old))
-                                continue
-                            for value in replaced_values:
-                                entry_values.remove(value)
-                        else:
-                            entry_values.remove(old)
+                        entry_values.remove(old)
+                    except ValueError:
+                        self.debug('replace: %s not found, skipping', safe_output(attr, old))
+                    else:
                         entry_values.append(new)
                         self.debug('replace: updated value %s', safe_output(attr, entry_values))
                         entry[attr] = entry_values
-                    except ValueError:
-                        self.debug('replace: %s not found, skipping', safe_output(attr, old))
 
         return entry
 
@@ -700,49 +643,9 @@ class LDAPUpdate:
             self.debug("%s", message)
         self.debug("dn: %s", e.dn)
         for a, value in e.items():
-            if isinstance(value, (list, tuple)):
-                self.debug('%s:', a)
-                for l in value:
-                    self.debug("\t%s", safe_output(a, l))
-            else:
-                self.debug('%s: %s', a, safe_output(a, value))
-
-    def is_schema_updated(self, s):
-        """Compare the schema in 's' with the current schema in the DS to
-           see if anything has changed. This should account for syntax
-           differences (like added parens that make no difference but are
-           detected as a change by generateModList()).
-
-           This doesn't handle re-ordering of attributes. They are still
-           detected as changes, so foo $ bar != bar $ foo.
-
-           return True if the schema has changed
-           return False if it has not
-        """
-        signature = inspect.getargspec(ldap.schema.SubSchema.__init__)
-        if 'check_uniqueness' in signature.args:
-            s = ldap.schema.SubSchema(s, check_uniqueness=0)
-        else:
-            s = ldap.schema.SubSchema(s)
-        s = s.ldap_entry()
-
-        # Get a fresh copy and convert into a SubSchema
-        n = self._get_entry(DN(('cn', 'schema')))[0]
-
-        # Convert IPA data types back to strings
-        d = dict()
-        for k,v in n.data.items():
-            d[k] = [str(x) for x in v]
-
-        # Convert to subschema dict
-        n = ldap.schema.SubSchema(d)
-        n = n.ldap_entry()
-
-        # Are they equal?
-        if s == n:
-            return False
-        else:
-            return True
+            self.debug('%s:', a)
+            for l in value:
+                self.debug("\t%s", safe_output(a, l))
 
     def _update_record(self, update):
         found = False
@@ -759,7 +662,7 @@ class LDAPUpdate:
             if len(e) > 1:
                 # we should only ever get back one entry
                 raise BadSyntax, "More than 1 entry returned on a dn search!? %s" % new_entry.dn
-            entry = self._entry_to_entity(e[0])
+            entry = e[0]
             found = True
             self.info("Updating existing entry: %s", entry.dn)
         except errors.NotFound:
@@ -784,10 +687,6 @@ class LDAPUpdate:
         added = False
         updated = False
         if not found:
-            # New entries get their orig_data set to the entry itself. We want to
-            # empty that so that everything appears new when generating the
-            # modlist
-            # entry.orig_data = {}
             try:
                 if self.live_run:
                     if len(entry):
@@ -809,23 +708,16 @@ class LDAPUpdate:
         else:
             # Update LDAP
             try:
-                changes = self.conn.generateModList(entry.origDataDict(), entry.toDict())
-                if (entry.dn == DN(('cn', 'schema'))):
-                    d = dict()
-                    e = entry.toDict()
-                    for k,v in e.items():
-                        d[k] = [str(x) for x in v]
-                    updated = self.is_schema_updated(d)
-                else:
-                    if len(changes) >= 1:
-                        updated = True
+                changes = entry.generate_modlist()
+                if len(changes) >= 1:
+                    updated = True
                 safe_changes = []
                 for (type, attr, values) in changes:
                     safe_changes.append((type, attr, safe_output(attr, values)))
                 self.debug("%s" % safe_changes)
                 self.debug("Live %d, updated %d" % (self.live_run, updated))
                 if self.live_run and updated:
-                    self.conn.updateEntry(entry.dn, entry.origDataDict(), entry.toDict())
+                    self.conn.update_entry(entry)
                 self.info("Done")
             except errors.EmptyModlist:
                 self.info("Entry already up-to-date")
@@ -843,7 +735,7 @@ class LDAPUpdate:
         if entry.dn.endswith(DN(('cn', 'index'), ('cn', 'userRoot'),
                                 ('cn', 'ldbm database'), ('cn', 'plugins'),
                                 ('cn', 'config'))) and (added or updated):
-            taskid = self.create_index_task(entry.single_value('cn'))
+            taskid = self.create_index_task(entry.single_value['cn'])
             self.monitor_index_task(taskid)
         return
 
@@ -886,38 +778,20 @@ class LDAPUpdate:
 
     def create_connection(self):
         if self.online:
-            if self.ldapi:
-                self.conn = ipaldap.IPAdmin(ldapi=True, realm=self.realm)
-            else:
-                self.conn = ipaldap.IPAdmin(self.sub_dict['FQDN'],
-                                            ldapi=False,
-                                            realm=self.realm)
-            try:
-                if self.dm_password:
-                    self.conn.do_simple_bind(binddn=DN(('cn', 'directory manager')), bindpw=self.dm_password)
-                elif os.getegid() == 0:
-                    try:
-                        # autobind
-                        self.conn.do_external_bind(self.pw_name)
-                    except errors.NotFound:
-                        # Fall back
-                        self.conn.do_sasl_gssapi_bind()
-                else:
-                    self.conn.do_sasl_gssapi_bind()
-            except ldap.LOCAL_ERROR, e:
-                raise RuntimeError('%s' % e.args[0].get('info', '').strip())
+            self.conn = connect(
+                ldapi=self.ldapi, realm=self.realm, fqdn=self.sub_dict['FQDN'],
+                dm_password=self.dm_password, pw_name=self.pw_name)
         else:
             raise RuntimeError("Offline updates are not supported.")
 
     def _run_updates(self, all_updates):
         # For adds and updates we want to apply updates from shortest
-        # to greatest length of the DN. cn=schema must always go first to add
-        # new objectClasses and attributeTypes
+        # to greatest length of the DN.
         # For deletes we want the reverse
         def update_sort_key(dn_update):
             dn, update = dn_update
             assert isinstance(dn, DN)
-            return dn != DN(('cn', 'schema')), len(dn)
+            return len(dn)
 
         sorted_updates = sorted(all_updates.iteritems(), key=update_sort_key)
 
@@ -929,6 +803,18 @@ class LDAPUpdate:
         for dn, update in sorted_updates:
             self._delete_record(update)
 
+    def pre_schema_update(self, ordered=False):
+        """Execute the update before the LDPA schema is updated.
+        """
+        if self.plugins:
+            self.info('PRE_SCHEMA_UPDATE')
+            all_updates = {}
+            updates = api.Backend.updateclient.update(PRE_SCHEMA_UPDATE, self.dm_password, self.ldapi, self.live_run)
+            self.merge_updates(all_updates, updates)
+            self._run_updates(all_updates)
+
+        return self.modified
+
     def update(self, files, ordered=False):
         """Execute the update. files is a list of the update files to use.
 
@@ -1003,3 +889,9 @@ class LDAPUpdate:
         self._run_updates(updates)
 
         return self.modified
+
+    def close_connection(self):
+        """Close ldap connection"""
+        if self.conn:
+            self.conn.unbind()
+            self.conn = None
