@@ -74,18 +74,18 @@ class GenerateUpdateMixin(object):
         for entry in definitions_managed_entries:
             assert isinstance(entry.dn, DN)
             if deletes:
-                old_dn = entry.data['managedtemplate'][0]
+                old_dn = entry['managedtemplate'][0]
                 assert isinstance(old_dn, DN)
                 try:
-                    (old_dn, entry) = ldap.get_entry(old_dn, ['*'])
+                    entry = ldap.get_entry(old_dn, ['*'])
                 except errors.NotFound, e:
                     pass
                 else:
                     # Compute the new dn by replacing the old container with the new container
-                    new_dn = EditableDN(old_dn)
+                    new_dn = EditableDN(entry.dn)
                     if new_dn.replace(old_template_container, new_template_container) != 1:
                         self.error("unable to replace '%s' with '%s' in '%s'",
-                                   old_template_container, new_template_container, old_dn)
+                                   old_template_container, new_template_container, entry.dn)
                         continue
 
                     new_dn = DN(new_dn)
@@ -95,21 +95,21 @@ class GenerateUpdateMixin(object):
                                   'default': entry_to_update(entry)}
 
                     # Delete the old entry
-                    old_update = {'dn': old_dn, 'deleteentry': None}
+                    old_update = {'dn': entry.dn, 'deleteentry': None}
 
                     # Add the delete and replacement updates to the list of all updates
-                    update_list.append({old_dn: old_update, new_dn: new_update})
+                    update_list.append({entry.dn: old_update, new_dn: new_update})
 
             else:
                 # Update the template dn by replacing the old containter with the new container
-                old_dn = entry.data['managedtemplate'][0]
+                old_dn = entry['managedtemplate'][0]
                 new_dn = EditableDN(old_dn)
                 if new_dn.replace(old_template_container, new_template_container) != 1:
                     self.error("unable to replace '%s' with '%s' in '%s'",
                                old_template_container, new_template_container, old_dn)
                     continue
                 new_dn = DN(new_dn)
-                entry.data['managedtemplate'] = new_dn
+                entry['managedtemplate'] = new_dn
 
                 # Edit the dn, then convert it back to an immutable DN
                 old_dn = entry.dn
@@ -122,7 +122,7 @@ class GenerateUpdateMixin(object):
 
                 # The old attributes become defaults for the new entry
                 new_update = {'dn': new_dn,
-                              'default': entry_to_update(entry.data)}
+                              'default': entry_to_update(entry)}
 
                 # Add the replacement update to the collection of all updates
                 update_list.append({new_dn: new_update})
