@@ -117,6 +117,17 @@ NONOBJECT_PERMISSIONS = {
         },
         'default_privileges': {'IPA Masters Readers'},
     },
+    'System: Compat Tree ID View targets': {
+        'replaces_global_anonymous_aci': True,
+        'ipapermlocation':  api.env.basedn,
+        'ipapermtarget': DN('cn=*,cn=compat', api.env.basedn),
+        'ipapermtargetfilter': {'(objectclass=ipaOverrideTarget)'},
+        'ipapermbindruletype': 'anonymous',
+        'ipapermright': {'read', 'search', 'compare'},
+        'ipapermdefaultattr': {
+            'ipaAnchorUUID',
+        },
+    },
     'System: Read DNA Configuration': {
         'replaces_global_anonymous_aci': True,
         'ipapermlocation': DN('cn=dna,cn=ipa,cn=etc', api.env.basedn),
@@ -139,6 +150,29 @@ NONOBJECT_PERMISSIONS = {
             'cn', 'objectclass', 'usercertificate',
         },
     },
+    'System: Add CA Certificate For Renewal': {
+        'ipapermlocation': DN('cn=ca_renewal,cn=ipa,cn=etc', api.env.basedn),
+        'ipapermtarget': DN(
+            'cn=caSigningCert cert-pki-ca,cn=ca_renewal,cn=ipa,cn=etc',
+            api.env.basedn),
+        'ipapermtargetfilter': {'(objectclass=pkiuser)'},
+        'ipapermbindruletype': 'permission',
+        'ipapermright': {'add'},
+        'default_privileges': {'Certificate Administrators'},
+    },
+    'System: Modify CA Certificate For Renewal': {
+        'ipapermlocation': DN('cn=ca_renewal,cn=ipa,cn=etc', api.env.basedn),
+        'ipapermtarget': DN(
+            'cn=caSigningCert cert-pki-ca,cn=ca_renewal,cn=ipa,cn=etc',
+            api.env.basedn),
+        'ipapermtargetfilter': {'(objectclass=pkiuser)'},
+        'ipapermbindruletype': 'permission',
+        'ipapermright': {'write'},
+        'ipapermdefaultattr': {
+            'usercertificate',
+        },
+        'default_privileges': {'Certificate Administrators'},
+    },
     'System: Read CA Certificate': {
         'replaces_global_anonymous_aci': True,
         'ipapermlocation': DN('cn=CAcert,cn=ipa,cn=etc', api.env.basedn),
@@ -150,6 +184,52 @@ NONOBJECT_PERMISSIONS = {
             'authorityrevocationlist', 'crosscertificatepair',
         },
     },
+    'System: Modify CA Certificate': {
+        'ipapermlocation': DN('cn=CAcert,cn=ipa,cn=etc', api.env.basedn),
+        'ipapermtargetfilter': {'(objectclass=pkica)'},
+        'ipapermbindruletype': 'permission',
+        'ipapermright': {'write'},
+        'ipapermdefaultattr': {
+            'cacertificate',
+        },
+        'default_privileges': {'Certificate Administrators'},
+    },
+    'System: Read Certificate Store Entries': {
+        'ipapermlocation': DN('cn=certificates,cn=ipa,cn=etc', api.env.basedn),
+        'ipapermtargetfilter': {'(objectclass=ipacertificate)'},
+        'ipapermbindruletype': 'anonymous',
+        'ipapermright': {'read', 'search', 'compare'},
+        'ipapermdefaultattr': {
+            'cn', 'objectclass', 'ipacertsubject', 'ipacertissuerserial',
+            'ipapublickey', 'ipaconfigstring', 'cacertificate', 'ipakeytrust',
+            'ipakeyusage', 'ipakeyextusage',
+        },
+    },
+    'System: Add Certificate Store Entry': {
+        'ipapermlocation': DN('cn=certificates,cn=ipa,cn=etc', api.env.basedn),
+        'ipapermtargetfilter': {'(objectclass=ipacertificate)'},
+        'ipapermbindruletype': 'permission',
+        'ipapermright': {'add'},
+        'default_privileges': {'Certificate Administrators'},
+    },
+    'System: Modify Certificate Store Entry': {
+        'ipapermlocation': DN('cn=certificates,cn=ipa,cn=etc', api.env.basedn),
+        'ipapermtargetfilter': {'(objectclass=ipacertificate)'},
+        'ipapermbindruletype': 'permission',
+        'ipapermright': {'write'},
+        'ipapermdefaultattr': {
+            'ipacertissuerserial', 'ipaconfigstring', 'cacertificate',
+            'ipakeytrust', 'ipakeyusage', 'ipakeyextusage',
+        },
+        'default_privileges': {'Certificate Administrators'},
+    },
+    'System: Remove Certificate Store Entry': {
+        'ipapermlocation': DN('cn=certificates,cn=ipa,cn=etc', api.env.basedn),
+        'ipapermtargetfilter': {'(objectclass=ipacertificate)'},
+        'ipapermbindruletype': 'permission',
+        'ipapermright': {'delete'},
+        'default_privileges': {'Certificate Administrators'},
+    },
     'System: Read Replication Information': {
         'replaces_global_anonymous_aci': True,
         'ipapermlocation': DN('cn=replication,cn=etc', api.env.basedn),
@@ -240,6 +320,26 @@ NONOBJECT_PERMISSIONS = {
             'winsyncsubtreepair',
         },
         'default_privileges': {'Replication Administrators'},
+    },
+    'System: Read DUA Profile': {
+        'ipapermlocation': DN('ou=profile', api.env.basedn),
+        'ipapermtargetfilter': {
+            '(|'
+                '(objectclass=organizationalUnit)'
+                '(objectclass=DUAConfigProfile)'
+            ')'
+        },
+        'ipapermbindruletype': 'anonymous',
+        'ipapermright': {'read', 'search', 'compare'},
+        'ipapermdefaultattr': {
+            'objectclass', 'ou', 'cn', 'defaultServerList',
+            'preferredServerList', 'defaultSearchBase', 'defaultSearchScope',
+            'searchTimeLimit', 'bindTimeLimit', 'credentialLevel',
+            'authenticationMethod', 'followReferrals', 'dereferenceAliases',
+            'serviceSearchDescriptor', 'serviceCredentialLevel',
+            'serviceAuthenticationMethod', 'objectclassMap', 'attributeMap',
+            'profileTTL'
+        },
     }
 }
 
