@@ -528,39 +528,43 @@ class ReplicationManager(object):
         print "The user for the Windows PassSync service is %s" % pass_dn
         try:
             conn.get_entry(pass_dn)
-            print "Windows PassSync entry exists, not resetting password"
-            return
+            print "Windows PassSync system account exists, not resetting password"
         except errors.NotFound:
-            pass
+            # The user doesn't exist, add it
+            print "Adding Windows PassSync system account"
+            entry = conn.make_entry(
+                pass_dn,
+                objectclass=["account", "simplesecurityobject"],
+                uid=["passsync"],
+                userPassword=[password],
+            )
+            conn.add_entry(entry)
 
-        # The user doesn't exist, add it
-        entry = conn.make_entry(
-            pass_dn,
-            objectclass=["account", "simplesecurityobject"],
-            uid=["passsync"],
-            userPassword=[password],
-        )
-        conn.add_entry(entry)
-
-        # Add it to the list of users allowed to bypass password policy
+        # Add the user to the list of users allowed to bypass password policy
         extop_dn = DN(('cn', 'ipa_pwd_extop'), ('cn', 'plugins'), ('cn', 'config'))
         entry = conn.get_entry(extop_dn)
-        pass_mgrs = entry.get('passSyncManagersDNs')
-        if not pass_mgrs:
-            pass_mgrs = []
-        if not isinstance(pass_mgrs, list):
-            pass_mgrs = [pass_mgrs]
+        pass_mgrs = entry.get('passSyncManagersDNs', [])
         pass_mgrs.append(pass_dn)
         mod = [(ldap.MOD_REPLACE, 'passSyncManagersDNs', pass_mgrs)]
-        conn.modify_s(extop_dn, mod)
-
-        # And finally grant it permission to write passwords
-        mod = [(ldap.MOD_ADD, 'aci',
-            ['(targetattr = "userPassword || krbPrincipalKey || sambaLMPassword || sambaNTPassword || passwordHistory")(version 3.0; acl "Windows PassSync service can write passwords"; allow (write) userdn="ldap:///%s";)' % pass_dn])]
         try:
-            conn.modify_s(self.suffix, mod)
+            conn.modify_s(extop_dn, mod)
         except ldap.TYPE_OR_VALUE_EXISTS:
-            root_logger.debug("passsync aci already exists in suffix %s on %s" % (self.suffix, conn.host))
+            root_logger.debug("Plugin '%s' already '%s' in passSyncManagersDNs",
+                    extop_dn, pass_dn)
+
+        # And finally add it is a member of PassSync privilege to allow
+        # displaying user NT attributes and reset passwords
+        passsync_privilege_dn = DN(('cn','PassSync Service'),
+                api.env.container_privilege,
+                api.env.basedn)
+        members = entry.get('member', [])
+        members.append(pass_dn)
+        mod = [(ldap.MOD_REPLACE, 'member', members)]
+        try:
+            conn.modify_s(passsync_privilege_dn, mod)
+        except ldap.TYPE_OR_VALUE_EXISTS:
+            root_logger.debug("PassSync service '%s' already have '%s' as member",
+                    passsync_privilege_dn, pass_dn)
 
     def setup_winsync_agmt(self, entry, win_subtree=None):
         if win_subtree is None:
@@ -1201,6 +1205,117 @@ class ReplicationManager(object):
             elif not err:
                 err = e
 
+        try:
+            entry = self.conn.get_entry(
+                DN(('cn', 'ipa'), ('cn', 'etc'), self.suffix), ['aci'])
+
+            sub = {'suffix': self.suffix, 'fqdn': replica}
+            try:
+                entry.raw['aci'].remove(
+                    '(target = "ldap:///cn=*,cn=ca_renewal,cn=ipa,cn=etc,'
+                    '%(suffix)s")(version 3.0; acl "Add CA Certificates for '
+                    'renewals"; allow(add) userdn = "ldap:///fqdn=%(fqdn)s,'
+                    'cn=computers,cn=accounts,%(suffix)s";)' % sub)
+            except ValueError:
+                pass
+            try:
+                entry.raw['aci'].remove(
+                    '(target = "ldap:///cn=*,cn=ca_renewal,cn=ipa,cn=etc,'
+                    '%(suffix)s")(targetattr = "userCertificate")'
+                    '(version 3.0; acl "Modify CA Certificates for renewals"; '
+                    'allow(write) userdn = "ldap:///fqdn=%(fqdn)s,'
+                    'cn=computers,cn=accounts,%(suffix)s";)' % sub)
+            except ValueError:
+                pass
+            try:
+                entry.raw['aci'].remove(
+                    '(target = "ldap:///cn=CAcert,cn=ipa,cn=etc,%(suffix)s")'
+                    '(targetattr = cACertificate)(version 3.0; acl "Modify CA '
+                    'Certificate"; allow (write) userdn = "ldap:///fqdn='
+                    '%(fqdn)s,cn=computers,cn=accounts,%(suffix)s";)' % sub)
+            except ValueError:
+                pass
+
+            try:
+                self.conn.update_entry(entry)
+            except errors.EmptyModlist:
+                pass
+        except errors.NotFound:
+            pass
+        except Exception, e:
+            if not force:
+                raise e
+            elif not err:
+                err = e
+
+        try:
+            entry = self.conn.get_entry(
+                DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'),
+                   self.suffix),
+                ['aci'])
+
+            sub = {'suffix': self.suffix, 'fqdn': replica}
+            try:
+                entry.raw['aci'].remove(
+                    '(targetfilter = "(objectClass=nsContainer)")'
+                    '(targetattr = "cn || objectClass || ipaConfigString")'
+                    '(version 3.0; acl "Read IPA Masters"; allow (read, '
+                    'search, compare) userdn = "ldap:///fqdn=%(fqdn)s,'
+                    'cn=computers,cn=accounts,%(suffix)s";)' % sub)
+            except ValueError:
+                pass
+            try:
+                entry.raw['aci'].remove(
+                    '(targetfilter = "(objectClass=nsContainer)")'
+                    '(targetattr = "ipaConfigString")(version 3.0; acl '
+                    '"Modify IPA Masters"; allow (write) userdn = '
+                    '"ldap:///fqdn=%(fqdn)s,cn=computers,cn=accounts,'
+                    '%(suffix)s";)' % sub)
+            except ValueError:
+                pass
+
+            try:
+                self.conn.update_entry(entry)
+            except errors.EmptyModlist:
+                pass
+        except errors.NotFound:
+            pass
+        except Exception, e:
+            if not force:
+                raise e
+            elif not err:
+                err = e
+
+        try:
+            entry = self.conn.get_entry(
+                DN(('cn', 'certificates'), ('cn', 'ipa'), ('cn', 'etc'),
+                   self.suffix),
+                ['aci'])
+
+            sub = {'suffix': self.suffix, 'fqdn': replica}
+            try:
+                entry.raw['aci'].remove(
+                    '(targetfilter = "(&(objectClass=ipaCertificate)'
+                    '(ipaConfigString=ipaCA))")(targetattr = '
+                    '"ipaCertIssuerSerial || cACertificate")(version 3.0; acl '
+                    '"Modify CA Certificate Store Entry"; allow (write) '
+                    'userdn = "ldap:///fqdn=%(fqdn)s,cn=computers,cn=accounts,'
+                    '%(suffix)s";)' % sub)
+            except ValueError:
+                pass
+
+            try:
+                self.conn.update_entry(entry)
+            except errors.EmptyModlist:
+                pass
+        except errors.NotFound:
+            pass
+        except Exception, e:
+            if not force:
+                raise e
+            elif not err:
+                err = e
+
         try:
             basedn = DN(('cn', 'etc'), self.suffix)
             filter = '(dnaHostname=%s)' % replica
