@@ -20,21 +20,17 @@
 import sys
 import os, socket
 import tempfile
-import pwd
 import time
 import datetime
+import traceback
 
 from ipapython import sysrestore, ipautil, dogtag, ipaldap
 from ipapython.dn import DN
 from ipapython.ipa_log_manager import *
-from ipalib import errors
+from ipalib import errors, certstore
 from ipaplatform import services
 from ipaplatform.paths import paths
 
-# Autobind modes
-AUTO = 1
-ENABLED = 2
-DISABLED = 3
 
 # The service name as stored in cn=masters,cn=ipa,cn=etc. In the tuple
 # the first value is the *nix service name, the second the start order.
@@ -48,6 +44,9 @@ SERVICE_LIST = {
     'ADTRUST': ('smb', 60),
     'EXTID': ('winbind', 70),
     'OTPD': ('ipa-otpd', 80),
+    'DNSKeyExporter': ('ipa-ods-exporter', 90),
+    'DNSSEC': ('ods-enforcerd', 100),
+    'DNSKeySync': ('ipa-dnskeysyncd', 110),
 }
 
 def print_msg(message, output_fd=sys.stdout):
@@ -73,7 +72,9 @@ def format_seconds(seconds):
 
 
 class Service(object):
-    def __init__(self, service_name, service_desc=None, sstore=None, dm_password=None, ldapi=True, autobind=AUTO):
+    def __init__(self, service_name, service_desc=None, sstore=None,
+                 dm_password=None, ldapi=True, autobind=ipaldap.AUTOBIND_AUTO,
+                 start_tls=False):
         self.service_name = service_name
         self.service_desc = service_desc
         self.service = services.service(service_name)
@@ -82,6 +83,7 @@ class Service(object):
         self.dm_password = dm_password
         self.ldapi = ldapi
         self.autobind = autobind
+        self.start_tls = start_tls
 
         self.fqdn = socket.gethostname()
         self.admin_conn = None
@@ -107,28 +109,14 @@ class Service(object):
                 if not self.realm:
                     raise errors.NotFound(reason="realm is missing for %s" % (self))
                 conn = ipaldap.IPAdmin(ldapi=self.ldapi, realm=self.realm)
+            elif self.start_tls:
+                conn = ipaldap.IPAdmin(self.fqdn, port=389, protocol='ldap',
+                                       cacert=paths.IPA_CA_CRT,
+                                       start_tls=self.start_tls)
             else:
                 conn = ipaldap.IPAdmin(self.fqdn, port=389)
-            if self.dm_password:
-                conn.do_simple_bind(bindpw=self.dm_password)
-            elif self.autobind in [AUTO, ENABLED]:
-                if os.getegid() == 0 and self.ldapi:
-                    try:
-                        # autobind
-                        pw_name = pwd.getpwuid(os.geteuid()).pw_name
-                        conn.do_external_bind(pw_name)
-                    except errors.NotFound, e:
-                        if self.autobind == AUTO:
-                            # Fall back
-                            conn.do_sasl_gssapi_bind()
-                        else:
-                            # autobind was required and failed, raise
-                            # exception that it failed
-                            raise e
-                else:
-                    conn.do_sasl_gssapi_bind()
-            else:
-                conn.do_sasl_gssapi_bind()
+
+            conn.do_bind(self.dm_password, autobind=self.autobind)
         except Exception, e:
             root_logger.debug("Could not connect to the Directory Server on %s: %s" % (self.fqdn, str(e)))
             raise
@@ -269,6 +257,21 @@ class Service(object):
         except Exception, e:
             root_logger.critical("Could not add certificate to service %s entry: %s" % (self.principal, str(e)))
 
+    def import_ca_certs(self, db, ca_is_configured, conn=None):
+        if conn is None:
+            if not self.admin_conn:
+                self.ldap_connect()
+            conn = self.admin_conn
+
+        try:
+            ca_certs = certstore.get_ca_certs_nss(
+                conn, self.suffix, self.realm, ca_is_configured)
+        except errors.NotFound:
+            pass
+        else:
+            for cert, nickname, trust_flags in ca_certs:
+                db.add_cert(cert, nickname, trust_flags)
+
     def is_configured(self):
         return self.sstore.has_state(self.service_name)
 
@@ -302,6 +305,15 @@ class Service(object):
     def is_enabled(self):
         return self.service.is_enabled()
 
+    def mask(self):
+        return self.service.mask()
+
+    def unmask(self):
+        return self.service.unmask()
+
+    def is_masked(self):
+        return self.service.is_masked()
+
     def backup_state(self, key, value):
         self.sstore.backup_state(self.service_name, key, value)
 
@@ -314,8 +326,8 @@ class Service(object):
     def print_msg(self, message):
         print_msg(message, self.output_fd)
 
-    def step(self, message, method):
-        self.steps.append((message, method))
+    def step(self, message, method, run_after_failure=False):
+        self.steps.append((message, method, run_after_failure))
 
     def start_creation(self, start_message=None, end_message=None,
         show_service_name=True, runtime=-1):
@@ -360,15 +372,33 @@ class Service(object):
         else:
             self.print_msg(start_message)
 
-        step = 0
-        for (message, method) in self.steps:
-            self.print_msg("  [%d/%d]: %s" % (step+1, len(self.steps), message))
+        def run_step(message, method):
+            self.print_msg(message)
             s = datetime.datetime.now()
             method()
             e = datetime.datetime.now()
             d = e - s
             root_logger.debug("  duration: %d seconds" % d.seconds)
-            step += 1
+
+        step = 0
+        steps_iter = iter(self.steps)
+        try:
+            for message, method, run_after_failure in steps_iter:
+                full_msg = "  [%d/%d]: %s" % (step+1, len(self.steps), message)
+                run_step(full_msg, method)
+                step += 1
+        except BaseException as e:
+            if not (isinstance(e, SystemExit) and e.code == 0):
+                # show the traceback, so it's not lost if cleanup method fails
+                root_logger.debug("%s" % traceback.format_exc())
+                self.print_msg('  [error] %s: %s' % (type(e).__name__, e))
+
+                # run through remaining methods marked run_after_failure
+                for message, method, run_after_failure in steps_iter:
+                    if run_after_failure:
+                        run_step("  [cleanup]: %s" % message, method)
+
+            raise
 
         self.print_msg(end_message)
 
@@ -381,6 +411,32 @@ class Service(object):
             self.ldap_connect()
 
         entry_name = DN(('cn', name), ('cn', fqdn), ('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'), ldap_suffix)
+
+        # enable disabled service
+        try:
+            entry = self.admin_conn.get_entry(entry_name, ['ipaConfigString'])
+        except errors.NotFound:
+            pass
+        else:
+            if any(u'enabledservice' == val.lower()
+                   for val in entry.get('ipaConfigString', [])):
+                root_logger.debug("service %s startup entry already enabled", name)
+                return
+
+            entry.setdefault('ipaConfigString', []).append(u'enabledService')
+
+            try:
+                self.admin_conn.update_entry(entry)
+            except errors.EmptyModlist:
+                root_logger.debug("service %s startup entry already enabled", name)
+                return
+            except:
+                root_logger.debug("failed to enable service %s startup entry", name)
+                raise
+
+            root_logger.debug("service %s startup entry enabled", name)
+            return
+
         order = SERVICE_LIST[name][1]
         entry = self.admin_conn.make_entry(
             entry_name,
@@ -393,9 +449,48 @@ class Service(object):
         try:
             self.admin_conn.add_entry(entry)
         except (errors.DuplicateEntry), e:
-            root_logger.debug("failed to add %s Service startup entry" % name)
+            root_logger.debug("failed to add service %s startup entry", name)
             raise e
 
+    def ldap_disable(self, name, fqdn, ldap_suffix):
+        assert isinstance(ldap_suffix, DN)
+        if not self.admin_conn:
+            self.ldap_connect()
+
+        entry_dn = DN(('cn', name), ('cn', fqdn), ('cn', 'masters'),
+                        ('cn', 'ipa'), ('cn', 'etc'), ldap_suffix)
+        search_kw = {'ipaConfigString': u'enabledService'}
+        filter = self.admin_conn.make_filter(search_kw)
+        try:
+            entries, truncated = self.admin_conn.find_entries(
+                filter=filter,
+                attrs_list=['ipaConfigString'],
+                base_dn=entry_dn,
+                scope=self.admin_conn.SCOPE_BASE)
+        except errors.NotFound:
+            root_logger.debug("service %s startup entry already disabled", name)
+            return
+
+        assert len(entries) == 1  # only one entry is expected
+        entry = entries[0]
+
+        # case insensitive
+        for value in entry.get('ipaConfigString', []):
+            if value.lower() == u'enabledservice':
+                entry['ipaConfigString'].remove(value)
+                break
+
+        try:
+            self.admin_conn.update_entry(entry)
+        except errors.EmptyModlist:
+            pass
+        except:
+            root_logger.debug("failed to disable service %s startup entry", name)
+            raise
+
+        root_logger.debug("service %s startup entry disabled", name)
+
+
 class SimpleServiceInstance(Service):
     def create_instance(self, gensvc_name=None, fqdn=None, dm_password=None, ldap_suffix=None, realm=None):
         self.gensvc_name = gensvc_name
