@@ -25,28 +25,38 @@ Also see the `ipalib.rpc` module.
 
 from xml.sax.saxutils import escape
 from xmlrpclib import Fault
-from wsgiref.util import shift_path_info
-import base64
 import os
-import string
 import datetime
-from decimal import Decimal
 import urlparse
-import time
 import json
+import traceback
 
-from ipalib import plugable, capabilities
+import ldap.controls
+from pyasn1.type import univ, namedtype
+from pyasn1.codec.ber import encoder
+
+from ipalib import plugable, errors
+from ipalib.capabilities import VERSION_WITHOUT_CAPABILITIES
 from ipalib.backend import Executioner
-from ipalib.errors import PublicError, InternalError, CommandError, JSONError, ConversionError, CCacheError, RefererError, InvalidSessionPassword, NotFound, ACIError, ExecutionError
-from ipalib.request import context, Connection, destroy_context
-from ipalib.rpc import xml_dumps, xml_loads
+from ipalib.errors import (PublicError, InternalError, CommandError, JSONError,
+    CCacheError, RefererError, InvalidSessionPassword, NotFound, ACIError,
+    ExecutionError, PasswordExpired)
+from ipalib.request import context, destroy_context
+from ipalib.rpc import (xml_dumps, xml_loads,
+    json_encode_binary, json_decode_binary)
 from ipalib.util import parse_time_duration, normalize_name
 from ipapython.dn import DN
 from ipaserver.plugins.ldap2 import ldap2
-from ipalib.session import session_mgr, AuthManager, get_ipa_ccache_name, load_ccache_data, bind_ipa_ccache, release_ipa_ccache, fmt_time, default_max_session_duration
+from ipalib.session import (
+    session_mgr, AuthManager, get_ipa_ccache_name,
+    load_ccache_data, bind_ipa_ccache, release_ipa_ccache, fmt_time,
+    default_max_session_duration, krbccache_dir, krbccache_prefix)
 from ipalib.backend import Backend
-from ipalib.krb_utils import krb5_parse_ccache, KRB5_CCache, krb_ticket_expiration_threshold, krb5_format_principal_name
+from ipalib.krb_utils import (
+    KRB5_CCache, krb_ticket_expiration_threshold, krb5_format_principal_name,
+    krb5_format_service_principal_name)
 from ipapython import ipautil
+from ipaplatform.paths import paths
 from ipapython.version import VERSION
 from ipalib.text import _
 
@@ -101,7 +111,7 @@ _unauthorized_template = """<html>
 </body>
 </html>"""
 
-_pwchange_template = """<html>
+_success_template = """<html>
 <head>
 <title>200 Success</title>
 </head>
@@ -288,6 +298,8 @@ class WSGIExecutioner(Executioner):
     content_type = None
     key = ''
 
+    _system_commands = {}
+
     def set_api(self, api):
         super(WSGIExecutioner, self).set_api(api)
         if 'wsgi_dispatch' in self.api.Backend:
@@ -306,6 +318,7 @@ class WSGIExecutioner(Executioner):
         args = ()
         options = {}
 
+        e = None
         if not 'HTTP_REFERER' in environ:
             return self.marshal(result, RefererError(referer='missing'), _id)
         if not environ['HTTP_REFERER'].startswith('https://%s/ipa' % self.api.env.host) and not self.env.in_tree:
@@ -328,10 +341,15 @@ class WSGIExecutioner(Executioner):
                 (name, args, options, _id) = self.unmarshal(data)
             else:
                 (name, args, options, _id) = self.simple_unmarshal(environ)
-            if name not in self.Command:
+            if name in self._system_commands:
+                result = self._system_commands[name](self, *args, **options)
+            elif name not in self.Command:
                 raise CommandError(name=name)
-            result = self.Command[name](*args, **options)
+            else:
+                result = self.Command[name](*args, **options)
         except PublicError, e:
+            if self.api.env.debug:
+                self.debug('WSGI wsgi_execute PublicError: %s', traceback.format_exc())
             error = e
         except StandardError, e:
             self.exception(
@@ -340,6 +358,8 @@ class WSGIExecutioner(Executioner):
             error = InternalError()
         finally:
             os.environ['LANG'] = lang
+
+        principal = getattr(context, 'principal', 'UNKNOWN')
         if name and name in self.Command:
             try:
                 params = self.Command[name].args_options_2_params(*args, **options)
@@ -349,14 +369,25 @@ class WSGIExecutioner(Executioner):
                 )
                 # get at least some context of what is going on
                 params = options
-            principal = getattr(context, 'principal', 'UNKNOWN')
             if error:
-                self.info('%s: %s(%s): %s', principal, name, ', '.join(self.Command[name]._repr_iter(**params)), e.__class__.__name__)
+                result_string = type(e).__name__
             else:
-                self.info('%s: %s(%s): SUCCESS', principal, name, ', '.join(self.Command[name]._repr_iter(**params)))
+                result_string = 'SUCCESS'
+            self.info('[%s] %s: %s(%s): %s',
+                      type(self).__name__,
+                      principal,
+                      name,
+                      ', '.join(self.Command[name]._repr_iter(**params)),
+                      result_string)
         else:
-            self.info('%s: %s', context.principal, e.__class__.__name__)
-        return self.marshal(result, error, _id)
+            self.info('[%s] %s: %s: %s',
+                      type(self).__name__,
+                      principal,
+                      name,
+                      type(e).__name__)
+
+        version = options.get('version', VERSION_WITHOUT_CAPABILITIES)
+        return self.marshal(result, error, _id, version)
 
     def simple_unmarshal(self, environ):
         name = environ['PATH_INFO'].strip('/')
@@ -393,103 +424,11 @@ class WSGIExecutioner(Executioner):
     def unmarshal(self, data):
         raise NotImplementedError('%s.unmarshal()' % self.fullname)
 
-    def marshal(self, result, error, _id=None):
+    def marshal(self, result, error, _id=None,
+                version=VERSION_WITHOUT_CAPABILITIES):
         raise NotImplementedError('%s.marshal()' % self.fullname)
 
 
-def json_encode_binary(val):
-    '''
-   JSON cannot encode binary values. We encode binary values in Python str
-   objects and text in Python unicode objects. In order to allow a binary
-   object to be passed through JSON we base64 encode it thus converting it to
-   text which JSON can transport. To assure we recognize the value is a base64
-   encoded representation of the original binary value and not confuse it with
-   other text we convert the binary value to a dict in this form:
-
-   {'__base64__' : base64_encoding_of_binary_value}
-
-   This modification of the original input value cannot be done "in place" as
-   one might first assume (e.g. replacing any binary items in a container
-   (e.g. list, tuple, dict) with the base64 dict because the container might be
-   an immutable object (i.e. a tuple). Therefore this function returns a copy
-   of any container objects it encounters with tuples replaced by lists. This
-   is O.K. because the JSON encoding will map both lists and tuples to JSON
-   arrays.
-   '''
-
-    if isinstance(val, dict):
-        new_dict = {}
-        for k,v in val.items():
-            new_dict[k] = json_encode_binary(v)
-        return new_dict
-    elif isinstance(val, (list, tuple)):
-        new_list = [json_encode_binary(v) for v in val]
-        return new_list
-    elif isinstance(val, str):
-        return {'__base64__' : base64.b64encode(val)}
-    elif isinstance(val, Decimal):
-        return {'__base64__' : base64.b64encode(str(val))}
-    elif isinstance(val, DN):
-        return str(val)
-    else:
-        return val
-
-def json_decode_binary(val):
-    '''
-    JSON cannot transport binary data. In order to transport binary data we
-    convert binary data to a form like this:
-
-   {'__base64__' : base64_encoding_of_binary_value}
-
-   see json_encode_binary()
-
-    After JSON had decoded the JSON stream back into a Python object we must
-    recursively scan the object looking for any dicts which might represent
-    binary values and replace the dict containing the base64 encoding of the
-    binary value with the decoded binary value. Unlike the encoding problem
-    where the input might consist of immutable object, all JSON decoded
-    container are mutable so the conversion could be done in place. However we
-    don't modify objects in place because of side effects which may be
-    dangerous. Thus we elect to spend a few more cycles and avoid the
-    possibility of unintended side effects in favor of robustness.
-    '''
-
-    if isinstance(val, dict):
-        if val.has_key('__base64__'):
-            return base64.b64decode(val['__base64__'])
-        else:
-            new_dict = {}
-            for k,v in val.items():
-                if isinstance(v, dict) and v.has_key('__base64__'):
-                        new_dict[k] = base64.b64decode(v['__base64__'])
-                else:
-                    new_dict[k] = json_decode_binary(v)
-            return new_dict
-    elif isinstance(val, list):
-        new_list = []
-        n = len(val)
-        i = 0
-        while i < n:
-            v = val[i]
-            if isinstance(v, dict) and v.has_key('__base64__'):
-                binary_val = base64.b64decode(v['__base64__'])
-                new_list.append(binary_val)
-            else:
-                new_list.append(json_decode_binary(v))
-            i += 1
-        return new_list
-    else:
-        if isinstance(val, basestring):
-            try:
-                return val.decode('utf-8')
-            except UnicodeDecodeError:
-                raise ConversionError(
-                    name=val,
-                    error='incorrect type'
-                )
-        else:
-            return val
-
 class jsonserver(WSGIExecutioner, HTTP_Status):
     """
     JSON RPC server.
@@ -510,13 +449,14 @@ class jsonserver(WSGIExecutioner, HTTP_Status):
         response = super(jsonserver, self).__call__(environ, start_response)
         return response
 
-    def marshal(self, result, error, _id=None):
+    def marshal(self, result, error, _id=None,
+                version=VERSION_WITHOUT_CAPABILITIES):
         if error:
             assert isinstance(error, PublicError)
             error = dict(
                 code=error.errno,
                 message=error.strerror,
-                name=error.__class__.__name__,
+                name=unicode(error.__class__.__name__),
             )
         principal = getattr(context, 'principal', 'UNKNOWN')
         response = dict(
@@ -526,7 +466,7 @@ class jsonserver(WSGIExecutioner, HTTP_Status):
             principal=unicode(principal),
             version=unicode(VERSION),
         )
-        response = json_encode_binary(response)
+        response = json_encode_binary(response, version)
         return json.dumps(response, sort_keys=True, indent=4)
 
     def unmarshal(self, data):
@@ -674,42 +614,39 @@ class KerberosSession(object):
         return ['']
 
 
-class xmlserver(WSGIExecutioner, HTTP_Status, KerberosSession):
+class KerberosWSGIExecutioner(WSGIExecutioner, HTTP_Status, KerberosSession):
+    """Base class for xmlserver and jsonserver_kerb
     """
-    Execution backend plugin for XML-RPC server.
-
-    Also see the `ipalib.rpc.xmlclient` plugin.
-    """
-
-    content_type = 'text/xml'
-    key = '/xml'
 
     def _on_finalize(self):
-        self.__system = {
-            'system.listMethods': self.listMethods,
-            'system.methodSignature': self.methodSignature,
-            'system.methodHelp': self.methodHelp,
-        }
-        super(xmlserver, self)._on_finalize()
+        super(KerberosWSGIExecutioner, self)._on_finalize()
         self.kerb_session_on_finalize()
 
     def __call__(self, environ, start_response):
-        '''
-        '''
-
-        self.debug('WSGI xmlserver.__call__:')
+        self.debug('KerberosWSGIExecutioner.__call__:')
         user_ccache=environ.get('KRB5CCNAME')
-        headers = [('Content-Type', 'text/xml; charset=utf-8')]
+
+        headers = [('Content-Type', '%s; charset=utf-8' % self.content_type)]
+
         if user_ccache is None:
-            self.internal_error(environ, start_response,
-                                'xmlserver.__call__: KRB5CCNAME not defined in HTTP request environment')
+
+            status = HTTP_STATUS_SERVER_ERROR
+            response_headers = [('Content-Type', 'text/html; charset=utf-8')]
+
+            self.log.error(
+                '%s: %s', status,
+                'KerberosWSGIExecutioner.__call__: '
+                'KRB5CCNAME not defined in HTTP request environment')
+
             return self.marshal(None, CCacheError())
         try:
             self.create_context(ccache=user_ccache)
-            response = super(xmlserver, self).__call__(environ, start_response)
-            if getattr(context, 'session_data', None) is None and \
-              self.env.context != 'lite':
-                self.finalize_kerberos_acquisition('xmlserver', user_ccache, environ, start_response, headers)
+            response = super(KerberosWSGIExecutioner, self).__call__(
+                environ, start_response)
+            session_data = getattr(context, 'session_data', None)
+            if (session_data is None and self.env.context != 'lite'):
+                self.finalize_kerberos_acquisition(
+                    'xmlserver', user_ccache, environ, start_response, headers)
         except PublicError, e:
             status = HTTP_STATUS_SUCCESS
             response = status
@@ -719,26 +656,78 @@ class xmlserver(WSGIExecutioner, HTTP_Status, KerberosSession):
             destroy_context()
         return response
 
+
+class xmlserver(KerberosWSGIExecutioner):
+    """
+    Execution backend plugin for XML-RPC server.
+
+    Also see the `ipalib.rpc.xmlclient` plugin.
+    """
+
+    content_type = 'text/xml'
+    key = '/xml'
+
     def listMethods(self, *params):
-        return tuple(name.decode('UTF-8') for name in self.Command)
+        """list methods for XML-RPC introspection"""
+        if params:
+            raise errors.ZeroArgumentError(name='system.listMethods')
+        return (tuple(unicode(name) for name in self.Command) +
+                tuple(unicode(name) for name in self._system_commands))
+
+    def _get_method_name(self, name, *params):
+        """Get a method name for XML-RPC introspection commands"""
+        if not params:
+            raise errors.RequirementError(name='method name')
+        elif len(params) > 1:
+            raise errors.MaxArgumentError(name=name, count=1)
+        [method_name] = params
+        return method_name
 
     def methodSignature(self, *params):
-        return u'methodSignature not implemented'
+        """get method signature for XML-RPC introspection"""
+        method_name = self._get_method_name('system.methodSignature', *params)
+        if method_name in self._system_commands:
+            # TODO
+            # for now let's not go out of our way to document standard XML-RPC
+            return u'undef'
+        elif method_name in self.Command:
+            # All IPA commands return a dict (struct),
+            # and take a params, options - list and dict (array, struct)
+            return [[u'struct', u'array', u'struct']]
+        else:
+            raise errors.CommandError(name=method_name)
 
     def methodHelp(self, *params):
-        return u'methodHelp not implemented'
+        """get method docstring for XML-RPC introspection"""
+        method_name = self._get_method_name('system.methodHelp', *params)
+        if method_name in self._system_commands:
+            return u''
+        elif method_name in self.Command:
+            return unicode(self.Command[method_name].__doc__ or '')
+        else:
+            raise errors.CommandError(name=method_name)
+
+    _system_commands = {
+        'system.listMethods': listMethods,
+        'system.methodSignature': methodSignature,
+        'system.methodHelp': methodHelp,
+    }
 
     def unmarshal(self, data):
         (params, name) = xml_loads(data)
+        if name in self._system_commands:
+            # For XML-RPC introspection, return params directly
+            return (name, params, {}, None)
         (args, options) = params_2_args_options(params)
         if 'version' not in options:
             # Keep backwards compatibility with client containing
             # bug https://fedorahosted.org/freeipa/ticket/3294:
             # If `version` is not given in XML-RPC, assume an old version
-            options['version'] = capabilities.VERSION_WITHOUT_CAPABILITIES
+            options['version'] = VERSION_WITHOUT_CAPABILITIES
         return (name, args, options, None)
 
-    def marshal(self, result, error, _id=None):
+    def marshal(self, result, error, _id=None,
+                version=VERSION_WITHOUT_CAPABILITIES):
         if error:
             self.debug('response: %s: %s', error.__class__.__name__, str(error))
             response = Fault(error.errno, error.strerror)
@@ -746,7 +735,7 @@ class xmlserver(WSGIExecutioner, HTTP_Status, KerberosSession):
             if isinstance(result, dict):
                 self.debug('response: entries returned %d', result.get('count', 1))
             response = (result,)
-        return xml_dumps(response, methodresponse=True)
+        return xml_dumps(response, version, methodresponse=True)
 
 
 class jsonserver_session(jsonserver, KerberosSession):
@@ -846,33 +835,14 @@ class jsonserver_session(jsonserver, KerberosSession):
 
         return response
 
-class jsonserver_kerb(jsonserver):
+
+class jsonserver_kerb(jsonserver, KerberosWSGIExecutioner):
     """
     JSON RPC server protected with kerberos auth.
     """
 
     key = '/json'
 
-    def __call__(self, environ, start_response):
-        '''
-        '''
-
-        self.debug('WSGI jsonserver_kerb.__call__:')
-
-        user_ccache=environ.get('KRB5CCNAME')
-        if user_ccache is None:
-            self.internal_error(environ, start_response,
-                                'jsonserver_kerb.__call__: KRB5CCNAME not defined in HTTP request environment')
-            return self.marshal(None, CCacheError())
-        self.create_context(ccache=user_ccache)
-
-        try:
-            response = super(jsonserver_kerb, self).__call__(environ, start_response)
-        finally:
-            destroy_context()
-
-        return response
-
 
 class login_kerberos(Backend, KerberosSession, HTTP_Status):
     key = '/session/login_kerberos'
@@ -977,56 +947,53 @@ class login_password(Backend, KerberosSession, HTTP_Status):
 
         # Get the ccache we'll use and attempt to get credentials in it with user,password
         ipa_ccache_name = get_ipa_ccache_name()
-        reason = 'invalid-password'
         try:
             self.kinit(user, self.api.env.realm, password, ipa_ccache_name)
-        except InvalidSessionPassword, e:
-            # Ok, now why is this bad. Is the password simply bad or is the
-            # password expired?
-            try:
-                dn = DN(('uid', user),
-                        self.api.env.container_user,
-                        self.api.env.basedn)
-                conn = ldap2(shared_instance=False,
-                             ldap_uri=self.api.env.ldap_uri)
-                conn.connect(bind_dn=dn, bind_pw=password)
-
-                # password is ok, must be expired, lets double-check
-                (userdn, entry_attrs) = conn.get_entry(dn,
-                    ['krbpasswordexpiration'])
-                if 'krbpasswordexpiration' in entry_attrs:
-                    expiration = entry_attrs['krbpasswordexpiration'][0]
-                    try:
-                        exp = time.strptime(expiration, '%Y%m%d%H%M%SZ')
-                        if exp <= time.gmtime():
-                            reason = 'password-expired'
-                    except ValueError, v:
-                        self.error('Unable to convert %s to a time string'
-                            % expiration)
-
-            except Exception:
-                # It doesn't really matter how we got here but the user's
-                # password is not accepted or the user is unknown.
-                pass
-            finally:
-                if conn.isconnected():
-                    conn.destroy_connection()
-
-            return self.unauthorized(environ, start_response, str(e), reason)
+        except PasswordExpired as e:
+            return self.unauthorized(environ, start_response, str(e), 'password-expired')
+        except InvalidSessionPassword as e:
+            return self.unauthorized(environ, start_response, str(e), 'invalid-password')
 
         return self.finalize_kerberos_acquisition('login_password', ipa_ccache_name, environ, start_response)
 
     def kinit(self, user, realm, password, ccache_name):
+        # get http service ccache as an armor for FAST to enable OTP authentication
+        armor_principal = krb5_format_service_principal_name(
+            'HTTP', self.api.env.host, realm)
+        keytab = paths.IPA_KEYTAB
+        armor_name = "%sA_%s" % (krbccache_prefix, user)
+        armor_path = os.path.join(krbccache_dir, armor_name)
+
+        self.debug('Obtaining armor ccache: principal=%s keytab=%s ccache=%s',
+                   armor_principal, keytab, armor_path)
+
+        (stdout, stderr, returncode) = ipautil.run(
+            [paths.KINIT, '-kt', keytab, armor_principal],
+            env={'KRB5CCNAME': armor_path}, raiseonerr=False)
+
+        if returncode != 0:
+            raise CCacheError()
+
         # Format the user as a kerberos principal
         principal = krb5_format_principal_name(user, realm)
 
-        (stdout, stderr, returncode) = ipautil.run(['/usr/bin/kinit', principal],
-                                                   env={'KRB5CCNAME':ccache_name},
-                                                   stdin=password, raiseonerr=False)
+        (stdout, stderr, returncode) = ipautil.run(
+            [paths.KINIT, principal, '-T', armor_path],
+            env={'KRB5CCNAME': ccache_name, 'LC_ALL': 'C'},
+            stdin=password, raiseonerr=False)
+
         self.debug('kinit: principal=%s returncode=%s, stderr="%s"',
                    principal, returncode, stderr)
 
+        self.debug('Cleanup the armor ccache')
+        ipautil.run(
+            [paths.KDESTROY, '-A', '-c', armor_path],
+            env={'KRB5CCNAME': armor_path},
+            raiseonerr=False)
+
         if returncode != 0:
+            if stderr.strip() == 'kinit: Cannot read password while getting initial credentials':
+                raise PasswordExpired(principal=principal, message=unicode(stderr))
             raise InvalidSessionPassword(principal=principal, message=unicode(stderr))
 
 class change_password(Backend, HTTP_Status):
@@ -1061,7 +1028,7 @@ class change_password(Backend, HTTP_Status):
             return self.bad_request(environ, start_response, "cannot parse query data")
 
         data = {}
-        for field in ('user', 'old_password', 'new_password'):
+        for field in ('user', 'old_password', 'new_password', 'otp'):
             value = query_dict.get(field, None)
             if value is not None:
                 if len(value) == 1:
@@ -1069,7 +1036,7 @@ class change_password(Backend, HTTP_Status):
                 else:
                     return self.bad_request(environ, start_response, "more than one %s parameter"
                                             % field)
-            else:
+            elif field != 'otp':  # otp is optional
                 return self.bad_request(environ, start_response, "no %s specified" % field)
 
         # start building the response
@@ -1084,9 +1051,12 @@ class change_password(Backend, HTTP_Status):
                      self.api.env.container_user, self.api.env.basedn)
 
         try:
+            pw = data['old_password']
+            if data.get('otp'):
+                pw = data['old_password'] + data['otp']
             conn = ldap2(shared_instance=False,
                          ldap_uri=self.api.env.ldap_uri)
-            conn.connect(bind_dn=bind_dn, bind_pw=data['old_password'])
+            conn.connect(bind_dn=bind_dn, bind_pw=pw)
         except (NotFound, ACIError):
             result = 'invalid-password'
             message = 'The old password or username is not correct.'
@@ -1096,7 +1066,7 @@ class change_password(Backend, HTTP_Status):
                     data['user'], str(e))
         else:
             try:
-                conn.modify_password(bind_dn, data['new_password'], data['old_password'])
+                conn.modify_password(bind_dn, data['new_password'], data['old_password'], skip_bind=True)
             except ExecutionError, e:
                 result = 'policy-error'
                 policy_error = escape(str(e))
@@ -1120,10 +1090,110 @@ class change_password(Backend, HTTP_Status):
             response_headers.append(('X-IPA-Pwchange-Policy-Error', policy_error))
 
         start_response(status, response_headers)
-        output = _pwchange_template % dict(title=str(title),
-                                           message=str(message))
+        output = _success_template % dict(title=str(title),
+                                          message=str(message))
         return [output]
 
+class sync_token(Backend, HTTP_Status):
+    content_type = 'text/plain'
+    key = '/session/sync_token'
+
+    class OTPSyncRequest(univ.Sequence):
+        OID = "2.16.840.1.113730.3.8.10.6"
+
+        componentType = namedtype.NamedTypes(
+            namedtype.NamedType('firstCode', univ.OctetString()),
+            namedtype.NamedType('secondCode', univ.OctetString()),
+            namedtype.OptionalNamedType('tokenDN', univ.OctetString())
+        )
+
+    def __init__(self):
+        super(sync_token, self).__init__()
+
+    def _on_finalize(self):
+        super(sync_token, self)._on_finalize()
+        self.api.Backend.wsgi_dispatch.mount(self, self.key)
+
+    def __call__(self, environ, start_response):
+        # Make sure this is a form request.
+        content_type = environ.get('CONTENT_TYPE', '').lower()
+        if not content_type.startswith('application/x-www-form-urlencoded'):
+            return self.bad_request(environ, start_response, "Content-Type must be application/x-www-form-urlencoded")
+
+        # Make sure this is a POST request.
+        method = environ.get('REQUEST_METHOD', '').upper()
+        if method == 'POST':
+            query_string = read_input(environ)
+        else:
+            return self.bad_request(environ, start_response, "HTTP request method must be POST")
+
+        # Parse the query string to a dictionary.
+        try:
+            query_dict = urlparse.parse_qs(query_string)
+        except Exception, e:
+            return self.bad_request(environ, start_response, "cannot parse query data")
+        data = {}
+        for field in ('user', 'password', 'first_code', 'second_code', 'token'):
+            value = query_dict.get(field, None)
+            if value is not None:
+                if len(value) == 1:
+                    data[field] = value[0]
+                else:
+                    return self.bad_request(environ, start_response, "more than one %s parameter"
+                                            % field)
+            elif field != 'token':
+                return self.bad_request(environ, start_response, "no %s specified" % field)
+
+        # Create the request control.
+        sr = self.OTPSyncRequest()
+        sr.setComponentByName('firstCode', data['first_code'])
+        sr.setComponentByName('secondCode', data['second_code'])
+        if 'token' in data:
+            try:
+                token_dn = DN(data['token'])
+            except ValueError:
+                token_dn = DN((self.api.Object.otptoken.primary_key.name, data['token']),
+                              self.api.env.container_otp, self.api.env.basedn)
+
+            sr.setComponentByName('tokenDN', str(token_dn))
+        rc = ldap.controls.RequestControl(sr.OID, True, encoder.encode(sr))
+
+        # Resolve the user DN
+        bind_dn = DN((self.api.Object.user.primary_key.name, data['user']),
+                     self.api.env.container_user, self.api.env.basedn)
+
+        # Start building the response.
+        status = HTTP_STATUS_SUCCESS
+        response_headers = [('Content-Type', 'text/html; charset=utf-8')]
+        title = 'Token sync rejected'
+
+        # Perform the synchronization.
+        conn = ldap2(shared_instance=False, ldap_uri=self.api.env.ldap_uri)
+        try:
+            conn.connect(bind_dn=bind_dn,
+                         bind_pw=data['password'],
+                         serverctrls=[rc,])
+            result = 'ok'
+            title = "Token sync successful"
+            message = "Token was synchronized."
+        except (NotFound, ACIError):
+            result = 'invalid-credentials'
+            message = 'The username, password or token codes are not correct.'
+        except Exception, e:
+            result = 'error'
+            message = "Could not connect to LDAP server."
+            self.error("token_sync: cannot authenticate '%s' to LDAP server: %s",
+                       data['user'], str(e))
+        finally:
+            if conn.isconnected():
+                conn.destroy_connection()
+
+        # Report status and return.
+        response_headers.append(('X-IPA-TokenSync-Result', result))
+        start_response(status, response_headers)
+        output = _success_template % dict(title=str(title),
+                                          message=str(message))
+        return [output]
 
 class xmlserver_session(xmlserver, KerberosSession):
     """
