@@ -30,6 +30,7 @@ import StringIO
 from ldif import LDIFWriter
 
 from ipapython import ipautil
+from ipaplatform.paths import paths
 from ipapython.dn import DN
 from ipapython.ipa_log_manager import log_mgr
 from ipatests.test_integration import util
@@ -42,6 +43,10 @@ log = log_mgr.get_logger(__name__)
 def prepare_host(host):
     if isinstance(host, Host):
         env_filename = os.path.join(host.config.test_dir, 'env.sh')
+
+        # First we try to run simple echo command to test the connection
+        host.run_command(['true'], set_env=False)
+
         host.collect_log(env_filename)
         host.transport.mkdir_recursive(host.config.test_dir)
         host.put_file_contents(env_filename, env_to_script(host.to_env()))
@@ -71,8 +76,8 @@ def backup_file(host, filename):
 
 
 def fix_etc_hosts(host):
-    backup_file(host, '/etc/hosts')
-    contents = host.get_file_contents('/etc/hosts')
+    backup_file(host, paths.HOSTS)
+    contents = host.get_file_contents(paths.HOSTS)
     # Remove existing mentions of the host's FQDN, short name, and IP
     contents = re.sub('\s%s(\s|$)' % re.escape(host.hostname), ' ', contents,
                       flags=re.MULTILINE)
@@ -83,12 +88,12 @@ def fix_etc_hosts(host):
     # Add the host's info again
     contents += '\n%s %s %s\n' % (host.ip, host.hostname, host.shortname)
     log.debug('Writing the following to /etc/hosts:\n%s', contents)
-    host.put_file_contents('/etc/hosts', contents)
+    host.put_file_contents(paths.HOSTS, contents)
 
 
 def fix_hostname(host):
-    backup_file(host, '/etc/hostname')
-    host.put_file_contents('/etc/hostname', host.hostname + '\n')
+    backup_file(host, paths.ETC_HOSTNAME)
+    host.put_file_contents(paths.ETC_HOSTNAME, host.hostname + '\n')
     host.run_command(['hostname', host.hostname])
 
     backupname = os.path.join(host.config.test_dir, 'backup_hostname')
@@ -96,15 +101,27 @@ def fix_hostname(host):
 
 
 def fix_resolv_conf(host):
-    backup_file(host, '/etc/resolv.conf')
-    lines = host.get_file_contents('/etc/resolv.conf').splitlines()
+    backup_file(host, paths.RESOLV_CONF)
+    lines = host.get_file_contents(paths.RESOLV_CONF).splitlines()
     lines = ['#' + l if l.startswith('nameserver') else l for l in lines]
     for other_host in host.domain.hosts:
         if other_host.role in ('master', 'replica'):
             lines.append('nameserver %s' % other_host.ip)
     contents = '\n'.join(lines)
     log.debug('Writing the following to /etc/resolv.conf:\n%s', contents)
-    host.put_file_contents('/etc/resolv.conf', contents)
+    host.put_file_contents(paths.RESOLV_CONF, contents)
+
+
+def fix_apache_semaphores(master):
+    systemd_available = master.transport.file_exists(paths.SYSTEMCTL)
+
+    if systemd_available:
+        master.run_command(['systemctl', 'stop', 'httpd'], raiseonerr=False)
+    else:
+        master.run_command([paths.SBIN_SERVICE, 'httpd', 'stop'], raiseonerr=False)
+
+    master.run_command('for line in `ipcs -s | grep apache | cut -d " " -f 2`; '
+                       'do ipcrm -s $line; done', raiseonerr=False)
 
 
 def unapply_fixes(host):
@@ -119,17 +136,22 @@ def restore_files(host):
     backupname = os.path.join(host.config.test_dir, 'file_backup')
     rmname = os.path.join(host.config.test_dir, 'file_remove')
 
-    # Restore the backed up files
-    host.run_command('cp -arvf %s/* /' % ipautil.shell_quote(backupname),
-                     raiseonerr=False)
-
-    # Restore context of the backed-up files
+    # Prepare command for restoring context of the backed-up files
     sed_remove_backupdir = 's/%s//g' % backupname.replace('/', '\/')
-    host.run_command("find %s | "
-                     "sed '%s' | "
-                     "sed '/^$/d' | "
-                     "xargs -d '\n' "
-                     "/sbin/restorecon -v" % (backupname, sed_remove_backupdir))
+    restorecon_command = (
+        "find %s | "
+        "sed '%s' | "
+        "sed '/^$/d' | "
+        "xargs -d '\n' "
+        "/sbin/restorecon -v" % (backupname, sed_remove_backupdir))
+
+    # Prepare command for actual restoring of the backed up files
+    copyfiles_command = 'if [ -d %(dir)s/ ]; then cp -arvf %(dir)s/* /; fi' % {
+        'dir': ipautil.shell_quote(backupname)}
+
+    # Run both commands in one session. For more information, see:
+    # https://fedorahosted.org/freeipa/ticket/4133
+    host.run_command('%s ; (%s ||:)' % (copyfiles_command, restorecon_command))
 
     # Remove all the files that did not exist and were 'backed up'
     host.run_command(['xargs', '-d', r'\n', '-a', rmname, 'rm', '-vf'],
@@ -163,13 +185,14 @@ def enable_replication_debugging(host):
 
 
 def install_master(host):
-    host.collect_log('/var/log/ipaserver-install.log')
-    host.collect_log('/var/log/ipaclient-install.log')
+    host.collect_log(paths.IPASERVER_INSTALL_LOG)
+    host.collect_log(paths.IPACLIENT_INSTALL_LOG)
     inst = host.domain.realm.replace('.', '-')
-    host.collect_log('/var/log/dirsrv/slapd-%s/errors' % inst)
-    host.collect_log('/var/log/dirsrv/slapd-%s/access' % inst)
+    host.collect_log(paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
+    host.collect_log(paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
 
     apply_common_fixes(host)
+    fix_apache_semaphores(host)
 
     host.run_command(['ipa-server-install', '-U',
                       '-r', host.domain.name,
@@ -179,22 +202,24 @@ def install_master(host):
                       '--forwarder', host.config.dns_forwarder])
 
     enable_replication_debugging(host)
+    setup_sssd_debugging(host)
 
     kinit_admin(host)
 
 
 def install_replica(master, replica, setup_ca=True):
-    replica.collect_log('/var/log/ipareplica-install.log')
-    replica.collect_log('/var/log/ipareplica-conncheck.log')
+    replica.collect_log(paths.IPAREPLICA_INSTALL_LOG)
+    replica.collect_log(paths.IPAREPLICA_CONNCHECK_LOG)
 
     apply_common_fixes(replica)
+    fix_apache_semaphores(replica)
 
     master.run_command(['ipa-replica-prepare',
                         '-p', replica.config.dirman_password,
                         '--ip-address', replica.ip,
                         replica.hostname])
     replica_bundle = master.get_file_contents(
-        '/var/lib/ipa/replica-info-%s.gpg' % replica.hostname)
+        paths.REPLICA_INFO_GPG_TEMPLATE % replica.hostname)
     replica_filename = os.path.join(replica.config.test_dir,
                                     'replica-info.gpg')
     replica.put_file_contents(replica_filename, replica_bundle)
@@ -209,12 +234,13 @@ def install_replica(master, replica, setup_ca=True):
     replica.run_command(args)
 
     enable_replication_debugging(replica)
+    setup_sssd_debugging(replica)
 
     kinit_admin(replica)
 
 
-def install_client(master, client):
-    client.collect_log('/var/log/ipaclient-install.log')
+def install_client(master, client, extra_args=()):
+    client.collect_log(paths.IPACLIENT_INSTALL_LOG)
 
     apply_common_fixes(client)
 
@@ -223,8 +249,10 @@ def install_client(master, client):
                         '--realm', client.domain.realm,
                         '-p', client.config.admin_name,
                         '-w', client.config.admin_password,
-                        '--server', master.hostname])
+                        '--server', master.hostname]
+                        + list(extra_args))
 
+    setup_sssd_debugging(client)
     kinit_admin(client)
 
 
@@ -235,11 +263,11 @@ def install_adtrust(host):
     """
 
     # ipa-adtrust-install appends to ipaserver-install.log
-    host.collect_log('/var/log/ipaserver-install.log')
+    host.collect_log(paths.IPASERVER_INSTALL_LOG)
 
     inst = host.domain.realm.replace('.', '-')
-    host.collect_log('/var/log/dirsrv/slapd-%s/errors' % inst)
-    host.collect_log('/var/log/dirsrv/slapd-%s/access' % inst)
+    host.collect_log(paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
+    host.collect_log(paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
 
     kinit_admin(host)
     host.run_command(['ipa-adtrust-install', '-U',
@@ -250,7 +278,15 @@ def install_adtrust(host):
 
     # Restart named because it lost connection to dirsrv
     # (Directory server restarts during the ipa-adtrust-install)
-    host.run_command(['systemctl', 'restart', 'named'])
+    # we use two services named and named-pkcs11,
+    # if named is masked restart named-pkcs11
+    result = host.run_command(['systemctl', 'is-enabled', 'named'],
+                              raiseonerr=False)
+    if result.stdout_text.startswith("masked"):
+        host.run_command(['systemctl', 'restart', 'named-pkcs11'])
+    else:
+        host.run_command(['systemctl', 'restart', 'named'])
+
 
     # Check that named is running and has loaded the information from LDAP
     dig_command = ['dig', 'SRV', '+short', '@localhost',
@@ -287,15 +323,7 @@ def configure_dns_for_trust(master, ad):
 
     kinit_admin(master)
 
-    if is_subdomain(master.domain.name, ad.domain.name):
-        master.run_command(['ipa', 'dnszone-add', ad.domain.name,
-                            '--name-server', ad.hostname,
-                            '--admin-email', 'hostmaster@%s' % ad.domain.name,
-                            '--forwarder', ad.ip,
-                            '--forward-policy', 'only',
-                            '--ip-address', ad.ip,
-                            '--force'])
-    elif is_subdomain(ad.domain.name, master.domain.name):
+    if is_subdomain(ad.domain.name, master.domain.name):
         master.run_command(['ipa', 'dnsrecord-add', master.domain.name,
                             '%s.%s' % (ad.shortname, ad.netbios),
                             '--a-ip-address', ad.ip])
@@ -308,13 +336,10 @@ def configure_dns_for_trust(master, ad):
         master.run_command(['ipa', 'dnszone-mod', master.domain.name,
                             '--allow-transfer', ad.ip])
     else:
-        master.run_command(['ipa', 'dnszone-add', ad.domain.name,
-                            '--name-server', ad.hostname,
-                            '--admin-email', 'hostmaster@%s' % ad.domain.name,
+        master.run_command(['ipa', 'dnsforwardzone-add', ad.domain.name,
                             '--forwarder', ad.ip,
                             '--forward-policy', 'only',
-                            '--ip-address', ad.ip,
-                            '--force'])
+                            ])
 
 
 def establish_trust_with_ad(master, ad, extra_args=()):
@@ -327,7 +352,7 @@ def establish_trust_with_ad(master, ad, extra_args=()):
     """
 
     # Force KDC to reload MS-PAC info by trying to get TGT for HTTP
-    master.run_command(['kinit', '-kt', '/etc/httpd/conf/ipa.keytab',
+    master.run_command(['kinit', '-kt', paths.IPA_KEYTAB,
                         'HTTP/%s' % master.hostname])
     master.run_command(['systemctl', 'restart', 'krb5kdc.service'])
     master.run_command(['kdestroy', '-A'])
@@ -370,7 +395,7 @@ def configure_auth_to_local_rule(master, ad):
              % (ad.domain.realm, ad.domain.realm, ad.domain.name))
     line2 = "  auth_to_local = DEFAULT"
 
-    krb5_conf_content = master.get_file_contents('/etc/krb5.conf')
+    krb5_conf_content = master.get_file_contents(paths.KRB5_CONF)
     krb5_lines = [line.rstrip() for line in krb5_conf_content.split('\n')]
     realm_section_index = krb5_lines.index(section_identifier)
 
@@ -378,32 +403,60 @@ def configure_auth_to_local_rule(master, ad):
     krb5_lines.insert(realm_section_index + 2, line2)
 
     krb5_conf_new_content = '\n'.join(krb5_lines)
-    master.put_file_contents('/etc/krb5.conf', krb5_conf_new_content)
+    master.put_file_contents(paths.KRB5_CONF, krb5_conf_new_content)
 
     master.run_command(['systemctl', 'restart', 'sssd'])
 
 
+def setup_sssd_debugging(host):
+    """
+    Sets debug level to 7 in each section of sssd.conf file.
+    """
+
+    # Set debug level in each section of sssd.conf file to 7
+    # First, remove any previous occurences
+    host.run_command(['sed', '-i',
+                      '/debug_level = 7/d',
+                      paths.SSSD_CONF
+                     ], raiseonerr=False)
+
+    # Add the debug directive to each section
+    host.run_command(['sed', '-i',
+                      '/\[*\]/ a\debug_level = 7',
+                      paths.SSSD_CONF
+                     ], raiseonerr=False)
+
+
+    host.collect_log('/var/log/sssd/*')
+
+    # Clear the cache and restart SSSD
+    clear_sssd_cache(host)
+
+
 def clear_sssd_cache(host):
     """
     Clears SSSD cache by removing the cache files. Restarts SSSD.
     """
 
-    systemd_available = host.transport.file_exists('/bin/systemctl')
+    systemd_available = host.transport.file_exists(paths.SYSTEMCTL)
 
     if systemd_available:
-        host.run_command(['systemctl', 'start', 'sssd'])
+        host.run_command(['systemctl', 'stop', 'sssd'])
     else:
-        host.run_command(['/sbin/service', 'sssd', 'start'])
+        host.run_command([paths.SBIN_SERVICE, 'sssd', 'stop'])
 
     host.run_command("find /var/lib/sss/db -name '*.ldb' | "
                      "xargs rm -fv")
-    host.run_command(['rm', '-fv', '/var/lib/sss/mc/group'])
-    host.run_command(['rm', '-fv', '/var/lib/sss/mc/passwd'])
+    host.run_command(['rm', '-fv', paths.SSSD_MC_GROUP])
+    host.run_command(['rm', '-fv', paths.SSSD_MC_PASSWD])
 
     if systemd_available:
         host.run_command(['systemctl', 'start', 'sssd'])
     else:
-        host.run_command(['/sbin/service', 'sssd', 'start'])
+        host.run_command([paths.SBIN_SERVICE, 'sssd', 'start'])
+
+    # To avoid false negatives due to SSSD not responding yet
+    time.sleep(10)
 
 
 def sync_time(host, server):
@@ -432,15 +485,24 @@ def kinit_admin(host):
 
 
 def uninstall_master(host):
-    host.collect_log('/var/log/ipaserver-uninstall.log')
+    host.collect_log(paths.IPASERVER_UNINSTALL_LOG)
 
     host.run_command(['ipa-server-install', '--uninstall', '-U'],
                      raiseonerr=False)
+    host.run_command(['pkidestroy', '-s', 'CA', '-i', 'pki-tomcat'],
+                     raiseonerr=False)
+    host.run_command(['rm', '-rf',
+                      paths.TOMCAT_TOPLEVEL_DIR,
+                      paths.SYSCONFIG_PKI_TOMCAT,
+                      paths.SYSCONFIG_PKI_TOMCAT_PKI_TOMCAT_DIR,
+                      paths.VAR_LIB_PKI_TOMCAT_DIR,
+                      paths.PKI_TOMCAT],
+                      raiseonerr=False)
     unapply_fixes(host)
 
 
 def uninstall_client(host):
-    host.collect_log('/var/log/ipaclient-uninstall.log')
+    host.collect_log(paths.IPACLIENT_UNINSTALL_LOG)
 
     host.run_command(['ipa-client-install', '--uninstall', '-U'],
                      raiseonerr=False)
@@ -572,6 +634,9 @@ def install_topo(topo, master, replicas, clients,
     installed = {master}
     if not skip_master:
         install_master(master)
+
+    add_a_records_for_hosts_in_master_domain(master)
+
     for parent, child in get_topo(topo)(master, replicas):
         if child in installed:
             log.info('Connecting replica %s to %s' % (parent, child))
@@ -596,7 +661,7 @@ def _entries_to_ldif(entries):
     io = StringIO.StringIO()
     writer = LDIFWriter(io)
     for entry in entries:
-        writer.unparse(str(entry.dn), entry)
+        writer.unparse(str(entry.dn), dict(entry))
     return io.getvalue()
 
 
@@ -620,11 +685,11 @@ def wait_for_replication(ldap, timeout=30):
             filter='(objectclass=nsds5replicationagreement)',
             attrs_list=[status_attr, progress_attr])
         log.debug('Replication agreements: \n%s', _entries_to_ldif(entries))
-        if any(not e.single_value(status_attr).startswith('0 ')
+        if any(not e.single_value[status_attr].startswith('0 ')
                for e in entries):
             log.error('Replication error')
             continue
-        if any(e.single_value(progress_attr) == 'TRUE' for e in entries):
+        if any(e.single_value[progress_attr] == 'TRUE' for e in entries):
             log.debug('Replication in progress (waited %s/%ss)',
                       i, timeout)
         else:
@@ -632,3 +697,28 @@ def wait_for_replication(ldap, timeout=30):
             break
     else:
         log.error('Giving up wait for replication to finish')
+
+
+def add_a_records_for_hosts_in_master_domain(master):
+    for host in master.domain.hosts:
+        # We don't need to take care of the zone creation since it is master
+        # domain
+        add_a_record(master, host)
+
+
+def add_a_record(master, host):
+    # Find out if the record is already there
+    cmd = master.run_command(['ipa',
+                              'dnsrecord-find',
+                              master.domain.name,
+                              host.hostname,
+                              '--a-rec', host.ip],
+                             raiseonerr=False)
+
+    # If not, add it
+    if cmd.returncode != 0:
+        master.run_command(['ipa',
+                            'dnsrecord-add',
+                            master.domain.name,
+                            host.hostname,
+                            '--a-rec', host.ip])
