@@ -181,3 +181,23 @@ pwpolicy = [
     u'nscontainer',
     u'top',
 ]
+
+idview = [
+    u'ipaIDView',
+    u'nsContainer',
+    u'top'
+]
+
+idoverrideuser = [
+    u'ipaOverrideAnchor',
+    u'top',
+    u'ipaUserOverride',
+    u'ipasshuser',
+    u'ipaSshGroupOfPubKeys'
+]
+
+idoverridegroup = [
+    u'ipaOverrideAnchor',
+    u'top',
+    u'ipaGroupOverride',
+]
