@@ -26,6 +26,7 @@ from ipatests.test_xmlrpc import objectclasses
 from xmlrpc_test import (Declarative, fuzzy_digits, fuzzy_uuid, fuzzy_set_ci,
                          add_sid, add_oc)
 from ipapython.dn import DN
+from ipatests.test_xmlrpc.test_user_plugin import get_user_result
 
 group1 = u'testgroup1'
 group2 = u'testgroup2'
@@ -553,8 +554,8 @@ class test_group(Declarative):
             desc='Remove group %r with external membership' % (group3),
             command=('group_del', [group3], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=group3,
+                result=dict(failed=[]),
+                value=[group3],
                 summary=u'Deleted group "testgroup3"',
             ),
         ),
@@ -692,8 +693,8 @@ class test_group(Declarative):
             desc='Delete %r' % group1,
             command=('group_del', [group1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=group1,
+                result=dict(failed=[]),
+                value=[group1],
                 summary=u'Deleted group "testgroup1"',
             )
         ),
@@ -727,8 +728,8 @@ class test_group(Declarative):
             desc='Delete %r' % group2,
             command=('group_del', [group2], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=group2,
+                result=dict(failed=[]),
+                value=[group2],
                 summary=u'Deleted group "testgroup2"',
             )
         ),
@@ -796,29 +797,7 @@ class test_group(Declarative):
             expected=dict(
                 value=user1,
                 summary=u'Added user "%s"' % user1,
-                result=add_sid(dict(
-                    gecos=[u'Test User1'],
-                    givenname=[u'Test'],
-                    homedirectory=[u'/home/%s' % user1],
-                    krbprincipalname=[u'%s@%s' % (user1, api.env.realm)],
-                    loginshell=[u'/bin/sh'],
-                    objectclass=add_oc(objectclasses.user, u'ipantuserattrs'),
-                    sn=[u'User1'],
-                    uid=[user1],
-                    uidnumber=[fuzzy_digits],
-                    gidnumber=[fuzzy_digits],
-                    mail=[u'%s@%s' % (user1, api.env.domain)],
-                    displayname=[u'Test User1'],
-                    cn=[u'Test User1'],
-                    initials=[u'TU'],
-                    ipauniqueid=[fuzzy_uuid],
-                    mepmanagedentry=[get_group_dn(user1)],
-                    memberof_group=[u'ipausers'],
-                    dn=DN(('uid',user1),('cn','users'),('cn','accounts'),
-                          api.env.basedn),
-                    has_keytab=False,
-                    has_password=False,
-                )),
+                result=get_user_result(user1, u'Test', u'User1', 'add'),
             ),
         ),
 
@@ -880,8 +859,8 @@ class test_group(Declarative):
             desc='Now delete the unmanaged group %r' % user1,
             command=('group_del', [user1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=user1,
+                result=dict(failed=[]),
+                value=[user1],
                 summary=u'Deleted group "%s"' % user1,
             )
         ),
@@ -896,9 +875,9 @@ class test_group(Declarative):
             desc='Delete %r' % user1,
             command=('user_del', [user1], {}),
             expected=dict(
-                result=dict(failed=u''),
+                result=dict(failed=[]),
                 summary=u'Deleted user "tuser1"',
-                value=user1,
+                value=[user1],
             ),
         ),
 
@@ -910,30 +889,14 @@ class test_group(Declarative):
             expected=dict(
                 value=user1,
                 summary=u'Added user "tuser1"',
-                result=add_sid(dict(
-                    gecos=[u'Test User1'],
-                    givenname=[u'Test'],
+                result=get_user_result(
+                    user1, u'Test', u'User1', 'add',
                     description=[],
-                    homedirectory=[u'/home/tuser1'],
-                    krbprincipalname=[u'tuser1@' + api.env.realm],
-                    loginshell=[u'/bin/sh'],
                     objectclass=add_oc(objectclasses.user_base,
                                        u'ipantuserattrs'),
-                    sn=[u'User1'],
-                    uid=[user1],
-                    uidnumber=[fuzzy_digits],
                     gidnumber=[u'1000'],
-                    mail=[u'%s@%s' % (user1, api.env.domain)],
-                    displayname=[u'Test User1'],
-                    cn=[u'Test User1'],
-                    initials=[u'TU'],
-                    ipauniqueid=[fuzzy_uuid],
-                    dn=DN(('uid','tuser1'),('cn','users'),('cn','accounts'),
-                          api.env.basedn),
-                    memberof_group=[u'ipausers'],
-                    has_keytab=False,
-                    has_password=False,
-                )),
+                    omit=['mepmanagedentry'],
+                ),
             ),
         ),
 
@@ -1041,10 +1004,77 @@ class test_group(Declarative):
             desc='Delete %r' % user1,
             command=('user_del', [user1], {}),
             expected=dict(
-                result=dict(failed=u''),
+                result=dict(failed=[]),
                 summary=u'Deleted user "%s"' % user1,
-                value=user1,
+                value=[user1],
+            ),
+        ),
+    ]
+
+class test_group_remove_group_from_protected_group(Declarative):
+    cleanup_commands = [
+        ('group_del', [group1], {}),
+    ]
+    tests = [
+        # Test scenario from ticket #4448
+        # https://fedorahosted.org/freeipa/ticket/4448
+        dict(
+            desc='Add group %s' % group1,
+            command=('group_add', [group1], dict(description=u'Test desc 1')),
+            expected=dict(
+                value=group1,
+                summary=u'Added group "%s"' % group1,
+                result=dict(
+                    cn=[group1],
+                    description=[u'Test desc 1'],
+                    objectclass=objectclasses.posixgroup,
+                    gidnumber=[fuzzy_digits],
+                    ipauniqueid=[fuzzy_uuid],
+                    dn=get_group_dn(group1),
+                ),
             ),
         ),
 
+        dict(
+            desc='Add %s group to admins group' % group1,
+            command=('group_add_member', [u'admins'], dict(group=group1)),
+            expected=dict(
+                completed=1,
+                failed=dict(
+                    member=dict(
+                        group=tuple(),
+                        user=tuple(),
+                    ),
+                ),
+                result=dict(
+                        dn=get_group_dn('admins'),
+                        member_user=[u'admin'],
+                        member_group=[group1],
+                        gidnumber=[fuzzy_digits],
+                        cn=[u'admins'],
+                        description=[u'Account administrators group'],
+                ),
+            ),
+        ),
+
+        dict(
+            desc='Remove %s group from admins group' % group1,
+            command=('group_remove_member', [u'admins'], dict(group=group1)),
+            expected=dict(
+                completed=1,
+                failed=dict(
+                    member=dict(
+                        group=tuple(),
+                        user=tuple(),
+                    ),
+                ),
+                result=dict(
+                    dn=get_group_dn(u'admins'),
+                    cn=[u'admins'],
+                    gidnumber=[fuzzy_digits],
+                    member_user=[u'admin'],
+                    description=[u'Account administrators group'],
+                ),
+            ),
+        ),
     ]
