@@ -1,6 +1,7 @@
 #ifndef __IPA_KRB5_H_
 #define __IPA_KRB5_H_
 
+#include <lber.h>
 #include <krb5/krb5.h>
 #include <kdb.h>
 
@@ -27,6 +28,9 @@ struct keys_container {
 
 #define KEYTAB_SET_OID "2.16.840.1.113730.3.8.10.1"
 #define KEYTAB_RET_OID "2.16.840.1.113730.3.8.10.2"
+#define KEYTAB_GET_OID "2.16.840.1.113730.3.8.10.5"
+
+int krb5_klog_syslog(int, const char *, ...);
 
 void
 ipa_krb5_free_ktypes(krb5_context context, krb5_enctype *val);
@@ -68,6 +72,9 @@ void free_keys_contents(krb5_context krbctx, struct keys_container *keys);
 struct berval *create_key_control(struct keys_container *keys,
                                   const char *principalName);
 
+int ipa_string_to_enctypes(const char *str, struct krb_key_salt **encsalts,
+                           int *num_encsalts, char **err_msg);
+
 int create_keys(krb5_context krbctx,
                 krb5_principal princ,
                 char *password,
